@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -15,21 +15,23 @@ import org.eclipse.core.runtime.IPath;
 /** 
  * An entry on a Java project classpath identifying one or more package fragment
  * roots. A classpath entry has a content kind (either source, 
- * <code>K_SOURCE</code>, or binary, <code>K_BINARY</code>), which is inherited
+ * {@link IPackageFragmentRoot#K_SOURCE}, or binary, {@link IPackageFragmentRoot#K_BINARY}), which is inherited
  * by each package fragment root and package fragment associated with the entry.
  * <p>
  * A classpath entry can refer to any of the following:<ul>
  * 
  *	<li>Source code in the current project. In this case, the entry identifies a
  *		root folder in the current project containing package fragments and
- *		<code>.java</code> source files. The root folder itself represents a default
- *		package, subfolders represent package fragments, and <code>.java</code> files
+ *		source files with one of the {@link JavaCore#getJavaLikeExtensions() 
+ *		Java-like extensions}. The root folder itself represents a default
+ *		package, subfolders represent package fragments, and files with a
+ *     Java-like extension (e.g. <code>.java</code> files)
  *		represent compilation units. All compilation units will be compiled when
  * 		the project is built. The classpath entry must specify the
  *		absolute path to the root folder. Entries of this kind are 
- *		associated with the <code>CPE_SOURCE</code> constant.
+ *		associated with the {@link #CPE_SOURCE} constant.
  *      Source classpath entries can carry inclusion and exclusion patterns for
- *      selecting which <code>.java</code> source files appear as compilation
+ *      selecting which source files appear as compilation
  *      units and get compiled when the project is built.
  *  </li>
  * 
@@ -38,7 +40,7 @@ import org.eclipse.core.runtime.IPath;
  *		package fragments and <code>.class</code> files.  The classpath entry
  *		must specify the absolute path to the JAR (or root folder), and in case it refers
  *		to an external JAR, then there is no associated resource in the workbench. Entries 
- *		of this kind are associated with the <code>CPE_LIBRARY</code> constant.</li>
+ *		of this kind are associated with the {@link #CPE_LIBRARY} constant.</li>
  * 
  *	<li>A required project. In this case the entry identifies another project in
  *		the workspace. The required project is used as a binary library when compiling
@@ -49,50 +51,50 @@ import org.eclipse.core.runtime.IPath;
  *		is performed against a required project's source code, and compilation is 
  *		performed against a required project's last built state.  The
  *		classpath entry must specify the absolute path to the
- *		project. Entries of this kind are  associated with the <code>CPE_PROJECT</code>
+ *		project. Entries of this kind are  associated with the {@link #CPE_PROJECT}
  *		constant. 
  * 		Note: referencing a required project with a classpath entry refers to the source 
  *     code or associated <code>.class</code> files located in its output location. 
  *     It will also automatically include any other libraries or projects that the required project's classpath 
  *     refers to, iff the corresponding classpath entries are tagged as being exported 
- *     (<code>IClasspathEntry#isExported</code>). 
+ *     ({@link IClasspathEntry#isExported}). 
  *    Unless exporting some classpath entries, classpaths are not chained by default - 
  *    each project must specify its own classpath in its entirety.</li>
  * 
  *  <li> A path beginning in a classpath variable defined globally to the workspace.
- *		Entries of this kind are  associated with the <code>CPE_VARIABLE</code> constant.  
- *      Classpath variables are created using <code>JavaCore#setClasspathVariable</code>,
+ *		Entries of this kind are  associated with the {@link #CPE_VARIABLE} constant.  
+ *      Classpath variables are created using {@link JavaCore#setClasspathVariable(String, IPath, org.eclipse.core.runtime.IProgressMonitor)},
  * 		and gets resolved, to either a project or library entry, using
- *      <code>JavaCore#getResolvedClasspathVariable</code>.
- *		It is also possible to register an automatic initializer (<code>ClasspathVariableInitializer</code>),
+ *      {@link JavaCore#getResolvedClasspathEntry(IClasspathEntry)}.
+ *		It is also possible to register an automatic initializer ({@link ClasspathVariableInitializer}),
  * 	which will be invoked through the extension point "org.aspectj.org.eclipse.jdt.core.classpathVariableInitializer".
  * 	After resolution, a classpath variable entry may either correspond to a project or a library entry. </li>
  * 
  *  <li> A named classpath container identified by its container path.
  *     A classpath container provides a way to indirectly reference a set of classpath entries through
- *     a classpath entry of kind <code>CPE_CONTAINER</code>. Typically, a classpath container can
+ *     a classpath entry of kind {@link #CPE_CONTAINER}. Typically, a classpath container can
  *     be used to describe a complex library composed of multiple JARs, projects or classpath variables,
  *     considering also that containers can be mapped differently on each project. Several projects can
  *     reference the same generic container path, but have each of them actually bound to a different
  *     container object.
  *     The container path is a formed by a first ID segment followed with extra segments, 
  *     which can be used as additional hints for resolving this container reference. If no container was ever 
- *     recorded for this container path onto this project (using <code>setClasspathContainer</code>, 
- * 	then a <code>ClasspathContainerInitializer</code> will be activated if any was registered for this 
+ *     recorded for this container path onto this project (using {@link JavaCore#setClasspathContainer}, 
+ * 	then a {@link ClasspathContainerInitializer} will be activated if any was registered for this 
  * 	container ID onto the extension point "org.aspectj.org.eclipse.jdt.core.classpathContainerInitializer".
- * 	A classpath container entry can be resolved explicitly using <code>JavaCore#getClasspathContainer</code>
+ * 	A classpath container entry can be resolved explicitly using {@link JavaCore#getClasspathContainer}
  * 	and the resulting container entries can contain any non-container entry. In particular, it may contain variable
  *     entries, which in turn needs to be resolved before being directly used. 
  * 	<br> Also note that the container resolution APIs include an IJavaProject argument, so as to allow the same
  * 	container path to be interpreted in different ways for different projects. </li>
  * </ul>
  * </p>
- * The result of <code>IJavaProject#getResolvedClasspath</code> will have all entries of type
- * <code>CPE_VARIABLE</code> and <code>CPE_CONTAINER</code> resolved to a set of 
- * <code>CPE_SOURCE</code>, <code>CPE_LIBRARY</code> or <code>CPE_PROJECT</code>
+ * The result of {@link IJavaProject#getResolvedClasspath} will have all entries of type
+ * {@link #CPE_VARIABLE} and {@link #CPE_CONTAINER} resolved to a set of 
+ * {@link #CPE_SOURCE}, {@link #CPE_LIBRARY} or {@link #CPE_PROJECT}
  * classpath entries.
  * <p>
- * Any classpath entry other than a source folder (kind <code>CPE_SOURCE</code>) can
+ * Any classpath entry other than a source folder (kind {@link #CPE_SOURCE}) can
  * be marked as being exported. Exported entries are automatically contributed to
  * dependent projects, along with the project's default output folder, which is
  * implicitly exported, and any auxiliary output folders specified on source
@@ -100,7 +102,7 @@ import org.eclipse.core.runtime.IPath;
  * followed by the any exported entries.
  * <p>
  * This interface is not intended to be implemented by clients.
- * Classpath entries can be created via methods on <code>JavaCore</code>.
+ * Classpath entries can be created via methods on {@link JavaCore}.
  * </p>
  *
  * @see JavaCore#newLibraryEntry(org.eclipse.core.runtime.IPath, org.eclipse.core.runtime.IPath, org.eclipse.core.runtime.IPath)
@@ -168,11 +170,11 @@ public interface IClasspathEntry {
 	 * Returns the kind of files found in the package fragments identified by this
 	 * classpath entry.
 	 *
-	 * @return <code>IPackageFragmentRoot.K_SOURCE</code> for files containing
-	 *   source code, and <code>IPackageFragmentRoot.K_BINARY</code> for binary
+	 * @return {@link IPackageFragmentRoot#K_SOURCE} for files containing
+	 *   source code, and {@link IPackageFragmentRoot#K_BINARY} for binary
 	 *   class files.
-	 *   There is no specified value for an entry denoting a variable (<code>CPE_VARIABLE</code>)
-	 *   or a classpath container (<code>CPE_CONTAINER</code>).
+	 *   There is no specified value for an entry denoting a variable ({@link #CPE_VARIABLE})
+	 *   or a classpath container ({@link #CPE_CONTAINER}).
 	 */
 	int getContentKind();
 
@@ -181,16 +183,16 @@ public interface IClasspathEntry {
 	 *
 	 * @return one of:
 	 * <ul>
-	 * <li><code>CPE_SOURCE</code> - this entry describes a source root in
+	 * <li>{@link #CPE_SOURCE} - this entry describes a source root in
 	 		its project
-	 * <li><code>CPE_LIBRARY</code> - this entry describes a folder or JAR
+	 * <li>{@link #CPE_LIBRARY} - this entry describes a folder or JAR
 	 		containing binaries
-	 * <li><code>CPE_PROJECT</code> - this entry describes another project
+	 * <li>{@link #CPE_PROJECT} - this entry describes another project
 	 *
-	 * <li><code>CPE_VARIABLE</code> - this entry describes a project or library
+	 * <li>{@link #CPE_VARIABLE} - this entry describes a project or library
 	 *  	indirectly via a classpath variable in the first segment of the path
 	 * *
-	 * <li><code>CPE_CONTAINER</code> - this entry describes set of entries
+	 * <li>{@link #CPE_CONTAINER} - this entry describes set of entries
 	 *  	referenced indirectly via a classpath container
 	 * </ul>
 	 */
@@ -237,13 +239,13 @@ public interface IClasspathEntry {
 	 * Combinations of *'s and ?'s are allowed.
 	 * </p>
 	 * <p>
-	 * The special pattern '**' matches zero or more segments. In a source enry,
+	 * The special pattern '**' matches zero or more segments. In a source entry,
 	 * a path like <code>tests/</code> that ends in a trailing separator is interpreted
 	 * as <code>tests/&#42;&#42;</code>, and would match everything under
 	 * the folder named <code>tests</code>.
 	 * </p>
 	 * <p>
-	 * Example patterns in source entries:
+	 * Example patterns in source entries (assuming that "java" is the only {@link JavaCore#getJavaLikeExtensions() Java-like extension}):
 	 * <ul>
 	 * <li>
 	 * <code>tests/&#42;&#42;</code> (or simply <code>tests/</code>) 
@@ -342,11 +344,11 @@ public interface IClasspathEntry {
 	/**
 	 * Returns the full path to the specific location where the builder writes 
 	 * <code>.class</code> files generated for this source entry 
-	 * (entry kind <code>CPE_SOURCE</code>).
+	 * (entry kind {@link #CPE_SOURCE}).
 	 * <p>
 	 * Source entries can optionally be associated with a specific output location.
 	 * If none is provided, the source entry will be implicitly associated with its project
-	 * default output location (see <code>IJavaProject#getOutputLocation</code>).
+	 * default output location (see {@link IJavaProject#getOutputLocation}).
 	 * </p><p>
 	 * NOTE: A specific output location cannot coincidate with another source/library entry.
 	 * </p>
@@ -362,24 +364,24 @@ public interface IClasspathEntry {
 	 * Returns the path of this classpath entry.
 	 *
 	 * The meaning of the path of a classpath entry depends on its entry kind:<ul>
-	 *	<li>Source code in the current project (<code>CPE_SOURCE</code>) -  
+	 *	<li>Source code in the current project ({@link #CPE_SOURCE}) -  
 	 *      The path associated with this entry is the absolute path to the root folder. </li>
-	 *	<li>A binary library in the current project (<code>CPE_LIBRARY</code>) - the path
+	 *	<li>A binary library in the current project ({@link #CPE_LIBRARY}) - the path
 	 *		associated with this entry is the absolute path to the JAR (or root folder), and 
 	 *		in case it refers to an external JAR, then there is no associated resource in 
 	 *		the workbench.
-	 *	<li>A required project (<code>CPE_PROJECT</code>) - the path of the entry denotes the
+	 *	<li>A required project ({@link #CPE_PROJECT}) - the path of the entry denotes the
 	 *		path to the corresponding project resource.</li>
-	 *  <li>A variable entry (<code>CPE_VARIABLE</code>) - the first segment of the path 
+	 *  <li>A variable entry ({@link #CPE_VARIABLE}) - the first segment of the path 
 	 *      is the name of a classpath variable. If this classpath variable
-	 *		is bound to the path <it>P</it>, the path of the corresponding classpath entry
-	 *		is computed by appending to <it>P</it> the segments of the returned
+	 *		is bound to the path <i>P</i>, the path of the corresponding classpath entry
+	 *		is computed by appending to <i>P</i> the segments of the returned
 	 *		path without the variable.</li>
-	 *  <li> A container entry (<code>CPE_CONTAINER</code>) - the path of the entry
+	 *  <li> A container entry ({@link #CPE_CONTAINER}) - the path of the entry
 	 * 	is the name of the classpath container, which can be bound indirectly to a set of classpath 
 	 * 	entries after resolution. The containerPath is a formed by a first ID segment followed with 
 	 *     extra segments that can be used as additional hints for resolving this container 
-	 * 	reference (also see <code>IClasspathContainer</code>).
+	 * 	reference (also see {@link IClasspathContainer}).
 	 * </li>
 	 * </ul>
 	 *
@@ -408,7 +410,7 @@ public interface IClasspathEntry {
 	 * Returns the path within the source archive or folder where package fragments
 	 * are located. An empty path indicates that packages are located at
 	 * the root of the source archive or folder. Returns a non-<code>null</code> value
-	 * if and only if <code>getSourceAttachmentPath</code> returns 
+	 * if and only if {@link #getSourceAttachmentPath} returns 
 	 * a non-<code>null</code> value.
 	 *
 	 * @return the path within the source archive or folder, or <code>null</code> if
@@ -419,7 +421,7 @@ public interface IClasspathEntry {
 	/**
 	 * Returns whether this entry is exported to dependent projects.
 	 * Always returns <code>false</code> for source entries (kind
-	 * <code>CPE_SOURCE</code>), which cannot be exported.
+	 * {@link #CPE_SOURCE}), which cannot be exported.
 	 * 
 	 * @return <code>true</code> if exported, and <code>false</code> otherwise
 	 * @since 2.0
@@ -429,7 +431,7 @@ public interface IClasspathEntry {
 	/**
 	 * This is a helper method, which returns the resolved classpath entry denoted 
 	 * by an entry (if it is a variable entry). It is obtained by resolving the variable 
-	 * reference in the first segment. Returns <node>null</code> if unable to resolve using 
+	 * reference in the first segment. Returns <code>null</code> if unable to resolve using 
 	 * the following algorithm:
 	 * <ul>
 	 * <li> if variable segment cannot be resolved, returns <code>null</code></li>
@@ -445,7 +447,7 @@ public interface IClasspathEntry {
 	 *	<p> 
 	 * Note that this deprecated API doesn't handle CPE_CONTAINER entries.
 	 * 
-	 * @deprecated - use JavaCore.getResolvedClasspathEntry(...)
+	 * @deprecated Use {@link JavaCore#getResolvedClasspathEntry(IClasspathEntry)} instead
 	 */
 	IClasspathEntry getResolvedEntry();	
 }
