@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,6 +11,7 @@
 package org.aspectj.org.eclipse.jdt.core;
 
 import org.eclipse.core.resources.IResourceDelta;
+import org.aspectj.org.eclipse.jdt.core.dom.CompilationUnit;
 
 /**
  * A Java element delta describes changes in Java element between two discrete
@@ -18,45 +19,45 @@ import org.eclipse.core.resources.IResourceDelta;
  * changed, and any children that have changed.
  * <p>
  * Deltas have a different status depending on the kind of change they represent.  
- * The list below summarizes each status (as returned by <code>getKind</code>)
+ * The list below summarizes each status (as returned by {@link #getKind})
  * and its meaning (see individual constants for a more detailled description):
  * <ul>
- * <li><code>ADDED</code> - The element described by the delta has been added.</li>
- * <li><code>REMOVED</code> - The element described by the delta has been removed.</li>
- * <li><code>CHANGED</code> - The element described by the delta has been changed in some way.  
- * Specification of the type of change is provided by <code>getFlags</code> which returns the following values:
+ * <li>{@link #ADDED} - The element described by the delta has been added.</li>
+ * <li>{@link #REMOVED} - The element described by the delta has been removed.</li>
+ * <li>{@link #CHANGED} - The element described by the delta has been changed in some way.  
+ * Specification of the type of change is provided by {@link #getFlags} which returns the following values:
  * <ul>
- * <li><code>F_ADDED_TO_CLASSPATH</code> - A classpath entry corresponding to the element
+ * <li>{@link #F_ADDED_TO_CLASSPATH} - A classpath entry corresponding to the element
  * has been added to the project's classpath. This flag is only valid if the element is an 
- * <code>IPackageFragmentRoot</code>.</li>
- * <li><code>F_ARCHIVE_CONTENT_CHANGED</code> - The contents of an archive
- * has changed in some way. This flag is only valid if the element is an <code>IPackageFragmentRoot</code>
+ * {@link IPackageFragmentRoot}.</li>
+ * <li>{@link #F_ARCHIVE_CONTENT_CHANGED} - The contents of an archive
+ * has changed in some way. This flag is only valid if the element is an {@link IPackageFragmentRoot}
  * which is an archive.</li>
- * <li><code>F_CHILDREN</code> - A child of the element has changed in some way.  This flag
- * is only valid if the element is an <code>IParent</code>.</li>
- * <li><code>F_CLASSPATH_REORDER</code> - A classpath entry corresponding to the element
+ * <li>{@link #F_CHILDREN} - A child of the element has changed in some way.  This flag
+ * is only valid if the element is an {@link IParent}.</li>
+ * <li>{@link #F_CLASSPATH_REORDER} - A classpath entry corresponding to the element
  * has changed position in the project's classpath. This flag is only valid if the element is an 
- * <code>IPackageFragmentRoot</code>.</li>
- * <li><code>F_CLOSED</code> - The underlying <code>IProject</code>
- * has been closed. This flag is only valid if the element is an <code>IJavaProject</code>.</li>
- * <li><code>F_CONTENT</code> - The contents of the element have been altered.  This flag
+ * {@link IPackageFragmentRoot}.</li>
+ * <li>{@link #F_CLOSED} - The underlying {@link org.eclipse.core.resources.IProject}
+ * has been closed. This flag is only valid if the element is an {@link IJavaProject}.</li>
+ * <li>{@link #F_CONTENT} - The contents of the element have been altered.  This flag
  * is only valid for elements which correspond to files.</li>
- *<li><code>F_FINE_GRAINED</code> - The delta is a fine-grained delta, that is, an analysis down
+ *<li>{@link #F_FINE_GRAINED} - The delta is a fine-grained delta, that is, an analysis down
  * to the members level was done to determine if there were structural changes to members of the element.</li>
- * <li><code>F_MODIFIERS</code> - The modifiers on the element have changed in some way. 
- * This flag is only valid if the element is an <code>IMember</code>.</li>
- * <li><code>F_OPENED</code> - The underlying <code>IProject</code>
- * has been opened. This flag is only valid if the element is an <code>IJavaProject</code>.</li>
- * <li><code>F_REMOVED_FROM_CLASSPATH</code> - A classpath entry corresponding to the element 
+ * <li>{@link #F_MODIFIERS} - The modifiers on the element have changed in some way. 
+ * This flag is only valid if the element is an {@link IMember}.</li>
+ * <li>{@link #F_OPENED} - The underlying {@link org.eclipse.core.resources.IProject}
+ * has been opened. This flag is only valid if the element is an {@link IJavaProject}.</li>
+ * <li>{@link #F_REMOVED_FROM_CLASSPATH} - A classpath entry corresponding to the element 
  * has been removed from the project's classpath. This flag is only valid if the element is an 
- * <code>IPackageFragmentRoot</code>.</li>
- * <li><code>F_SOURCEATTACHED</code> - The source attachment path or the source attachment root path
+ * {@link IPackageFragmentRoot}.</li>
+ * <li>{@link #F_SOURCEATTACHED} - The source attachment path or the source attachment root path
  * of a classpath entry corresponding to the element was added. This flag is only valid if the element is an 
- * <code>IPackageFragmentRoot</code>.</li>
- * <li><code>F_SOURCEDETACHED</code> - The source attachment path or the source attachment root path
+ * {@link IPackageFragmentRoot}.</li>
+ * <li>{@link #F_SOURCEDETACHED} - The source attachment path or the source attachment root path
  * of a classpath entry corresponding to the element was removed. This flag is only valid if the element is an 
- * <code>IPackageFragmentRoot</code>.</li>
- * <li><code>F_SUPER_TYPES</code> - One of the supertypes of an <code>IType</code> has changed</li>.
+ * {@link IPackageFragmentRoot}.</li>
+ * <li>{@link #F_SUPER_TYPES} - One of the supertypes of an {@link IType} has changed</li>.
  * </ul>
  * </li>
  * </ul>
@@ -64,11 +65,11 @@ import org.eclipse.core.resources.IResourceDelta;
  * <p>
  * Move operations are indicated by other change flags, layered on top
  * of the change flags described above. If element A is moved to become B,
- * the delta for the  change in A will have status <code>REMOVED</code>,
- * with change flag <code>F_MOVED_TO</code>. In this case,
- * <code>getMovedToElement</code> on delta A will return the handle for B.
- * The  delta for B will have status <code>ADDED</code>, with change flag
- * <code>F_MOVED_FROM</code>, and <code>getMovedFromElement</code> on delta
+ * the delta for the  change in A will have status {@link #REMOVED},
+ * with change flag {@link #F_MOVED_TO}. In this case,
+ * {@link #getMovedToElement} on delta A will return the handle for B.
+ * The  delta for B will have status {@link #ADDED}, with change flag
+ * {@link #F_MOVED_FROM}, and {@link #getMovedFromElement} on delta
  * B will return the handle for A. (Note, the handle to A in this case represents
  * an element that no longer exists).
  * </p>
@@ -77,25 +78,25 @@ import org.eclipse.core.resources.IResourceDelta;
  * do not imply anything about the parent or children of the element.
  * </p>
  * <p>
- * The <code>F_ADDED_TO_CLASSPATH</code>, <code>F_REMOVED_FROM_CLASSPATH</code> and
- * <code>F_CLASSPATH_REORDER</code> flags are triggered by changes to a project's classpath. They do not mean that
+ * The {@link #F_ADDED_TO_CLASSPATH}, {@link #F_REMOVED_FROM_CLASSPATH} and
+ * {@link #F_CLASSPATH_REORDER} flags are triggered by changes to a project's classpath. They do not mean that
  * the underlying resource was added, removed or changed. For example, if a project P already contains a folder src, then 
- * adding a classpath entry with the 'P/src' path to the project's classpath will result in an <code>IJavaElementDelta</code> 
- * with the <code>F_ADDED_TO_CLASSPATH</code> flag for the <code>IPackageFragmentRoot</code> P/src.
+ * adding a classpath entry with the 'P/src' path to the project's classpath will result in an {@link IJavaElementDelta} 
+ * with the {@link #F_ADDED_TO_CLASSPATH} flag for the {@link IPackageFragmentRoot} P/src.
  * On the contrary, if a resource is physically added, removed or changed and this resource corresponds to a classpath
- * entry of the project, then an <code>IJavaElementDelta</code> with the <code>ADDED</code>, 
- * <code>REMOVED</code>, or <code>CHANGED</code> kind will be fired.
+ * entry of the project, then an {@link IJavaElementDelta} with the {@link #ADDED}, 
+ * {@link #REMOVED}, or {@link #CHANGED} kind will be fired.
  * </p>
  * <p>
  * Note that when a source attachment path or a source attachment root path is changed, then the flags of the delta contain
- * both <code>F_SOURCEATTACHED</code> and <code>F_SOURCEDETTACHED</code>.
+ * both {@link #F_SOURCEATTACHED} and {@link #F_SOURCEDETACHED}.
  * </p>
  * <p>
- * No assumptions should be made on whether the java element delta tree is rooted at the <code>IJavaModel</code>
+ * No assumptions should be made on whether the java element delta tree is rooted at the {@link IJavaModel}
  * level or not.
  * </p>
  * <p>
- * <code>IJavaElementDelta</code> object are not valid outside the dynamic scope
+ * {@link IJavaElementDelta} object are not valid outside the dynamic scope
  * of the notification.
  * </p>
  * <p>
@@ -128,90 +129,90 @@ public interface IJavaElementDelta {
 	 * Change flag indicating that the content of the element has changed.
 	 * This flag is only valid for elements which correspond to files.
 	 */
-	public int F_CONTENT = 0x00001;
+	public int F_CONTENT = 0x000001;
 
 	/**
 	 * Change flag indicating that the modifiers of the element have changed.
-	 * This flag is only valid if the element is an <code>IMember</code>. 
+	 * This flag is only valid if the element is an {@link IMember}. 
 	 */
-	public int F_MODIFIERS = 0x00002;
+	public int F_MODIFIERS = 0x000002;
 
 	/**
 	 * Change flag indicating that there are changes to the children of the element.
-	 * This flag is only valid if the element is an <code>IParent</code>. 
+	 * This flag is only valid if the element is an {@link IParent}. 
 	 */
-	public int F_CHILDREN = 0x00008;
+	public int F_CHILDREN = 0x000008;
 
 	/**
 	 * Change flag indicating that the element was moved from another location.
-	 * The location of the old element can be retrieved using <code>getMovedFromElement</code>.
+	 * The location of the old element can be retrieved using {@link #getMovedFromElement}.
 	 */
-	public int F_MOVED_FROM = 0x00010;
+	public int F_MOVED_FROM = 0x000010;
 
 	/**
 	 * Change flag indicating that the element was moved to another location.
-	 * The location of the new element can be retrieved using <code>getMovedToElement</code>.
+	 * The location of the new element can be retrieved using {@link #getMovedToElement}.
 	 */
-	public int F_MOVED_TO = 0x00020;
+	public int F_MOVED_TO = 0x000020;
 
 	/**
 	 * Change flag indicating that a classpath entry corresponding to the element has been added to the project's classpath. 
-	 * This flag is only valid if the element is an <code>IPackageFragmentRoot</code>.
+	 * This flag is only valid if the element is an {@link IPackageFragmentRoot}.
 	 */
-	public int F_ADDED_TO_CLASSPATH = 0x00040;
+	public int F_ADDED_TO_CLASSPATH = 0x000040;
 
 	/**
 	 * Change flag indicating that a classpath entry corresponding to the element has been removed from the project's 
-	 * classpath. This flag is only valid if the element is an <code>IPackageFragmentRoot</code>.
+	 * classpath. This flag is only valid if the element is an {@link IPackageFragmentRoot}.
 	 */
-	public int F_REMOVED_FROM_CLASSPATH = 0x00080;
+	public int F_REMOVED_FROM_CLASSPATH = 0x000080;
 
 	/**
 	 * Change flag indicating that a classpath entry corresponding to the element has changed position in the project's 
-	 * classpath. This flag is only valid if the element is an <code>IPackageFragmentRoot</code>.
-	 * @deprecated Use F_REORDER instead.
+	 * classpath. This flag is only valid if the element is an {@link IPackageFragmentRoot}.
+	 * @deprecated Use {@link #F_REORDER} instead.
 	 */
-	public int F_CLASSPATH_REORDER = 0x00100;
+	public int F_CLASSPATH_REORDER = 0x000100;
 	/**
 	 * Change flag indicating that the element has changed position relatively to its siblings. 
-	 * If the element is an <code>IPackageFragmentRoot</code>,  a classpath entry corresponding 
+	 * If the element is an {@link IPackageFragmentRoot},  a classpath entry corresponding 
 	 * to the element has changed position in the project's classpath.
 	 * 
 	 * @since 2.1
 	 */
-	public int F_REORDER = 0x00100;
+	public int F_REORDER = 0x000100;
 
 	/**
-	 * Change flag indicating that the underlying <code>IProject</code> has been
-	 * opened. This flag is only valid if the element is an <code>IJavaProject</code>. 
+	 * Change flag indicating that the underlying {@link org.eclipse.core.resources.IProject} has been
+	 * opened. This flag is only valid if the element is an {@link IJavaProject}. 
 	 */
-	public int F_OPENED = 0x00200;
+	public int F_OPENED = 0x000200;
 
 	/**
-	 * Change flag indicating that the underlying <code>IProject</code> has been
-	 * closed. This flag is only valid if the element is an <code>IJavaProject</code>. 
+	 * Change flag indicating that the underlying {@link org.eclipse.core.resources.IProject} has been
+	 * closed. This flag is only valid if the element is an {@link IJavaProject}. 
 	 */
-	public int F_CLOSED = 0x00400;
+	public int F_CLOSED = 0x000400;
 
 	/**
-	 * Change flag indicating that one of the supertypes of an <code>IType</code>
+	 * Change flag indicating that one of the supertypes of an {@link IType}
 	 * has changed.
 	 */
-	public int F_SUPER_TYPES = 0x00800;
+	public int F_SUPER_TYPES = 0x000800;
 
 	/**
 	 * Change flag indicating that the source attachment path or the source attachment root path of a classpath entry 
 	 * corresponding to the element was added. This flag is only valid if the element is an 
-	 * <code>IPackageFragmentRoot</code>.
+	 * {@link IPackageFragmentRoot}.
 	 */
-	public int F_SOURCEATTACHED = 0x01000;	
+	public int F_SOURCEATTACHED = 0x001000;	
 
 	/**
 	 * Change flag indicating that the source attachment path or the source attachment root path of a classpath entry 
 	 * corresponding to the element was removed. This flag is only valid if the element is an 
-	 * <code>IPackageFragmentRoot</code>.
+	 * {@link IPackageFragmentRoot}.
 	 */
-	public int F_SOURCEDETACHED = 0x02000;	
+	public int F_SOURCEDETACHED = 0x002000;	
 	
 	/**
 	 * Change flag indicating that this is a fine-grained delta, that is, an analysis down
@@ -219,50 +220,67 @@ public interface IJavaElementDelta {
 	 * members.
 	 * <p>
 	 * Clients can use this flag to find out if a compilation unit 
-     * that have a <code>F_CONTENT</code> change should assume that there are 
-     * no finer grained changes (<code>F_FINE_GRAINED</code> is set) or if 
-     * finer grained changes were not considered (<code>F_FINE_GRAINED</code> 
+     * that have a {@link #F_CONTENT} change should assume that there are 
+     * no finer grained changes ({@link #F_FINE_GRAINED} is set) or if 
+     * finer grained changes were not considered ({@link #F_FINE_GRAINED} 
      * is not set). 
      * 
      * @since 2.0
 	 */
-	public int F_FINE_GRAINED = 0x04000;
+	public int F_FINE_GRAINED = 0x004000;
 
 	/**
 	 * Change flag indicating that the element's archive content on the classpath has changed.
-	 * This flag is only valid if the element is an <code>IPackageFragmentRoot</code>
+	 * This flag is only valid if the element is an {@link IPackageFragmentRoot}
 	 * which is an archive.
 	 * 
 	 * @see IPackageFragmentRoot#isArchive()
 	 * @since 2.0
 	 */
-	public int F_ARCHIVE_CONTENT_CHANGED = 0x08000;
+	public int F_ARCHIVE_CONTENT_CHANGED = 0x008000;
 	
 	/**
 	 * Change flag indicating that a compilation unit has become a primary working copy, or that a 
 	 * primary working copy has reverted to a compilation unit.
-	 * This flag is only valid if the element is an <code>ICompilationUnit</code>.
+	 * This flag is only valid if the element is an {@link ICompilationUnit}.
 	 * 
 	 * @since 3.0
 	 */
-	public int F_PRIMARY_WORKING_COPY = 0x10000;
+	public int F_PRIMARY_WORKING_COPY = 0x010000;
 
 	/**
 	 * Change flag indicating that the raw classpath (or the output folder) of a project has changed. 
-	 * This flag is only valid if the element is an <code>IJavaProject</code>.
+	 * This flag is only valid if the element is an {@link IJavaProject}.
 	 *
 	 * @since 3.0
 	 */
-	public int F_CLASSPATH_CHANGED = 0x20000;
+	public int F_CLASSPATH_CHANGED = 0x020000;
 
 	/**
 	 * Change flag indicating that the resource of a primary compilation unit has changed.
-	 * This flag is only valid if the element is a primary <code>ICompilationUnit</code>.
+	 * This flag is only valid if the element is a primary {@link ICompilationUnit}.
 	 * 
 	 * @since 3.0
 	 */
-	public int F_PRIMARY_RESOURCE = 0x40000;
+	public int F_PRIMARY_RESOURCE = 0x040000;
 
+	/**
+	 * Change flag indicating that a reconcile operation has affected the compilation unit AST created in a 
+	 * previous reconcile operation. Use {@link #getCompilationUnitAST()} to retrieve the AST (if any is available).
+	 * This flag is only valid if the element is an {@link ICompilationUnit} in working copy mode.
+	 * 
+	 * @since 3.2
+	 */
+	public int F_AST_AFFECTED = 0x080000;
+	
+	/**
+	 * Change flag indicating that the categories of the element have changed.
+	 * This flag is only valid if the element is an {@link IMember}. 
+	 * 
+	 * @since 3.2
+	 */
+	public int F_CATEGORIES = 0x100000;
+	
 	/**
 	 * Returns deltas for the children that have been added.
 	 * @return deltas for the children that have been added
@@ -274,6 +292,22 @@ public interface IJavaElementDelta {
 	 * @return deltas for the affected (added, removed, or changed) children
 	 */
 	public IJavaElementDelta[] getAffectedChildren();
+	
+	/**
+	 * Returns the compilation unit AST created by the last reconcile operation on this delta's element.
+	 * This returns a non-null value if and only if:
+	 * <ul>
+	 * <li>the last reconcile operation on this working copy requested an AST</li>
+	 * <li>this delta's element is an {@link ICompilationUnit} in working copy mode</li>
+	 * <li>the delta comes from a {@link ElementChangedEvent#POST_RECONCILE} event
+	 * </ul>
+	 * 
+	 * @return the AST created during the last reconcile operation
+	 * @see ICompilationUnit#reconcile(int, boolean, WorkingCopyOwner, org.eclipse.core.runtime.IProgressMonitor)
+	 * @see #F_AST_AFFECTED
+	 * @since 3.2
+	 */
+	public CompilationUnit getCompilationUnitAST();
 
 	/**
 	 * Returns deltas for the children which have changed.
@@ -301,8 +335,8 @@ public interface IJavaElementDelta {
 	public int getFlags();
 
 	/**
-	 * Returns the kind of this delta - one of <code>ADDED</code>, <code>REMOVED</code>,
-	 * or <code>CHANGED</code>.
+	 * Returns the kind of this delta - one of {@link #ADDED}, {@link #REMOVED},
+	 * or {@link #CHANGED}.
 	 * 
 	 * @return the kind of this delta
 	 */
@@ -311,21 +345,21 @@ public interface IJavaElementDelta {
 	/**
 	 * Returns an element describing this element before it was moved
 	 * to its current location, or <code>null</code> if the
-	 * <code>F_MOVED_FROM</code> change flag is not set. 
+	 * {@link #F_MOVED_FROM} change flag is not set. 
 	 * 
 	 * @return an element describing this element before it was moved
 	 * to its current location, or <code>null</code> if the
-	 * <code>F_MOVED_FROM</code> change flag is not set
+	 * {@link #F_MOVED_FROM} change flag is not set
 	 */
 	public IJavaElement getMovedFromElement();
 
 	/**
 	 * Returns an element describing this element in its new location,
-	 * or <code>null</code> if the <code>F_MOVED_TO</code> change
+	 * or <code>null</code> if the {@link #F_MOVED_TO} change
 	 * flag is not set.
 	 * 
 	 * @return an element describing this element in its new location,
-	 * or <code>null</code> if the <code>F_MOVED_TO</code> change
+	 * or <code>null</code> if the {@link #F_MOVED_TO} change
 	 * flag is not set
 	 */
 	public IJavaElement getMovedToElement();
