@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,6 +11,7 @@
 
 package org.aspectj.org.eclipse.jdt.core.dom;
 
+import org.aspectj.org.eclipse.jdt.core.compiler.CategorizedProblem;
 import org.aspectj.org.eclipse.jdt.core.compiler.IProblem;
 
 /**
@@ -18,9 +19,9 @@ import org.aspectj.org.eclipse.jdt.core.compiler.IProblem;
  */
 class ASTSyntaxErrorPropagator extends ASTVisitor {
 
-	private IProblem[] problems;
+	private CategorizedProblem[] problems;
 	
-	ASTSyntaxErrorPropagator(IProblem[] problems) {
+	ASTSyntaxErrorPropagator(CategorizedProblem[] problems) {
 		// visit Javadoc.tags() as well
 		super(true);
 		this.problems = problems;
@@ -29,7 +30,7 @@ class ASTSyntaxErrorPropagator extends ASTVisitor {
 	private boolean checkAndTagAsMalformed(ASTNode node) {
 		boolean tagWithErrors = false;
 		search: for (int i = 0, max = this.problems.length; i < max; i++) {
-			IProblem problem = this.problems[i];
+			CategorizedProblem problem = this.problems[i];
 			switch(problem.getID()) {
 				case IProblem.ParsingErrorOnKeywordNoSuggestion :
 				case IProblem.ParsingErrorOnKeyword :
@@ -116,6 +117,27 @@ class ASTSyntaxErrorPropagator extends ASTVisitor {
 		return checkAndTagAsMalformed(node);		
 	}
 
+	/*
+	 * Method declared on ASTVisitor.
+	 */
+	public boolean visit(AnnotationTypeDeclaration node) {
+		return checkAndTagAsMalformed(node);		
+	}
+	
+	/*
+	 * Method declared on ASTVisitor.
+	 */
+	public boolean visit(EnumDeclaration node) {
+		return checkAndTagAsMalformed(node);		
+	}
+	
+	/*
+	 * Method declared on ASTVisitor.
+	 */
+	public boolean visit(TypeDeclaration node) {
+		return checkAndTagAsMalformed(node);		
+	}
+	
 	/*
 	 * Method declared on ASTVisitor.
 	 */
