@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004 IBM Corporation and others.
+ * Copyright (c) 2004, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -14,6 +14,8 @@ package org.aspectj.org.eclipse.jdt.core.dom;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.aspectj.org.eclipse.jdt.internal.compiler.util.Util;
+
 /**
  * AST node for a text element within a doc comment.
  * <pre>
@@ -54,7 +56,7 @@ public final class TextElement extends ASTNode implements IDocElement {
 	 * Clients must not modify the result.
 	 * 
 	 * @param apiLevel the API level; one of the
-	 * <code>AST.JLS&ast;</code> constants
+	 * <code>AST.JLS*</code> constants
 	 * @return a list of property descriptors (element type: 
 	 * {@link StructuralPropertyDescriptor})
 	 * @since 3.0
@@ -63,15 +65,10 @@ public final class TextElement extends ASTNode implements IDocElement {
 		return PROPERTY_DESCRIPTORS;
 	}
 	
-	/**
-	 * Canonical empty string.
-	 */
-	private static final String EMPTY_STRING = ""; //$NON-NLS-1$
-	
 	/**
 	 * The text element; defaults to the empty string.
 	 */
-	private String text = EMPTY_STRING;
+	private String text = Util.EMPTY_STRING;
 	
 	/**
 	 * Creates a new AST node for a text element owned by the given AST.
@@ -183,7 +180,7 @@ public final class TextElement extends ASTNode implements IDocElement {
 	 */
 	int memSize() {
 		int size = BASE_NODE_SIZE + 1 * 4;
-		if (this.text != EMPTY_STRING) {
+		if (this.text != Util.EMPTY_STRING) {
 			// everything but our empty string costs
 			size += stringSize(this.text);
 		}
