@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2005 IBM Corporation and others.
+ * Copyright (c) 2004, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,32 +11,37 @@
 package org.aspectj.org.eclipse.jdt.core.dom.rewrite;
 
 import java.util.Iterator;
+import java.util.List;
 import java.util.Map;
 
-import org.eclipse.text.edits.MultiTextEdit;
-import org.eclipse.text.edits.TextEdit;
-import org.eclipse.text.edits.TextEditGroup;
-
-import org.eclipse.jface.text.IDocument;
-
+import org.aspectj.org.eclipse.jdt.core.IClassFile;
+import org.aspectj.org.eclipse.jdt.core.ICompilationUnit;
+import org.aspectj.org.eclipse.jdt.core.ITypeRoot;
 import org.aspectj.org.eclipse.jdt.core.JavaCore;
-
+import org.aspectj.org.eclipse.jdt.core.JavaModelException;
 import org.aspectj.org.eclipse.jdt.core.dom.AST;
 import org.aspectj.org.eclipse.jdt.core.dom.ASTNode;
+import org.aspectj.org.eclipse.jdt.core.dom.ASTParser;
 import org.aspectj.org.eclipse.jdt.core.dom.ASTVisitor;
 import org.aspectj.org.eclipse.jdt.core.dom.Block;
 import org.aspectj.org.eclipse.jdt.core.dom.ChildListPropertyDescriptor;
+import org.aspectj.org.eclipse.jdt.core.dom.CompilationUnit;
 import org.aspectj.org.eclipse.jdt.core.dom.StructuralPropertyDescriptor;
-
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.ASTRewriteAnalyzer;
+import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.LineInformation;
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.NodeInfoStore;
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.NodeRewriteEvent;
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.RewriteEventStore;
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.TrackedNodePosition;
 import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.RewriteEventStore.CopySourceInfo;
+import org.eclipse.jface.text.IDocument;
+import org.eclipse.jface.text.TextUtilities;
+import org.eclipse.text.edits.MultiTextEdit;
+import org.eclipse.text.edits.TextEdit;
+import org.eclipse.text.edits.TextEditGroup;
 
 /**
- * Infrastucture for modifying code by describing changes to AST nodes.
+ * Infrastructure for modifying code by describing changes to AST nodes.
  * The AST rewriter collects descriptions of modifications to nodes and
  * translates these descriptions into text edits that can then be applied to
  * the original source. The key thing is that this is all done without actually
@@ -51,26 +56,24 @@ import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.RewriteEventStore.C
  * The following code snippet illustrated usage of this class:
  * </p>
  * <pre>
- * Document doc = new Document("import java.util.List;\nclass X {}\n");
+ * Document document = new Document("import java.util.List;\nclass X {}\n");
  * ASTParser parser = ASTParser.newParser(AST.JLS3);
  * parser.setSource(doc.get().toCharArray());
  * CompilationUnit cu = (CompilationUnit) parser.createAST(null);
  * AST ast = cu.getAST();
  * ImportDeclaration id = ast.newImportDeclaration();
- * id.setName(ast.newName(new String[] {"java", "util", "Set"});
+ * id.setName(ast.newName(new String[] {"java", "util", "Set"}));
  * ASTRewrite rewriter = ASTRewrite.create(ast);
  * TypeDeclaration td = (TypeDeclaration) cu.types().get(0);
  * ITrackedNodePosition tdLocation = rewriter.track(td);
- * ListRewriter lrw = rewriter.getListRewrite(cu,
- *                       CompilationUnit.IMPORTS_PROPERTY);
+ * ListRewrite lrw = rewriter.getListRewrite(cu, CompilationUnit.IMPORTS_PROPERTY);
  * lrw.insertLast(id, null);
  * TextEdit edits = rewriter.rewriteAST(document, null);
  * UndoEdit undo = edits.apply(document);
- * assert "import java.util.List;\nimport java.util.Set;\nclass X {}"
- *   .equals(doc.get().toCharArray());
+ * assert "import java.util.List;\nimport java.util.Set;\nclass X {}".equals(doc.get().toCharArray());
  * // tdLocation.getStartPosition() and tdLocation.getLength()
  * // are new source range for "class X {}" in doc.get()
- * </pre>
+ * </pre> 
  * <p>
  * This class is not intended to be subclassed.
  * </p>
@@ -173,23 +176,94 @@ public class ASTRewrite {
 		if (document == null) {
 			throw new IllegalArgumentException();
 		}
-		TextEdit result= new MultiTextEdit();
 		
 		ASTNode rootNode= getRootNode();
-		if (rootNode != null) {
-			//validateASTNotModified(rootNode);
-			
-			TargetSourceRangeComputer sourceRangeComputer= getExtendedSourceRangeComputer();
-			
-			this.eventStore.prepareMovedNodes(sourceRangeComputer);
-
-			// AspectJ extension - ask the factory in ASTRewriteAnalyzer for a visitor rather than building it directly.
-			ASTVisitor visitor= ASTRewriteAnalyzer.getAnalyzerVisitor(document, result, this.eventStore, this.nodeStore, options, sourceRangeComputer);
-			// original line: ASTRewriteAnalyzer visitor= new ASTRewriteAnalyzer(document, result, this.eventStore, this.nodeStore, options, sourceRangeComputer);
-			rootNode.accept(visitor); // throws IllegalArgumentException
-			
-			this.eventStore.revertMovedNodes();
+		if (rootNode == null) {
+			return new MultiTextEdit(); // no changes
 		}
+			
+		char[] content= document.get().toCharArray();
+		LineInformation lineInfo= LineInformation.create(document);
+		String lineDelim= TextUtilities.getDefaultLineDelimiter(document);
+		
+		ASTNode astRoot= rootNode.getRoot();
+		List commentNodes= astRoot instanceof CompilationUnit ? ((CompilationUnit) astRoot).getCommentList() : null;
+		return internalRewriteAST(content, lineInfo, lineDelim, commentNodes, options, rootNode);
+	}
+	
+	/**
+	 * Converts all modifications recorded by this rewriter into an object representing the the corresponding text
+	 * edits to the source of a {@link ITypeRoot} from which the AST was created from.
+	 * The type root's source itself is not modified by this method call.
+	 * <p>
+	 * Important: This API can only be used if the modified AST has been created from a
+	 * {@link ITypeRoot} with source. That means {@link ASTParser#setSource(ICompilationUnit)},
+	 * {@link ASTParser#setSource(IClassFile)} or {@link ASTParser#setSource(ITypeRoot)}
+	 * has been used when initializing the {@link ASTParser}. A {@link IllegalArgumentException} is thrown
+	 * otherwise. An {@link IllegalArgumentException} is also thrown when the type roots buffer does not correspond
+	 * anymore to the AST. Use {@link #rewriteAST(IDocument, Map)} for all ASTs created from other content.
+	 * </p>
+	 * <p>
+	 * For nodes in the original that are being replaced or deleted,
+	 * this rewriter computes the adjusted source ranges
+	 * by calling <code>getTargetSourceRangeComputer().computeSourceRange(node)</code>.
+	 * </p>
+	 * <p>
+	 * Calling this methods does not discard the modifications
+	 * on record. Subsequence modifications are added to the ones
+	 * already on record. If this method is called again later,
+	 * the resulting text edit object will accurately reflect
+	 * the net cumulative affect of all those changes.
+	 * </p>
+	 * 
+	 * @return text edit object describing the changes to the
+	 * document corresponding to the changes recorded by this rewriter
+	 * @throws JavaModelException A {@link JavaModelException} is thrown when
+	 * the underlying compilation units buffer could not be accessed.
+	 * @throws IllegalArgumentException An {@link IllegalArgumentException}
+	 * is thrown if the document passed does not correspond to the AST that is rewritten.
+	 * 
+	 * @since 3.2
+	 */
+	public TextEdit rewriteAST() throws JavaModelException, IllegalArgumentException {
+		ASTNode rootNode= getRootNode();
+		if (rootNode == null) {
+			return new MultiTextEdit(); // no changes
+		}
+		
+		ASTNode root= rootNode.getRoot();
+		if (!(root instanceof CompilationUnit)) {
+			throw new IllegalArgumentException("This API can only be used if the AST is created from a compilation unit or class file"); //$NON-NLS-1$
+		}
+		CompilationUnit astRoot= (CompilationUnit) root;
+		ITypeRoot typeRoot = astRoot.getTypeRoot();
+		if (typeRoot == null || typeRoot.getBuffer() == null) {
+			throw new IllegalArgumentException("This API can only be used if the AST is created from a compilation unit or class file"); //$NON-NLS-1$
+		}
+		
+		char[] content= typeRoot.getBuffer().getCharacters();
+		LineInformation lineInfo= LineInformation.create(astRoot);
+		String lineDelim= typeRoot.findRecommendedLineSeparator();
+		Map options= typeRoot.getJavaProject().getOptions(true);
+		
+		return internalRewriteAST(content, lineInfo, lineDelim, astRoot.getCommentList(), options, rootNode);
+	}
+	
+	private TextEdit internalRewriteAST(char[] content, LineInformation lineInfo, String lineDelim, List commentNodes, Map options, ASTNode rootNode) {
+		TextEdit result= new MultiTextEdit();
+		//validateASTNotModified(rootNode);
+		
+		TargetSourceRangeComputer sourceRangeComputer= getExtendedSourceRangeComputer();
+		this.eventStore.prepareMovedNodes(sourceRangeComputer);
+		
+		// AspectJ extension - ask the factory in ASTRewriteAnalyzer for a visitor rather than building it directly.
+		// ASTRewriteAnalyzer visitor= new ASTRewriteAnalyzer(content, lineInfo, lineDelim, result, this.eventStore, this.nodeStore, commentNodes, options, sourceRangeComputer);
+		ASTVisitor visitor = ASTRewriteAnalyzer.getAnalyzerVisitor(content, lineInfo, lineDelim, result, this.eventStore, this.nodeStore, commentNodes, options, sourceRangeComputer);
+		// End AspectJ Extension
+		
+		rootNode.accept(visitor); // throws IllegalArgumentException
+		
+		this.eventStore.revertMovedNodes();
 		return result;
 	}
 	
@@ -304,9 +378,9 @@ public class ASTRewrite {
      * (not part of the original AST) or a placeholder node (for example, one
      * created by {@link #createCopyTarget(ASTNode)}
 	 * or {@link #createStringPlaceholder(String, int)}); or it must be
-	 * </code>null</code>, indicating that the child should be deleted.
+	 * <code>null</code>, indicating that the child should be deleted.
 	 * If the given property is a simple property, the value must be the new
-	 * value (primitive types must be boxed) or </code>null</code>.
+	 * value (primitive types must be boxed) or <code>null</code>.
      * The AST itself is not actually modified in any way; rather, the rewriter
      * just records a note that this node has been changed in the specified way.
 	 * 
@@ -323,7 +397,7 @@ public class ASTRewrite {
 		if (node == null || property == null) {
 			throw new IllegalArgumentException();
 		}
-		validateIsInsideAST(node);
+		validateIsCorrectAST(node);
 		validatePropertyType(property, value);
 
 		NodeRewriteEvent nodeEvent= this.eventStore.getNodeEvent(node, property, true);
@@ -332,6 +406,30 @@ public class ASTRewrite {
 			this.eventStore.setEventEditGroup(nodeEvent, editGroup);
 		}
 	}
+	
+	/**
+	 * Returns the value of the given property as managed by this rewriter. If the property
+	 * has been removed, <code>null</code> is returned. If it has been replaced, the replacing value
+	 * is returned. If the property has not been changed yet, the original value is returned.
+	 * <p>
+	 * For child list properties use {@link ListRewrite#getRewrittenList()} to get access to the
+	 * rewritten nodes in a list. </p>
+	 * 
+	 * @param node the node
+	 * @param property the node's property
+	 * @return the value of the given property as managed by this rewriter
+	 * 
+	 * @since 3.2
+	 */
+	public Object get(ASTNode node, StructuralPropertyDescriptor property) {
+		if (node == null || property == null) {
+			throw new IllegalArgumentException();
+		}
+		if (property.isChildListProperty()) {
+			throw new IllegalArgumentException("Use the list rewriter to access nodes in a list"); //$NON-NLS-1$
+		}
+		return this.eventStore.getNewValue(node, property);
+	}
 
 	/**
 	 * Creates and returns a new rewriter for describing modifications to the
@@ -378,11 +476,13 @@ public class ASTRewrite {
 		return new TrackedNodePosition(group, node);
 	}	
 			
-	private void validateIsInsideAST(ASTNode node) {
+	private void validateIsExistingNode(ASTNode node) {
 		if (node.getStartPosition() == -1) {
 			throw new IllegalArgumentException("Node is not an existing node"); //$NON-NLS-1$
 		}
+	}
 	
+	private void validateIsCorrectAST(ASTNode node) {
 		if (node.getAST() != getAST()) {
 			throw new IllegalArgumentException("Node is not inside the AST"); //$NON-NLS-1$
 		}
@@ -402,12 +502,12 @@ public class ASTRewrite {
 		}
 //		if (node == null) {
 //			if (prop.isSimpleProperty() || (prop.isChildProperty() && ((ChildPropertyDescriptor) prop).isMandatory())) {
-//				String message= "Can not remove property " + prop.getId(); //$NON-NLS-1$
+//				String message= "Can not remove property " + prop.getId();
 //				throw new IllegalArgumentException(message);
 //			}
 //		} else {
 //			if (!prop.getNodeClass().isInstance(node)) {
-//				String message= node.getClass().getName() +  " is not a valid type for property " + prop.getId(); //$NON-NLS-1$
+//				String message= node.getClass().getName() +  " is not a valid type for property " + prop.getId();
 //				throw new IllegalArgumentException(message);
 //			}
 //		}
@@ -472,7 +572,8 @@ public class ASTRewrite {
 		if (node == null) {
 			throw new IllegalArgumentException();
 		}
-		validateIsInsideAST(node);
+		validateIsExistingNode(node);
+		validateIsCorrectAST(node);
 		CopySourceInfo info= getRewriteEventStore().markAsCopySource(node.getParent(), node.getLocationInParent(), node, isMove);
 	
 		ASTNode placeholder= getNodeStore().newPlaceholderNode(node.getNodeType());
@@ -519,7 +620,7 @@ public class ASTRewrite {
 
 	/**
 	 * Returns the extended source range computer for this AST rewriter.
-	 * The default value is a <code>new ExtendedSourceRangeComputer()</code>.
+	 * The default value is a <code>new TargetSourceRangeComputer()</code>.
 	 * 
 	 * @return an extended source range computer
 	 * @since 3.1
@@ -534,7 +635,7 @@ public class ASTRewrite {
 	
 	/**
 	 * Sets a custom target source range computer for this AST rewriter. This is advanced feature to modify how
-	 * comments are assotiated with nodes, which should be done only in special cases.
+	 * comments are associated with nodes, which should be done only in special cases.
 	 * 
 	 * @param computer a target source range computer,
 	 * or <code>null</code> to restore the default value of
