@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,6 +12,7 @@ package org.aspectj.org.eclipse.jdt.internal.compiler.ast;
 
 import org.aspectj.org.eclipse.jdt.internal.compiler.ASTVisitor;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.*;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.codegen.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.flow.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.*;
@@ -21,88 +22,111 @@ public class EqualExpression extends BinaryExpression {
 	public EqualExpression(Expression left, Expression right,int operator) {
 		super(left,right,operator);
 	}
-	public void checkNullComparison(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, FlowInfo initsWhenTrue, FlowInfo initsWhenFalse) {
+	private void checkNullComparison(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, FlowInfo initsWhenTrue, FlowInfo initsWhenFalse) {
 		
 		LocalVariableBinding local = this.left.localVariableBinding();
-		if (local != null) {
+		if (local != null && (local.type.tagBits & TagBits.IsBaseType) == 0) {
 			checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, right.nullStatus(flowInfo), this.left);
 		}
 		local = this.right.localVariableBinding();
-		if (local != null) {
+		if (local != null && (local.type.tagBits & TagBits.IsBaseType) == 0) {
 			checkVariableComparison(scope, flowContext, flowInfo, initsWhenTrue, initsWhenFalse, local, left.nullStatus(flowInfo), this.right);
 		}
 	}
 	private void checkVariableComparison(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, FlowInfo initsWhenTrue, FlowInfo initsWhenFalse, LocalVariableBinding local, int nullStatus, Expression reference) {
 		switch (nullStatus) {
 			case FlowInfo.NULL :
-				flowContext.recordUsingNullReference(scope, local, reference, FlowInfo.NULL, flowInfo);
 				if (((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
-					initsWhenTrue.markAsDefinitelyNull(local); // from thereon it is set
-					initsWhenFalse.markAsDefinitelyNonNull(local); // from thereon it is set
+					flowContext.recordUsingNullReference(scope, local, reference, 
+							FlowContext.CAN_ONLY_NULL_NON_NULL | FlowContext.IN_COMPARISON_NULL, flowInfo);
+					initsWhenTrue.markAsComparedEqualToNull(local); // from thereon it is set
+					initsWhenFalse.markAsComparedEqualToNonNull(local); // from thereon it is set
 				} else {
-					initsWhenTrue.markAsDefinitelyNonNull(local); // from thereon it is set
-					initsWhenFalse.markAsDefinitelyNull(local); // from thereon it is set
+					flowContext.recordUsingNullReference(scope, local, reference, 
+							FlowContext.CAN_ONLY_NULL_NON_NULL | FlowContext.IN_COMPARISON_NON_NULL, flowInfo);
+					initsWhenTrue.markAsComparedEqualToNonNull(local); // from thereon it is set
+					initsWhenFalse.markAsComparedEqualToNull(local); // from thereon it is set
 				}
 				break;
 			case FlowInfo.NON_NULL :
-				flowContext.recordUsingNullReference(scope, local, reference, FlowInfo.NON_NULL, flowInfo);
 				if (((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
-					initsWhenTrue.markAsDefinitelyNonNull(local); // from thereon it is set
+					flowContext.recordUsingNullReference(scope, local, reference, 
+							FlowContext.CAN_ONLY_NULL | FlowContext.IN_COMPARISON_NON_NULL, flowInfo);
+					initsWhenTrue.markAsComparedEqualToNonNull(local); // from thereon it is set
+				} else {
+					flowContext.recordUsingNullReference(scope, local, reference, 
+							FlowContext.CAN_ONLY_NULL | FlowContext.IN_COMPARISON_NULL, flowInfo);
 				}
 				break;
 		}	
+		// we do not impact enclosing try context because this kind of protection
+		// does not preclude the variable from being null in an enclosing scope
 	}
 	
 	public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
+		FlowInfo result;
 		if (((bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
-			if ((left.constant != NotAConstant) && (left.constant.typeID() == T_boolean)) {
+			if ((left.constant != Constant.NotAConstant) && (left.constant.typeID() == T_boolean)) {
 				if (left.constant.booleanValue()) { //  true == anything
 					//  this is equivalent to the right argument inits 
-					return right.analyseCode(currentScope, flowContext, flowInfo);
+					result = right.analyseCode(currentScope, flowContext, flowInfo);
 				} else { // false == anything
 					//  this is equivalent to the right argument inits negated
-					return right.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
+					result = right.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
 				}
-			}
-			if ((right.constant != NotAConstant) && (right.constant.typeID() == T_boolean)) {
+			} 
+			else if ((right.constant != Constant.NotAConstant) && (right.constant.typeID() == T_boolean)) {
 				if (right.constant.booleanValue()) { //  anything == true
-					//  this is equivalent to the right argument inits 
-					return left.analyseCode(currentScope, flowContext, flowInfo);
+					//  this is equivalent to the left argument inits 
+					result = left.analyseCode(currentScope, flowContext, flowInfo);
 				} else { // anything == false
 					//  this is equivalent to the right argument inits negated
-					return left.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
+					result = left.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
 				}
+			} 
+			else {
+				result = right.analyseCode(
+					currentScope, flowContext, 
+					left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits()).unconditionalInits();
 			}
-			return right.analyseCode(
-				currentScope, flowContext, 
-				left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits()).unconditionalInits();
 		} else { //NOT_EQUAL :
-			if ((left.constant != NotAConstant) && (left.constant.typeID() == T_boolean)) {
+			if ((left.constant != Constant.NotAConstant) && (left.constant.typeID() == T_boolean)) {
 				if (!left.constant.booleanValue()) { //  false != anything
 					//  this is equivalent to the right argument inits 
-					return right.analyseCode(currentScope, flowContext, flowInfo);
+					result = right.analyseCode(currentScope, flowContext, flowInfo);
 				} else { // true != anything
 					//  this is equivalent to the right argument inits negated
-					return right.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
+					result = right.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
 				}
 			}
-			if ((right.constant != NotAConstant) && (right.constant.typeID() == T_boolean)) {
+			else if ((right.constant != Constant.NotAConstant) && (right.constant.typeID() == T_boolean)) {
 				if (!right.constant.booleanValue()) { //  anything != false
 					//  this is equivalent to the right argument inits 
-					return left.analyseCode(currentScope, flowContext, flowInfo);
+					result = left.analyseCode(currentScope, flowContext, flowInfo);
 				} else { // anything != true
 					//  this is equivalent to the right argument inits negated
-					return left.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
+					result = left.analyseCode(currentScope, flowContext, flowInfo).asNegatedCondition();
 				}
+			} 
+			else {
+				result = right.analyseCode(
+					currentScope, flowContext, 
+					left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits()).
+					/* unneeded since we flatten it: asNegatedCondition(). */
+					unconditionalInits();
 			}
-			return right.analyseCode(
-				currentScope, flowContext, 
-				left.analyseCode(currentScope, flowContext, flowInfo).unconditionalInits()).asNegatedCondition().unconditionalInits();
 		}
+		if (result instanceof UnconditionalFlowInfo && 
+				(result.tagBits & FlowInfo.UNREACHABLE) == 0) { // the flow info is flat
+			result = FlowInfo.conditional(result.copy(), result.copy());
+			// TODO (maxime) check, reintroduced copy
+		}
+	  checkNullComparison(currentScope, flowContext, result, result.initsWhenTrue(), result.initsWhenFalse());
+	  return result;
 	}
 	
 	public final void computeConstant(TypeBinding leftType, TypeBinding rightType) {
-		if ((this.left.constant != NotAConstant) && (this.right.constant != NotAConstant)) {
+		if ((this.left.constant != Constant.NotAConstant) && (this.right.constant != Constant.NotAConstant)) {
 			this.constant =
 				Constant.computeConstantOperationEQUAL_EQUAL(
 					left.constant,
@@ -110,9 +134,9 @@ public class EqualExpression extends BinaryExpression {
 					right.constant,
 					rightType.id);
 			if (((this.bits & OperatorMASK) >> OperatorSHIFT) == NOT_EQUAL)
-				constant = Constant.fromValue(!constant.booleanValue());
+				constant = BooleanConstant.fromValue(!constant.booleanValue());
 		} else {
-			this.constant = NotAConstant;
+			this.constant = Constant.NotAConstant;
 			// no optimization for null == null
 		}
 	}
@@ -125,51 +149,29 @@ public class EqualExpression extends BinaryExpression {
 	 */
 	public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
 	
-		if (constant != NotAConstant) {
-			int pc = codeStream.position;
+		int pc = codeStream.position;
+		if (constant != Constant.NotAConstant) {
 			if (valueRequired) 
 				codeStream.generateConstant(constant, implicitConversion);
 			codeStream.recordPositionsFrom(pc, this.sourceStart);
 			return;
 		}
-		Label falseLabel;
-		bits |= OnlyValueRequiredMASK;
-		generateOptimizedBoolean(
-			currentScope, 
-			codeStream, 
-			null, 
-			falseLabel = new Label(codeStream), 
-			valueRequired);
-		if (falseLabel.hasForwardReferences()) {
-			if (valueRequired){
-				// comparison is TRUE 
-				codeStream.iconst_1();
-				if ((bits & ValueForReturnMASK) != 0){
-					codeStream.generateImplicitConversion(this.implicitConversion);
-					codeStream.generateReturnBytecode(this);
-					// comparison is FALSE
-					falseLabel.place();
-					codeStream.iconst_0();
-				} else {
-					Label endLabel = new Label(codeStream);
-					codeStream.goto_(endLabel);
-					codeStream.decrStackSize(1);
-					// comparison is FALSE
-					falseLabel.place();
-					codeStream.iconst_0();
-					endLabel.place();
-				}
-				codeStream.generateImplicitConversion(implicitConversion);
-			} else {
-				falseLabel.place();
-			}	
+		
+		if ((left.implicitConversion & COMPILE_TYPE_MASK) /*compile-time*/ == T_boolean) {
+			generateBooleanEqual(currentScope, codeStream, valueRequired);
+		} else {
+			generateNonBooleanEqual(currentScope, codeStream, valueRequired);
 		}
+		if (valueRequired) {
+			codeStream.generateImplicitConversion(implicitConversion);
+		}
+		codeStream.recordPositionsFrom(pc, this.sourceStart);
 	}
 	/**
 	 * Boolean operator code generation
 	 *	Optimized operations are: == and !=
 	 */
-	public void generateOptimizedBoolean(BlockScope currentScope, CodeStream codeStream, Label trueLabel, Label falseLabel, boolean valueRequired) {
+	public void generateOptimizedBoolean(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
 	
 		if (constant != Constant.NotAConstant) {
 			super.generateOptimizedBoolean(currentScope, codeStream, trueLabel, falseLabel, valueRequired);
@@ -189,20 +191,169 @@ public class EqualExpression extends BinaryExpression {
 			}
 		}
 	}
+
 	/**
 	 * Boolean generation for == with boolean operands
 	 *
 	 * Note this code does not optimize conditional constants !!!!
 	 */
-	public void generateOptimizedBooleanEqual(BlockScope currentScope, CodeStream codeStream, Label trueLabel, Label falseLabel, boolean valueRequired) {
+	public void generateBooleanEqual(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
+	
+		// optimized cases: <something equivalent to true> == x, <something equivalent to false> == x, 
+		// optimized cases: <something equivalent to false> != x, <something equivalent to true> != x, 
+		boolean isEqualOperator = ((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL;
+		Constant cst = left.optimizedBooleanConstant();
+		if (cst != Constant.NotAConstant) {
+			Constant rightCst = right.optimizedBooleanConstant();
+			if (rightCst != Constant.NotAConstant) {
+				// <something equivalent to true> == <something equivalent to true>, <something equivalent to false> != <something equivalent to true>
+				// <something equivalent to true> == <something equivalent to false>, <something equivalent to false> != <something equivalent to false>
+				left.generateCode(currentScope, codeStream, false);
+				right.generateCode(currentScope, codeStream, false);
+				if (valueRequired) {
+					boolean leftBool = cst.booleanValue();
+					boolean rightBool = rightCst.booleanValue();
+					if (isEqualOperator) {
+						if (leftBool == rightBool) {
+							codeStream.iconst_1();
+						} else {
+							codeStream.iconst_0();
+						}
+					} else {
+						if (leftBool != rightBool) {
+							codeStream.iconst_1();
+						} else {
+							codeStream.iconst_0();
+						}
+					}
+				}
+			} else if (cst.booleanValue() == isEqualOperator) {
+				// <something equivalent to true> == x, <something equivalent to false> != x
+				left.generateCode(currentScope, codeStream, false);
+				right.generateCode(currentScope, codeStream, valueRequired);
+			} else {
+				// <something equivalent to false> == x, <something equivalent to true> != x
+				if (valueRequired) {
+					BranchLabel falseLabel = new BranchLabel(codeStream);
+					left.generateCode(currentScope, codeStream, false);
+					right.generateOptimizedBoolean(currentScope, codeStream, null, falseLabel, valueRequired);
+					// comparison is TRUE 
+					codeStream.iconst_0();
+					if ((bits & IsReturnedValue) != 0){
+						codeStream.generateImplicitConversion(this.implicitConversion);
+						codeStream.generateReturnBytecode(this);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_1();
+					} else {
+						BranchLabel endLabel = new BranchLabel(codeStream);
+						codeStream.goto_(endLabel);
+						codeStream.decrStackSize(1);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_1();
+						endLabel.place();
+					}
+				} else {
+					left.generateCode(currentScope, codeStream, false);
+					right.generateCode(currentScope, codeStream, false);
+				}
+//				left.generateCode(currentScope, codeStream, false);
+//				right.generateCode(currentScope, codeStream, valueRequired);
+//				if (valueRequired) {
+//					codeStream.iconst_1();
+//					codeStream.ixor(); // negate
+//				}
+			}
+			return;
+		} 
+		cst = right.optimizedBooleanConstant();
+		if (cst != Constant.NotAConstant) {
+			if (cst.booleanValue() == isEqualOperator) {
+				// x == <something equivalent to true>, x != <something equivalent to false>
+				left.generateCode(currentScope, codeStream, valueRequired);
+				right.generateCode(currentScope, codeStream, false);
+			} else {
+				// x == <something equivalent to false>, x != <something equivalent to true>
+				if (valueRequired) {
+					BranchLabel falseLabel = new BranchLabel(codeStream);
+					left.generateOptimizedBoolean(currentScope, codeStream, null, falseLabel, valueRequired);
+					right.generateCode(currentScope, codeStream, false);
+					// comparison is TRUE 
+					codeStream.iconst_0();
+					if ((bits & IsReturnedValue) != 0){
+						codeStream.generateImplicitConversion(this.implicitConversion);
+						codeStream.generateReturnBytecode(this);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_1();
+					} else {
+						BranchLabel endLabel = new BranchLabel(codeStream);
+						codeStream.goto_(endLabel);
+						codeStream.decrStackSize(1);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_1();
+						endLabel.place();
+					}
+				} else {
+					left.generateCode(currentScope, codeStream, false);
+					right.generateCode(currentScope, codeStream, false);
+				}
+//				left.generateCode(currentScope, codeStream, valueRequired);
+//				right.generateCode(currentScope, codeStream, false);
+//				if (valueRequired) {
+//					codeStream.iconst_1();
+//					codeStream.ixor(); // negate
+//				}
+			}
+			return;
+		} 
+		// default case
+		left.generateCode(currentScope, codeStream, valueRequired);
+		right.generateCode(currentScope, codeStream, valueRequired);
+
+		if (valueRequired) {
+			if (isEqualOperator) {
+				BranchLabel falseLabel;
+				codeStream.if_icmpne(falseLabel = new BranchLabel(codeStream));
+				// comparison is TRUE 
+				codeStream.iconst_1();
+				if ((bits & IsReturnedValue) != 0){
+					codeStream.generateImplicitConversion(this.implicitConversion);
+					codeStream.generateReturnBytecode(this);
+					// comparison is FALSE
+					falseLabel.place();
+					codeStream.iconst_0();
+				} else {
+					BranchLabel endLabel = new BranchLabel(codeStream);
+					codeStream.goto_(endLabel);
+					codeStream.decrStackSize(1);
+					// comparison is FALSE
+					falseLabel.place();
+					codeStream.iconst_0();
+					endLabel.place();
+				}
+			} else {
+				codeStream.ixor();				
+			}
+		}
+	}
+	
+	/**
+	 * Boolean generation for == with boolean operands
+	 *
+	 * Note this code does not optimize conditional constants !!!!
+	 */
+	public void generateOptimizedBooleanEqual(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
 	
 		// optimized cases: true == x, false == x
-		if (left.constant != NotAConstant) {
+		if (left.constant != Constant.NotAConstant) {
 			boolean inline = left.constant.booleanValue();
 			right.generateOptimizedBoolean(currentScope, codeStream, (inline ? trueLabel : falseLabel), (inline ? falseLabel : trueLabel), valueRequired);
 			return;
 		} // optimized cases: x == true, x == false
-		if (right.constant != NotAConstant) {
+		if (right.constant != Constant.NotAConstant) {
 			boolean inline = right.constant.booleanValue();
 			left.generateOptimizedBoolean(currentScope, codeStream, (inline ? trueLabel : falseLabel), (inline ? falseLabel : trueLabel), valueRequired);
 			return;
@@ -232,11 +383,220 @@ public class EqualExpression extends BinaryExpression {
 	 * Boolean generation for == with non-boolean operands
 	 *
 	 */
-	public void generateOptimizedNonBooleanEqual(BlockScope currentScope, CodeStream codeStream, Label trueLabel, Label falseLabel, boolean valueRequired) {
+	public void generateNonBooleanEqual(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
+	
+		boolean isEqualOperator = ((this.bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL;
+		if (((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int) {
+			Constant cst;
+			if ((cst = left.constant) != Constant.NotAConstant && cst.intValue() == 0) {
+				// optimized case: 0 == x, 0 != x
+				right.generateCode(currentScope, codeStream, valueRequired);
+				if (valueRequired) {
+					BranchLabel falseLabel = new BranchLabel(codeStream);
+					if (isEqualOperator) {
+						codeStream.ifne(falseLabel);
+					} else {
+						codeStream.ifeq(falseLabel);
+					}
+					// comparison is TRUE 
+					codeStream.iconst_1();
+					if ((bits & IsReturnedValue) != 0){
+						codeStream.generateImplicitConversion(this.implicitConversion);
+						codeStream.generateReturnBytecode(this);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_0();
+					} else {
+						BranchLabel endLabel = new BranchLabel(codeStream);
+						codeStream.goto_(endLabel);
+						codeStream.decrStackSize(1);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_0();
+						endLabel.place();
+					}
+				}				
+				return;
+			}
+			if ((cst = right.constant) != Constant.NotAConstant && cst.intValue() == 0) {
+				// optimized case: x == 0, x != 0
+				left.generateCode(currentScope, codeStream, valueRequired);
+				if (valueRequired) {
+					BranchLabel falseLabel = new BranchLabel(codeStream);
+					if (isEqualOperator) {
+						codeStream.ifne(falseLabel);
+					} else {
+						codeStream.ifeq(falseLabel);
+					}
+					// comparison is TRUE 
+					codeStream.iconst_1();
+					if ((bits & IsReturnedValue) != 0){
+						codeStream.generateImplicitConversion(this.implicitConversion);
+						codeStream.generateReturnBytecode(this);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_0();
+					} else {
+						BranchLabel endLabel = new BranchLabel(codeStream);
+						codeStream.goto_(endLabel);
+						codeStream.decrStackSize(1);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_0();
+						endLabel.place();
+					}
+				}
+				return;
+			}
+		}
+
+		// null cases
+		if (right instanceof NullLiteral) {
+			if (left instanceof NullLiteral) {
+				// null == null, null != null
+				if (valueRequired) {
+					if (isEqualOperator) {
+						codeStream.iconst_1();
+					} else {
+						codeStream.iconst_0();
+					}
+				}
+			} else {
+				// x == null, x != null
+				left.generateCode(currentScope, codeStream, valueRequired);
+				if (valueRequired) {
+					BranchLabel falseLabel = new BranchLabel(codeStream);
+					if (isEqualOperator) {
+						codeStream.ifnonnull(falseLabel);
+					} else {
+						codeStream.ifnull(falseLabel);
+					}
+					// comparison is TRUE 
+					codeStream.iconst_1();
+					if ((bits & IsReturnedValue) != 0){
+						codeStream.generateImplicitConversion(this.implicitConversion);
+						codeStream.generateReturnBytecode(this);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_0();
+					} else {
+						BranchLabel endLabel = new BranchLabel(codeStream);
+						codeStream.goto_(endLabel);
+						codeStream.decrStackSize(1);
+						// comparison is FALSE
+						falseLabel.place();
+						codeStream.iconst_0();
+						endLabel.place();
+					}
+				}
+			}
+			return;
+		} else if (left instanceof NullLiteral) {
+			// null = x, null != x
+			right.generateCode(currentScope, codeStream, valueRequired);
+			if (valueRequired) {
+				BranchLabel falseLabel = new BranchLabel(codeStream);
+				if (isEqualOperator) {
+					codeStream.ifnonnull(falseLabel);
+				} else {
+					codeStream.ifnull(falseLabel);
+				}
+				// comparison is TRUE 
+				codeStream.iconst_1();
+				if ((bits & IsReturnedValue) != 0){
+					codeStream.generateImplicitConversion(this.implicitConversion);
+					codeStream.generateReturnBytecode(this);
+					// comparison is FALSE
+					falseLabel.place();
+					codeStream.iconst_0();
+				} else {
+					BranchLabel endLabel = new BranchLabel(codeStream);
+					codeStream.goto_(endLabel);
+					codeStream.decrStackSize(1);
+					// comparison is FALSE
+					falseLabel.place();
+					codeStream.iconst_0();
+					endLabel.place();
+				}
+			}				
+			return;
+		}
+	
+		// default case
+		left.generateCode(currentScope, codeStream, valueRequired);
+		right.generateCode(currentScope, codeStream, valueRequired);
+		if (valueRequired) {
+			BranchLabel falseLabel = new BranchLabel(codeStream);
+			if (isEqualOperator) {
+				switch ((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
+					case T_int :
+						codeStream.if_icmpne(falseLabel);
+						break;
+					case T_float :
+						codeStream.fcmpl();
+						codeStream.ifne(falseLabel);
+						break;
+					case T_long :
+						codeStream.lcmp();
+						codeStream.ifne(falseLabel);
+						break;
+					case T_double :
+						codeStream.dcmpl();
+						codeStream.ifne(falseLabel);
+						break;
+					default :
+						codeStream.if_acmpne(falseLabel);
+				}
+			} else {
+				switch ((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) { // operand runtime type
+					case T_int :
+						codeStream.if_icmpeq(falseLabel);
+						break;
+					case T_float :
+						codeStream.fcmpl();
+						codeStream.ifeq(falseLabel);
+						break;
+					case T_long :
+						codeStream.lcmp();
+						codeStream.ifeq(falseLabel);
+						break;
+					case T_double :
+						codeStream.dcmpl();
+						codeStream.ifeq(falseLabel);
+						break;
+					default :
+						codeStream.if_acmpeq(falseLabel);
+				}				
+			}
+			// comparison is TRUE 
+			codeStream.iconst_1();
+			if ((bits & IsReturnedValue) != 0){
+				codeStream.generateImplicitConversion(this.implicitConversion);
+				codeStream.generateReturnBytecode(this);
+				// comparison is FALSE
+				falseLabel.place();
+				codeStream.iconst_0();
+			} else {
+				BranchLabel endLabel = new BranchLabel(codeStream);
+				codeStream.goto_(endLabel);
+				codeStream.decrStackSize(1);
+				// comparison is FALSE
+				falseLabel.place();
+				codeStream.iconst_0();
+				endLabel.place();
+			}			
+		}
+	}
+	
+	/**
+	 * Boolean generation for == with non-boolean operands
+	 *
+	 */
+	public void generateOptimizedNonBooleanEqual(BlockScope currentScope, CodeStream codeStream, BranchLabel trueLabel, BranchLabel falseLabel, boolean valueRequired) {
 	
 		int pc = codeStream.position;
 		Constant inline;
-		if ((inline = right.constant) != NotAConstant) {
+		if ((inline = right.constant) != Constant.NotAConstant) {
 			// optimized case: x == 0
 			if ((((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int) && (inline.intValue() == 0)) {
 				left.generateCode(currentScope, codeStream, valueRequired);
@@ -259,7 +619,7 @@ public class EqualExpression extends BinaryExpression {
 				return;
 			}
 		}
-		if ((inline = left.constant) != NotAConstant) {
+		if ((inline = left.constant) != Constant.NotAConstant) {
 			// optimized case: 0 == x
 			if ((((left.implicitConversion & IMPLICIT_CONVERSION_MASK) >> 4) == T_int)
 				&& (inline.intValue() == 0)) {
@@ -289,19 +649,11 @@ public class EqualExpression extends BinaryExpression {
 			if (left instanceof NullLiteral) {
 				// null == null
 				if (valueRequired) {
-						if ((bits & OnlyValueRequiredMASK) != 0) {
-							if (((bits & OperatorMASK) >> OperatorSHIFT) == EQUAL_EQUAL) {
-								codeStream.iconst_1();
-							} else {
-								codeStream.iconst_0();
-							}
-						} else {
-							if (falseLabel == null) {
-								// implicit falling through the FALSE case
-								if (trueLabel != null) {
-									codeStream.goto_(trueLabel);
-								}
-							}
+					if (falseLabel == null) {
+						// implicit falling through the FALSE case
+						if (trueLabel != null) {
+							codeStream.goto_(trueLabel);
+						}
 					}
 				}
 			} else {
@@ -407,28 +759,28 @@ public class EqualExpression extends BinaryExpression {
 	public TypeBinding resolveType(BlockScope scope) {
 	
 			boolean leftIsCast, rightIsCast;
-			if ((leftIsCast = left instanceof CastExpression) == true) left.bits |= IgnoreNeedForCastCheckMASK; // will check later on
+			if ((leftIsCast = left instanceof CastExpression) == true) left.bits |= DisableUnnecessaryCastCheck; // will check later on
 			TypeBinding originalLeftType = left.resolveType(scope);
 	
-			if ((rightIsCast = right instanceof CastExpression) == true) right.bits |= IgnoreNeedForCastCheckMASK; // will check later on
+			if ((rightIsCast = right instanceof CastExpression) == true) right.bits |= DisableUnnecessaryCastCheck; // will check later on
 			TypeBinding originalRightType = right.resolveType(scope);
 	
 		// always return BooleanBinding
 		if (originalLeftType == null || originalRightType == null){
-			constant = NotAConstant;		
+			constant = Constant.NotAConstant;		
 			return null;
 		}
 	
 		// autoboxing support
-		boolean use15specifics = scope.compilerOptions().sourceLevel >= JDK1_5;
+		boolean use15specifics = scope.compilerOptions().sourceLevel >= ClassFileConstants.JDK1_5;
 		TypeBinding leftType = originalLeftType, rightType = originalRightType;
 		if (use15specifics) {
-			if (leftType != NullBinding && leftType.isBaseType()) {
+			if (leftType != TypeBinding.NULL && leftType.isBaseType()) {
 				if (!rightType.isBaseType()) {
 					rightType = scope.environment().computeBoxingType(rightType);
 				}
 			} else {
-				if (rightType != NullBinding && rightType.isBaseType()) {
+				if (rightType != TypeBinding.NULL && rightType.isBaseType()) {
 					leftType = scope.environment().computeBoxingType(leftType);
 				}
 			}
@@ -456,13 +808,13 @@ public class EqualExpression extends BinaryExpression {
 				CastExpression.checkNeedForArgumentCasts(scope, EQUAL_EQUAL, operatorSignature, left, leftType.id, leftIsCast, right, rightType.id, rightIsCast);
 			}
 			computeConstant(leftType, rightType);
-			return this.resolvedType = BooleanBinding;
+			return this.resolvedType = TypeBinding.BOOLEAN;
 		}
 	
 		// Object references 
 		// spec 15.20.3
-		if ((!leftType.isBaseType() || leftType == NullBinding) // cannot compare: Object == (int)0
-				&& (!rightType.isBaseType() || rightType == NullBinding)
+		if ((!leftType.isBaseType() || leftType == TypeBinding.NULL) // cannot compare: Object == (int)0
+				&& (!rightType.isBaseType() || rightType == TypeBinding.NULL)
 				&& (this.checkCastTypesCompatibility(scope, leftType, rightType, null) 
 						|| this.checkCastTypesCompatibility(scope, rightType, leftType, null))) {
 
@@ -470,14 +822,14 @@ public class EqualExpression extends BinaryExpression {
 			if ((rightType.id == T_JavaLangString) && (leftType.id == T_JavaLangString)) {
 				computeConstant(leftType, rightType);
 			} else {
-				constant = NotAConstant;
+				constant = Constant.NotAConstant;
 			}
 			TypeBinding objectType = scope.getJavaLangObject();
 			left.computeConversion(scope, objectType, leftType);
 			right.computeConversion(scope, objectType, rightType);
 			// check need for operand cast
-			boolean unnecessaryLeftCast = (left.bits & UnnecessaryCastMASK) != 0;
-			boolean unnecessaryRightCast = (right.bits & UnnecessaryCastMASK) != 0;
+			boolean unnecessaryLeftCast = (left.bits & UnnecessaryCast) != 0;
+			boolean unnecessaryRightCast = (right.bits & UnnecessaryCast) != 0;
 			if (unnecessaryLeftCast || unnecessaryRightCast) {
 				TypeBinding alternateLeftType = unnecessaryLeftCast ? ((CastExpression)left).expression.resolvedType : leftType;
 				TypeBinding alternateRightType = unnecessaryRightCast ? ((CastExpression)right).expression.resolvedType : rightType;
@@ -487,9 +839,9 @@ public class EqualExpression extends BinaryExpression {
 					if (unnecessaryRightCast) scope.problemReporter().unnecessaryCast((CastExpression)right);
 				}
 			}
-			return this.resolvedType = BooleanBinding;
+			return this.resolvedType = TypeBinding.BOOLEAN;
 		}
-		constant = NotAConstant;
+		constant = Constant.NotAConstant;
 		scope.problemReporter().notCompatibleTypesError(this, leftType, rightType);
 		return null;
 	}
