@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,6 +13,7 @@ package org.aspectj.org.eclipse.jdt.internal.compiler.ast;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ASTVisitor;
 import org.aspectj.org.eclipse.jdt.internal.compiler.codegen.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.flow.*;
+import org.aspectj.org.eclipse.jdt.internal.compiler.impl.Constant;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.*;
 
 public class InstanceOfExpression extends OperatorExpression {
@@ -20,29 +21,34 @@ public class InstanceOfExpression extends OperatorExpression {
 	public Expression expression;
 	public TypeReference type;
 
-	public InstanceOfExpression(
-		Expression expression,
-		TypeReference type,
-		int operator) {
+	public InstanceOfExpression(Expression expression, TypeReference type) {
 
 		this.expression = expression;
 		this.type = type;
-		this.bits |= operator << OperatorSHIFT;
+		type.bits |= IgnoreRawTypeCheck; // no need to worry about raw type usage
+		this.bits |= INSTANCEOF << OperatorSHIFT;
 		this.sourceStart = expression.sourceStart;
 		this.sourceEnd = type.sourceEnd;
 	}
 
-	public FlowInfo analyseCode(
+public FlowInfo analyseCode(
 		BlockScope currentScope,
 		FlowContext flowContext,
 		FlowInfo flowInfo) {
-
-		flowInfo = expression
-			.analyseCode(currentScope, flowContext, flowInfo)
-			.unconditionalInits();
-		expression.checkNullStatus(currentScope, flowContext, flowInfo, FlowInfo.NON_NULL);
-		return flowInfo;
+	LocalVariableBinding local = this.expression.localVariableBinding();
+	if (local != null && (local.type.tagBits & TagBits.IsBaseType) == 0) {
+		flowContext.recordUsingNullReference(currentScope, local, 
+			this.expression, FlowContext.CAN_ONLY_NULL | FlowContext.IN_INSTANCEOF, flowInfo);
+		flowInfo = expression.analyseCode(currentScope, flowContext, flowInfo).
+			unconditionalInits();
+		FlowInfo initsWhenTrue = flowInfo.copy();
+		initsWhenTrue.markAsComparedEqualToNonNull(local);
+		// no impact upon enclosing try context
+		return FlowInfo.conditional(initsWhenTrue, flowInfo.copy());
 	}
+	return expression.analyseCode(currentScope, flowContext, flowInfo).
+			unconditionalInits();
+}
 
 	/**
 	 * Code generation for instanceOfExpression
@@ -75,28 +81,26 @@ public class InstanceOfExpression extends OperatorExpression {
 
 	public TypeBinding resolveType(BlockScope scope) {
 
-		constant = NotAConstant;
+		constant = Constant.NotAConstant;
 		TypeBinding expressionType = expression.resolveType(scope);
 		TypeBinding checkedType = type.resolveType(scope, true /* check bounds*/);
 		if (expressionType == null || checkedType == null)
 			return null;
 
-		if (checkedType.isTypeVariable() || checkedType.isBoundParameterizedType() || checkedType.isGenericType()) {
+		if (!checkedType.isReifiable()) {
 			scope.problemReporter().illegalInstanceOfGenericType(checkedType, this);
-		} else {
-			boolean isLegal = checkCastTypesCompatibility(scope, checkedType, expressionType, null);
-			if (!isLegal) {
-				scope.problemReporter().notCompatibleTypesError(this, expressionType, checkedType);
-			}
+		} else if ((expressionType != TypeBinding.NULL && expressionType.isBaseType()) // disallow autoboxing
+				|| !checkCastTypesCompatibility(scope, checkedType, expressionType, null)) {
+			scope.problemReporter().notCompatibleTypesError(this, expressionType, checkedType);
 		}
-		return this.resolvedType = BooleanBinding;
+		return this.resolvedType = TypeBinding.BOOLEAN;
 	}
 	/**
 	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.ast.Expression#tagAsUnnecessaryCast(Scope,TypeBinding)
 	 */
 	public void tagAsUnnecessaryCast(Scope scope, TypeBinding castType) {
 		// null is not instanceof Type, recognize direct scenario
-		if (expression.resolvedType != NullBinding)
+		if (expression.resolvedType != TypeBinding.NULL)
 			scope.problemReporter().unnecessaryInstanceof(this, castType);
 	}
 	public void traverse(ASTVisitor visitor, BlockScope scope) {
