@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,6 +12,7 @@ package org.aspectj.org.eclipse.jdt.internal.compiler.ast;
 
 import org.aspectj.org.eclipse.jdt.internal.compiler.ASTVisitor;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.*;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.codegen.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.flow.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.*;
@@ -31,25 +32,26 @@ public class LocalDeclaration extends AbstractVariableDeclaration {
 		this.declarationEnd = sourceEnd;
 	}
 
-	public FlowInfo analyseCode(
-		BlockScope currentScope,
-		FlowContext flowContext,
-		FlowInfo flowInfo) {
-
-		// record variable initialization if any
-		if (flowInfo.isReachable()) {
-			bits |= IsLocalDeclarationReachableMASK; // only set if actually reached
-		}
-		if (this.initialization == null) 
-			return flowInfo;
-			
-		int nullStatus = this.initialization.nullStatus(flowInfo);
-		flowInfo =
-			this.initialization
-				.analyseCode(currentScope, flowContext, flowInfo)
-				.unconditionalInits();
-		
-		flowInfo.markAsDefinitelyAssigned(binding);
+public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
+	// record variable initialization if any
+	if ((flowInfo.tagBits & FlowInfo.UNREACHABLE) == 0) {
+		bits |= ASTNode.IsLocalDeclarationReachable; // only set if actually reached
+	}
+	if (this.initialization == null) { 
+		return flowInfo;
+	}
+	int nullStatus = this.initialization.nullStatus(flowInfo);
+	flowInfo =
+		this.initialization
+			.analyseCode(currentScope, flowContext, flowInfo)
+			.unconditionalInits();
+	if (!flowInfo.isDefinitelyAssigned(this.binding)){// for local variable debug attributes
+		this.bits |= FirstAssignmentToLocal;
+	} else {
+		this.bits &= ~FirstAssignmentToLocal;  // int i = (i = 0);
+	}	
+	flowInfo.markAsDefinitelyAssigned(binding);
+	if ((this.binding.type.tagBits & TagBits.IsBaseType) == 0) {
 		switch(nullStatus) {
 			case FlowInfo.NULL :
 				flowInfo.markAsDefinitelyNull(this.binding);
@@ -57,19 +59,24 @@ public class LocalDeclaration extends AbstractVariableDeclaration {
 			case FlowInfo.NON_NULL :
 				flowInfo.markAsDefinitelyNonNull(this.binding);
 				break;
+			default:
+				flowInfo.markAsDefinitelyUnknown(this.binding);
 		}
-		return flowInfo;
+		// no need to inform enclosing try block since its locals won't get
+		// known by the finally block
 	}
+	return flowInfo;
+}
 
 	public void checkModifiers() {
 
 		//only potential valid modifier is <<final>>
-		if (((modifiers & AccJustFlag) & ~AccFinal) != 0)
+		if (((modifiers & ExtraCompilerModifiers.AccJustFlag) & ~ClassFileConstants.AccFinal) != 0)
 			//AccModifierProblem -> other (non-visibility problem)
 			//AccAlternateModifierProblem -> duplicate modifier
 			//AccModifierProblem | AccAlternateModifierProblem -> visibility problem"
 
-			modifiers = (modifiers & ~AccAlternateModifierProblem) | AccModifierProblem;
+			modifiers = (modifiers & ~ExtraCompilerModifiers.AccAlternateModifierProblem) | ExtraCompilerModifiers.AccModifierProblem;
 	}
 
 	/**
@@ -82,53 +89,47 @@ public class LocalDeclaration extends AbstractVariableDeclaration {
 		if (binding.resolvedPosition != -1) {
 			codeStream.addVisibleLocalVariable(binding);
 		}
-		if ((bits & IsReachableMASK) == 0) {
+		if ((bits & IsReachable) == 0) {
 			return;
 		}
 		int pc = codeStream.position;
-		Constant inlinedValue;
 
 		// something to initialize?
-		if (initialization != null) {
-			// initialize to constant value?
-			if ((inlinedValue = initialization.constant) != NotAConstant) {
-				// forget initializing unused or final locals set to constant value (final ones are inlined)
-				if (binding.resolvedPosition != -1) { // may need to preserve variable
-					int initPC = codeStream.position;
-					codeStream.generateConstant(inlinedValue, initialization.implicitConversion);
-					codeStream.recordPositionsFrom(initPC, initialization.sourceStart);
-					codeStream.store(binding, false);
-					binding.recordInitializationStartPC(codeStream.position);
-					//				codeStream.lastInitStateIndexWhenRemovingInits = -2; // reinitialize remove index 
-					//				codeStream.lastInitStateIndexWhenAddingInits = -2; // reinitialize add index		
-				}
-			} else { // initializing to non-constant value
-				initialization.generateCode(currentScope, codeStream, true);
+		generateInit: {
+			if (this.initialization == null) 
+				break generateInit;
+			// forget initializing unused or final locals set to constant value (final ones are inlined)
+			if (binding.resolvedPosition < 0) {
+				// New AspectJ Extension
+				// old code:
+				//if (initialization.constant != Constant.NotAConstant)
+				//	break generateInit;
+				// same code:
 				// if binding unused generate then discard the value
-				if (binding.resolvedPosition != -1) {
-					// 26903, need extra cast to store null in array local var	
-					if (binding.type.isArrayType() 
-						&& (initialization.resolvedType == NullBinding	// arrayLoc = null
-							|| ((initialization instanceof CastExpression)	// arrayLoc = (type[])null
-								&& (((CastExpression)initialization).innermostCastedExpression().resolvedType == NullBinding)))){
-						codeStream.checkcast(binding.type); 
-					}					
-					codeStream.store(binding, false);
-					if (binding.initializationCount == 0) {
-						/* Variable may have been initialized during the code initializing it
-							e.g. int i = (i = 1);
-						*/
-						binding.recordInitializationStartPC(codeStream.position);
-						//					codeStream.lastInitStateIndexWhenRemovingInits = -2; // reinitialize remove index 
-						//					codeStream.lastInitStateIndexWhenAddingInits = -2; // reinitialize add index 
-					}
-				} else {
-					if ((binding.type == LongBinding) || (binding.type == DoubleBinding)) {
+				initialization.generateCode(currentScope, codeStream, true);
+				// new code:
+					if ((binding.type == TypeBinding.LONG) || (binding.type == TypeBinding.DOUBLE)) {
 						codeStream.pop2();
 					} else {
 						codeStream.pop();
 					}
-				}
+				// End AspectJ Extension
+				break generateInit;
+			}
+			initialization.generateCode(currentScope, codeStream, true);
+			// 26903, need extra cast to store null in array local var	
+			if (binding.type.isArrayType() 
+				&& (initialization.resolvedType == TypeBinding.NULL	// arrayLoc = null
+					|| ((initialization instanceof CastExpression)	// arrayLoc = (type[])null
+						&& (((CastExpression)initialization).innermostCastedExpression().resolvedType == TypeBinding.NULL)))){
+				codeStream.checkcast(binding.type); 
+			}					
+			codeStream.store(binding, false);
+			if ((this.bits & ASTNode.FirstAssignmentToLocal) != 0) {
+				/* Variable may have been initialized during the code initializing it
+					e.g. int i = (i = 1);
+				*/
+				binding.recordInitializationStartPC(codeStream.position);
 			}
 		}
 		codeStream.recordPositionsFrom(pc, this.sourceStart);
@@ -148,37 +149,33 @@ public class LocalDeclaration extends AbstractVariableDeclaration {
 
 		checkModifiers();
 		if (variableType != null) {
-			if (variableType == VoidBinding) {
+			if (variableType == TypeBinding.VOID) {
 				scope.problemReporter().variableTypeCannotBeVoid(this);
 				return;
 			}
-			if (variableType.isArrayType() && ((ArrayBinding) variableType).leafComponentType == VoidBinding) {
+			if (variableType.isArrayType() && ((ArrayBinding) variableType).leafComponentType == TypeBinding.VOID) {
 				scope.problemReporter().variableTypeCannotBeVoidArray(this);
 				return;
 			}
 		}
 		
 		Binding existingVariable = scope.getBinding(name, Binding.VARIABLE, this, false /*do not resolve hidden field*/);
-		boolean shouldInsertInScope = true;
 		if (existingVariable != null && existingVariable.isValidBinding()){
 			if (existingVariable instanceof LocalVariableBinding && this.hiddenVariableDepth == 0) {
-				shouldInsertInScope = false;
 				scope.problemReporter().redefineLocal(this);
 			} else {
 				scope.problemReporter().localVariableHiding(this, existingVariable, false);
 			}
 		}
 				
-		if (shouldInsertInScope) {
-			if ((modifiers & AccFinal)!= 0 && this.initialization == null) {
-				modifiers |= AccBlankFinal;
-			}
-			this.binding = new LocalVariableBinding(this, variableType, modifiers, false);
-			scope.addLocalVariable(binding);
-			this.binding.setConstant(NotAConstant);
-			// allow to recursivelly target the binding....
-			// the correct constant is harmed if correctly computed at the end of this method
+		if ((modifiers & ClassFileConstants.AccFinal)!= 0 && this.initialization == null) {
+			modifiers |= ExtraCompilerModifiers.AccBlankFinal;
 		}
+		this.binding = new LocalVariableBinding(this, variableType, modifiers, false);
+		scope.addLocalVariable(binding);
+		this.binding.setConstant(Constant.NotAConstant);
+		// allow to recursivelly target the binding....
+		// the correct constant is harmed if correctly computed at the end of this method
 
 		if (variableType == null) {
 			if (initialization != null)
@@ -207,36 +204,49 @@ public class LocalDeclaration extends AbstractVariableDeclaration {
 						if (initializationType.needsUncheckedConversion(variableType)) {
 						    scope.problemReporter().unsafeTypeConversion(this.initialization, initializationType, variableType);
 						}						
+						if (this.initialization instanceof CastExpression 
+								&& (this.initialization.bits & ASTNode.UnnecessaryCast) == 0) {
+							CastExpression.checkNeedForAssignedCast(scope, variableType, (CastExpression) this.initialization);
+						}	
 					} else if (scope.isBoxingCompatibleWith(initializationType, variableType) 
 										|| (initializationType.isBaseType()  // narrowing then boxing ?
-												&& scope.compilerOptions().sourceLevel >= JDK1_5 // autoboxing
+												&& scope.compilerOptions().sourceLevel >= ClassFileConstants.JDK1_5 // autoboxing
 												&& !variableType.isBaseType()
 												&& initialization.isConstantValueOfTypeAssignableToType(initializationType, scope.environment().computeBoxingType(variableType)))) {
 						this.initialization.computeConversion(scope, variableType, initializationType);
+						if (this.initialization instanceof CastExpression 
+								&& (this.initialization.bits & ASTNode.UnnecessaryCast) == 0) {
+							CastExpression.checkNeedForAssignedCast(scope, variableType, (CastExpression) this.initialization);
+						}	
 					} else {
-						scope.problemReporter().typeMismatchError(initializationType, variableType, this);
+						scope.problemReporter().typeMismatchError(initializationType, variableType, this.initialization);
 					}
 				}
 			}
-
+			// check for assignment with no effect
+			if (this.binding == Assignment.getDirectBinding(this.initialization)) {
+				scope.problemReporter().assignmentHasNoEffect(this, this.name);
+			}
 			// change the constant in the binding when it is final
 			// (the optimization of the constant propagation will be done later on)
 			// cast from constant actual type to variable type
-			if (binding != null) {
-				binding.setConstant(
-					binding.isFinal()
-						? initialization.constant.castTo((variableType.id << 4) + initialization.constant.typeID())
-						: NotAConstant);
-			}
+			binding.setConstant(
+				binding.isFinal()
+					? initialization.constant.castTo((variableType.id << 4) + initialization.constant.typeID())
+					: Constant.NotAConstant);
 		}
 		// only resolve annotation at the end, for constant to be positionned before (96991)
-		if (this.binding != null)
-			resolveAnnotations(scope, this.annotations, this.binding);
+		resolveAnnotations(scope, this.annotations, this.binding);
 	}
 
 	public void traverse(ASTVisitor visitor, BlockScope scope) {
 
 		if (visitor.visit(this, scope)) {
+			if (this.annotations != null) {
+				int annotationsLength = this.annotations.length;
+				for (int i = 0; i < annotationsLength; i++)
+					this.annotations[i].traverse(visitor, scope);
+			}
 			type.traverse(visitor, scope);
 			if (initialization != null)
 				initialization.traverse(visitor, scope);
