@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,8 +11,12 @@
 package org.aspectj.org.eclipse.jdt.internal.compiler.impl;
 
 public class ShortConstant extends Constant {
-	short value;
-public ShortConstant(short value) {
+private short value;
+
+public static Constant fromValue(short value) {
+	return new ShortConstant(value);
+}
+private ShortConstant(short value) {
 	this.value = value;
 }
 public byte byteValue() {
@@ -38,10 +42,7 @@ public short shortValue() {
 }
 public String stringValue() {
 	//spec 15.17.11
-	
-	String s = new Integer(value).toString() ;
-	if (s == null) return "null"; //$NON-NLS-1$
-	return s;
+	return String.valueOf(this.value);
 }
 public String toString(){
 
