@@ -10,21 +10,24 @@
  ******************************************************************************/
 
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
+
 //AspectJ Extension
 
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.ASTNode;
 
-
 /**
- * This interface is used by SourceTypeBinding to provide a delegated lookup
- * instance. It is used to support AspectJ's inter-type declarations.
+ * This interface is used by SourceTypeBinding to provide a delegated lookup instance. It is used to support AspectJ's inter-type
+ * declarations.
  * 
  * These methods are equivalent to those of the same names and sigs in SourceTypeBinding.
  */
 public interface IPrivilegedHandler {
-	
+
 	FieldBinding getPrivilegedAccessField(FieldBinding baseField, ASTNode location);
-	
+
+	boolean definesPrivilegedAccessToField(FieldBinding field);
+
 	MethodBinding getPrivilegedAccessMethod(MethodBinding baseMethod, ASTNode location);
+
 	void notePrivilegedTypeAccess(ReferenceBinding type, ASTNode location);
 }
