@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -80,6 +80,13 @@ public interface TypeIds {
 
 	final int T_JavaIoPrintStream = 53;
 
+	final int T_JavaLangReflectField = 54;
+	final int T_JavaLangReflectMethod = 55;
+
+	final int T_JavaIoExternalizable = 56;
+	final int T_JavaIoObjectStreamException = 57;
+	final int T_JavaIoException = 58;
+
 	final int NoId = Integer.MAX_VALUE;
 
 	public static final int IMPLICIT_CONVERSION_MASK = 0xFF;
