@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,13 +10,14 @@
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.core;
 
+import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.Map;
 import java.util.Stack;
 
+import org.eclipse.core.runtime.Assert;
 import org.aspectj.org.eclipse.jdt.core.*;
-import org.aspectj.org.eclipse.jdt.core.ICompilationUnit;
-import org.aspectj.org.eclipse.jdt.core.IJavaElement;
-import org.aspectj.org.eclipse.jdt.core.Signature;
+import org.aspectj.org.eclipse.jdt.core.compiler.CategorizedProblem;
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
 import org.aspectj.org.eclipse.jdt.core.compiler.IProblem;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ISourceElementRequestor;
@@ -59,6 +60,12 @@ public class CompilationUnitStructureRequestor extends ReferenceInfoAdapter impl
 	 * will be the type the method is contained in.
 	 */
 	protected Stack infoStack;
+	
+	/*
+	 * Map from JavaElementInfo to of ArrayList of IJavaElement representing the children 
+	 * of the given info.
+	 */
+	protected HashMap children;
 
 	/**
 	 * Stack of parent handles, corresponding to the info stack. We
@@ -86,7 +93,6 @@ public class CompilationUnitStructureRequestor extends ReferenceInfoAdapter impl
 	/**
 	 * Empty collections used for efficient initialization
 	 */
-	protected static String[] NO_STRINGS = new String[0];
 	protected static byte[] NO_BYTES= new byte[]{};
 
 	protected HashtableOfObject fieldRefCache;
@@ -102,8 +108,7 @@ protected CompilationUnitStructureRequestor(ICompilationUnit unit, CompilationUn
 /**
  * @see ISourceElementRequestor
  */
-public void acceptImport(int declarationStart, int declarationEnd, char[] name, boolean onDemand, int modifiers) {
-	JavaElementInfo parentInfo = (JavaElementInfo) this.infoStack.peek();
+public void acceptImport(int declarationStart, int declarationEnd, char[][] tokens, boolean onDemand, int modifiers) {
 	JavaElement parentHandle= (JavaElement) this.handleStack.peek();
 	if (!(parentHandle.getElementType() == IJavaElement.COMPILATION_UNIT)) {
 		Assert.isTrue(false); // Should not happen
@@ -113,12 +118,13 @@ public void acceptImport(int declarationStart, int declarationEnd, char[] name,
 	//create the import container and its info
 	ImportContainer importContainer= (ImportContainer)parentCU.getImportContainer();
 	if (this.importContainerInfo == null) {
-		this.importContainerInfo= new JavaElementInfo();
-		parentInfo.addChild(importContainer);
+		this.importContainerInfo = new JavaElementInfo();
+		JavaElementInfo parentInfo = (JavaElementInfo) this.infoStack.peek();
+		addToChildren(parentInfo, importContainer);
 		this.newElements.put(importContainer, this.importContainerInfo);
 	}
 	
-	String elementName = JavaModelManager.getJavaModelManager().intern(new String(name));
+	String elementName = JavaModelManager.getJavaModelManager().intern(new String(CharOperation.concatWith(tokens, '.')));
 	ImportDeclaration handle = new ImportDeclaration(importContainer, elementName, onDemand);
 	resolveDuplicates(handle);
 	
@@ -127,7 +133,7 @@ public void acceptImport(int declarationStart, int declarationEnd, char[] name,
 	info.setSourceRangeEnd(declarationEnd);
 	info.setFlags(modifiers);
 
-	this.importContainerInfo.addChild(handle);
+	addToChildren(this.importContainerInfo, handle);
 	this.newElements.put(handle, info);
 }
 /*
@@ -161,25 +167,31 @@ public void acceptPackage(int declarationStart, int declarationEnd, char[] name)
 		info.setSourceRangeStart(declarationStart);
 		info.setSourceRangeEnd(declarationEnd);
 
-		parentInfo.addChild(handle);
+		addToChildren(parentInfo, handle);
 		this.newElements.put(handle, info);
 
 }
-public void acceptProblem(IProblem problem) {
+public void acceptProblem(CategorizedProblem problem) {
 	if ((problem.getID() & IProblem.Syntax) != 0){
 		this.hasSyntaxErrors = true;
 	}
 }
+private void addToChildren(JavaElementInfo parentInfo, JavaElement handle) {
+	ArrayList childrenList = (ArrayList) this.children.get(parentInfo);
+	if (childrenList == null)
+		this.children.put(parentInfo, childrenList = new ArrayList());
+	childrenList.add(handle);
+}
 /**
  * Convert these type names to signatures.
  * @see Signature
  */
 /* default */ static String[] convertTypeNamesToSigs(char[][] typeNames) {
 	if (typeNames == null)
-		return NO_STRINGS;
+		return CharOperation.NO_STRINGS;
 	int n = typeNames.length;
 	if (n == 0)
-		return NO_STRINGS;
+		return CharOperation.NO_STRINGS;
 	JavaModelManager manager = JavaModelManager.getJavaModelManager();
 	String[] typeSigs = new String[n];
 	for (int i = 0; i < n; ++i) {
@@ -192,6 +204,7 @@ public void acceptProblem(IProblem problem) {
  */
 public void enterCompilationUnit() {
 	this.infoStack = new Stack();
+	this.children = new HashMap();
 	this.handleStack= new Stack();
 	this.infoStack.push(this.unitInfo);
 	this.handleStack.push(this.unit);
@@ -229,7 +242,8 @@ public void enterField(FieldInfo fieldInfo) {
 	
 	this.unitInfo.addAnnotationPositions(handle, fieldInfo.annotationPositions);
 
-	parentInfo.addChild(handle);
+	addToChildren(parentInfo, handle);
+	parentInfo.addCategories(handle, fieldInfo.categories);
 	this.newElements.put(handle, info);
 
 	this.infoStack.push(info);
@@ -257,7 +271,7 @@ public void enterInitializer(
 		info.setSourceRangeStart(declarationSourceStart);
 		info.setFlags(modifiers);
 
-		parentInfo.addChild(handle);
+		addToChildren(parentInfo, handle);
 		this.newElements.put(handle, info);
 
 		this.infoStack.push(info);
@@ -317,7 +331,8 @@ public void enterMethod(MethodInfo methodInfo) {
 	for (int i = 0, length = exceptionTypes.length; i < length; i++)
 		exceptionTypes[i] = manager.intern(exceptionTypes[i]);
 	this.unitInfo.addAnnotationPositions(handle, methodInfo.annotationPositions);
-	parentInfo.addChild(handle);
+	addToChildren(parentInfo, handle);
+	parentInfo.addCategories(handle, methodInfo.categories);
 	this.newElements.put(handle, info);
 	this.infoStack.push(info);
 	this.handleStack.push(handle);
@@ -338,10 +353,17 @@ public void enterType(TypeInfo typeInfo) {
 	JavaElementInfo parentInfo = (JavaElementInfo) this.infoStack.peek();
 	JavaElement parentHandle= (JavaElement) this.handleStack.peek();
 	String nameString= new String(typeInfo.name);
-	SourceType handle = handle = new SourceType(parentHandle, nameString); //NB: occurenceCount is computed in resolveDuplicates
+	SourceType handle = new SourceType(parentHandle, nameString); //NB: occurenceCount is computed in resolveDuplicates
 	resolveDuplicates(handle);
 	
-	SourceTypeElementInfo info = new SourceTypeElementInfo();
+	SourceTypeElementInfo info = 
+		typeInfo.anonymousMember ? 
+			new SourceTypeElementInfo() {
+				public boolean isAnonymousMember() {
+					return true;
+				}
+			} : 
+		new SourceTypeElementInfo();
 	info.setHandle(handle);
 	info.setSourceRangeStart(typeInfo.declarationStart);
 	info.setFlags(typeInfo.modifiers);
@@ -354,7 +376,10 @@ public void enterType(TypeInfo typeInfo) {
 	for (int i = 0, length = superinterfaces == null ? 0 : superinterfaces.length; i < length; i++)
 		superinterfaces[i] = manager.intern(superinterfaces[i]);
 	info.setSuperInterfaceNames(superinterfaces);
-	parentInfo.addChild(handle);
+	info.addCategories(handle, typeInfo.categories);
+	if (parentHandle.getElementType() == IJavaElement.TYPE)
+		((SourceTypeElementInfo) parentInfo).addCategories(handle, typeInfo.categories);
+	addToChildren(parentInfo, handle);
 	this.unitInfo.addAnnotationPositions(handle, typeInfo.annotationPositions);
 	this.newElements.put(handle, info);
 	this.infoStack.push(info);
@@ -372,7 +397,7 @@ protected void enterTypeParameter(TypeParameterInfo typeParameterInfo) {
 	JavaElementInfo parentInfo = (JavaElementInfo) this.infoStack.peek();
 	JavaElement parentHandle = (JavaElement) this.handleStack.peek();
 	String nameString = new String(typeParameterInfo.name);
-	TypeParameter handle = handle = new TypeParameter(parentHandle, nameString); //NB: occurenceCount is computed in resolveDuplicates
+	TypeParameter handle = new TypeParameter(parentHandle, nameString); //NB: occurenceCount is computed in resolveDuplicates
 	resolveDuplicates(handle);
 	
 	TypeParameterElementInfo info = new TypeParameterElementInfo();
@@ -404,6 +429,14 @@ protected void enterTypeParameter(TypeParameterInfo typeParameterInfo) {
  * @see ISourceElementRequestor
  */
 public void exitCompilationUnit(int declarationEnd) {
+	// set import container children
+	if (this.importContainerInfo != null) {
+		setChildren(this.importContainerInfo);
+	}
+	
+	// set children
+	setChildren(this.unitInfo);
+	
 	this.unitInfo.setSourceLength(declarationEnd + 1);
 
 	// determine if there were any parsing errors
@@ -421,6 +454,7 @@ public void exitConstructor(int declarationEnd) {
 public void exitField(int initializationStart, int declarationEnd, int declarationSourceEnd) {
 	SourceFieldElementInfo info = (SourceFieldElementInfo) this.infoStack.pop();
 	info.setSourceRangeEnd(declarationSourceEnd);
+	setChildren(info);
 	
 	// remember initializer source if field is a constant
 	if (initializationStart != -1) {
@@ -451,6 +485,7 @@ public void exitInitializer(int declarationEnd) {
 protected void exitMember(int declarationEnd) {
 	SourceRefElementInfo info = (SourceRefElementInfo) this.infoStack.pop();
 	info.setSourceRangeEnd(declarationEnd);
+	setChildren(info);
 	this.handleStack.pop();
 }
 /**
@@ -459,6 +494,7 @@ protected void exitMember(int declarationEnd) {
 public void exitMethod(int declarationEnd, int defaultValueStart, int defaultValueEnd) {
 	SourceMethodElementInfo info = (SourceMethodElementInfo) this.infoStack.pop();
 	info.setSourceRangeEnd(declarationEnd);
+	setChildren(info);
 	
 	// remember default value of annotation method
 	if (info.isAnnotationMethod()) {
@@ -484,4 +520,13 @@ protected void resolveDuplicates(SourceRefElement handle) {
 		handle.occurrenceCount++;
 	}
 }
+private void setChildren(JavaElementInfo info) {
+	ArrayList childrenList = (ArrayList) this.children.get(info);
+	if (childrenList != null) {
+		int length = childrenList.size();
+		IJavaElement[] elements = new IJavaElement[length];
+		childrenList.toArray(elements);
+		info.children = elements;
+	}
+}
 }
