@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -30,7 +30,8 @@ import org.aspectj.org.eclipse.jdt.core.dom.SimpleName;
 import org.aspectj.org.eclipse.jdt.core.dom.StructuralPropertyDescriptor;
 import org.aspectj.org.eclipse.jdt.core.dom.TypeDeclaration;
 import org.aspectj.org.eclipse.jdt.core.dom.rewrite.ASTRewrite;
-import org.aspectj.org.eclipse.jdt.internal.core.dom.rewrite.Indents;
+import org.aspectj.org.eclipse.jdt.core.formatter.IndentManipulation;
+import org.aspectj.org.eclipse.jdt.internal.compiler.parser.ScannerHelper;
 import org.eclipse.jface.text.IDocument;
 import org.eclipse.jface.text.TextUtilities;
 
@@ -104,7 +105,7 @@ protected ASTNode generateElementAST(ASTRewrite rewriter, IDocument document, IC
 				newSource.append(this.alteredName);
 				newSource.append(createdNodeSource.substring(nameEnd));
 			} else {
-				// syntacticaly incorrect source
+				// syntactically incorrect source
 				int createdNodeStart = this.createdNode.getStartPosition();
 				int createdNodeEnd = createdNodeStart + this.createdNode.getLength();
 				newSource.append(createdNodeSource.substring(createdNodeStart, nameStart));
@@ -122,20 +123,20 @@ protected ASTNode generateElementAST(ASTRewrite rewriter, IDocument document, IC
 private String removeIndentAndNewLines(String code, IDocument document, ICompilationUnit cu) {
 	IJavaProject project = cu.getJavaProject();
 	Map options = project.getOptions(true/*inherit JavaCore options*/);
-	int tabWidth = Indents.getTabWidth(options);
-	int indentWidth = Indents.getIndentWidth(options, tabWidth);
-	int indent = Indents.computeIndentUnits(code, tabWidth, indentWidth);
+	int tabWidth = IndentManipulation.getTabWidth(options);
+	int indentWidth = IndentManipulation.getIndentWidth(options);
+	int indent = IndentManipulation.measureIndentUnits(code, tabWidth, indentWidth);
 	int firstNonWhiteSpace = -1;
 	int length = code.length();
 	while (firstNonWhiteSpace < length-1)
-		if (!Character.isWhitespace(code.charAt(++firstNonWhiteSpace)))
+		if (!ScannerHelper.isWhitespace(code.charAt(++firstNonWhiteSpace)))
 			break;
 	int lastNonWhiteSpace = length;
 	while (lastNonWhiteSpace > 0)
-		if (!Character.isWhitespace(code.charAt(--lastNonWhiteSpace)))
+		if (!ScannerHelper.isWhitespace(code.charAt(--lastNonWhiteSpace)))
 			break;
 	String lineDelimiter = TextUtilities.getDefaultLineDelimiter(document);
-	return Indents.changeIndent(code.substring(firstNonWhiteSpace, lastNonWhiteSpace+1), indent, tabWidth, indentWidth, "", lineDelimiter); //$NON-NLS-1$
+	return IndentManipulation.changeIndent(code.substring(firstNonWhiteSpace, lastNonWhiteSpace+1), indent, tabWidth, indentWidth, "", lineDelimiter); //$NON-NLS-1$
 }
 /*
  * Renames the given node to the given name.
