@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,8 +11,8 @@
 package org.aspectj.org.eclipse.jdt.internal.core;
 
 public interface INamingRequestor {
-	void acceptNameWithPrefixAndSuffix(char[] name, boolean isFirstPrefix, boolean isFirstSuffix);
-	void acceptNameWithPrefix(char[] name, boolean isFirstPrefix);
-	void acceptNameWithSuffix(char[] name, boolean isFirstSuffix);
-	void acceptNameWithoutPrefixAndSuffix(char[] name);
+	void acceptNameWithPrefixAndSuffix(char[] name, boolean isFirstPrefix, boolean isFirstSuffix, int reusedCharacters);
+	void acceptNameWithPrefix(char[] name, boolean isFirstPrefix, int reusedCharacters);
+	void acceptNameWithSuffix(char[] name, boolean isFirstSuffix, int reusedCharacters);
+	void acceptNameWithoutPrefixAndSuffix(char[] name, int reusedCharacters);
 }
