@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -18,6 +18,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.env.AccessRuleSet;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.AccessRule;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SimpleLookupTable;
 import org.aspectj.org.eclipse.jdt.internal.core.ClasspathAccessRule;
+import org.aspectj.org.eclipse.jdt.internal.core.JavaModelManager;
 
 import java.io.*;
 import java.util.*;
@@ -31,7 +32,7 @@ ClasspathLocation[] binaryLocations;
 // keyed by the project relative path of the type (ie. "src1/p1/p2/A.java"), value is a ReferenceCollection or an AdditionalTypeCollection
 SimpleLookupTable references;
 // keyed by qualified type name "p1/p2/A", value is the project relative path which defines this type "src1/p1/p2/A.java"
-SimpleLookupTable typeLocators;
+public SimpleLookupTable typeLocators;
 
 int buildNumber;
 long lastStructuralBuildTime;
@@ -43,7 +44,7 @@ private long previousStructuralBuildTime;
 private StringSet structurallyChangedTypes;
 public static int MaxStructurallyChangedTypes = 100; // keep track of ? structurally changed types, otherwise consider all to be changed
 
-public static final byte VERSION = 0x0013; // added timestamps to external jars
+public static final byte VERSION = 0x0015; // changed access rule presentation
 
 static final byte SOURCE_FOLDER = 1;
 static final byte BINARY_FOLDER = 2;
@@ -65,10 +66,17 @@ protected State(JavaBuilder javaBuilder) {
 	this.typeLocators = new SimpleLookupTable(7);
 
 	this.buildNumber = 0; // indicates a full build
-	this.lastStructuralBuildTime = System.currentTimeMillis();
+	this.lastStructuralBuildTime = computeStructuralBuildTime(javaBuilder.lastState == null ? 0 : javaBuilder.lastState.lastStructuralBuildTime);
 	this.structuralBuildTimes = new SimpleLookupTable(3);
 }
 
+long computeStructuralBuildTime(long previousTime) {
+	long newTime = System.currentTimeMillis();
+	if (newTime <= previousTime)
+		newTime = previousTime + 1;
+	return newTime;
+}
+
 void copyFrom(State lastState) {
 	this.knownPackageNames = null;
 	this.previousStructuralBuildTime = lastState.previousStructuralBuildTime;
@@ -96,8 +104,7 @@ void copyFrom(State lastState) {
 				this.typeLocators.put(keyTable[i], valueTable[i]);
 	}
 }
-
-char[][] getDefinedTypeNamesFor(String typeLocator) {
+public char[][] getDefinedTypeNamesFor(String typeLocator) {
 	Object c = references.get(typeLocator);
 	if (c instanceof AdditionalTypeCollection)
 		return ((AdditionalTypeCollection) c).definedTypeNames;
@@ -144,9 +151,13 @@ boolean isKnownPackage(String qualifiedPackageName) {
 	return false;
 }
 
+boolean isKnownType(String qualifiedTypeName) {
+	return typeLocators.containsKey(qualifiedTypeName);
+}
+
 void record(String typeLocator, char[][][] qualifiedRefs, char[][] simpleRefs, char[] mainTypeName, ArrayList typeNames) {
 	if (typeNames.size() == 1 && CharOperation.equals(mainTypeName, (char[]) typeNames.get(0))) {
-			references.put(typeLocator, new ReferenceCollection(qualifiedRefs, simpleRefs));
+		references.put(typeLocator, new ReferenceCollection(qualifiedRefs, simpleRefs));
 	} else {
 		char[][] definedTypeNames = new char[typeNames.size()][]; // can be empty when no types are defined
 		typeNames.toArray(definedTypeNames);
@@ -316,7 +327,7 @@ private static char[] readName(DataInputStream in) throws IOException {
 private static char[][] readNames(DataInputStream in) throws IOException {
 	int length = in.readInt();
 	char[][] names = new char[length][];
-	for (int i = 0; i < length; i++) 
+	for (int i = 0; i < length; i++)
 		names[i] = readName(in);
 	return names;
 }
@@ -330,9 +341,12 @@ private static AccessRuleSet readRestriction(DataInputStream in) throws IOExcept
 		int problemId = in.readInt();
 		accessRules[i] = new ClasspathAccessRule(pattern, problemId);
 	}
-	String messageTemplate = in.readUTF();
-	AccessRuleSet accessRuleSet = new AccessRuleSet(accessRules);
-	accessRuleSet.messageTemplate = messageTemplate;
+	JavaModelManager manager = JavaModelManager.getJavaModelManager();
+	String[] messageTemplates = new String[AccessRuleSet.MESSAGE_TEMPLATES_LENGTH];
+	for (int i = 0; i < AccessRuleSet.MESSAGE_TEMPLATES_LENGTH; i++) {
+		messageTemplates[i] = manager.intern(in.readUTF());
+	}
+	AccessRuleSet accessRuleSet = new AccessRuleSet(accessRules, messageTemplates);
 	return accessRuleSet;
 }
 
@@ -347,7 +361,7 @@ boolean wasNoopBuild() {
 void tagAsStructurallyChanged() {
 	this.previousStructuralBuildTime = this.lastStructuralBuildTime;
 	this.structurallyChangedTypes = new StringSet(7);
-	this.lastStructuralBuildTime = System.currentTimeMillis();
+	this.lastStructuralBuildTime = computeStructuralBuildTime(this.previousStructuralBuildTime);
 }
 
 boolean wasStructurallyChanged(IProject prereqProject, State prereqState) {
@@ -374,10 +388,10 @@ void write(DataOutputStream out) throws IOException {
 	Object[] valueTable;
 
 /*
- * byte			VERSION
+ * byte		VERSION
  * String		project name
- * int				build number
- * int				last structural build number
+ * int			build number
+ * int			last structural build number
 */
 	out.writeByte(VERSION);
 	out.writeUTF(javaProjectName);
@@ -386,7 +400,7 @@ void write(DataOutputStream out) throws IOException {
 
 /*
  * ClasspathMultiDirectory[]
- * int				id
+ * int			id
  * String		path(s)
 */
 	out.writeInt(length = sourceLocations.length);
@@ -401,7 +415,7 @@ void write(DataOutputStream out) throws IOException {
 
 /*
  * ClasspathLocation[]
- * int				id
+ * int			id
  * String		path(s)
 */
 	out.writeInt(length = binaryLocations.length);
@@ -438,7 +452,7 @@ void write(DataOutputStream out) throws IOException {
 /*
  * Structural build numbers table
  * String		prereq project name
- * int				last structural build number
+ * int			last structural build number
 */
 	out.writeInt(length = structuralBuildTimes.elementSize);
 	if (length > 0) {
@@ -456,10 +470,10 @@ void write(DataOutputStream out) throws IOException {
 	}
 
 /*
- * String[]		Interned type locators
+ * String[]	Interned type locators
  */
 	out.writeInt(length = references.elementSize);
-	ArrayList internedTypeLocators = new ArrayList(length);
+	SimpleLookupTable internedTypeLocators = new SimpleLookupTable(length);
 	if (length > 0) {
 		keyTable = references.keyTable;
 		for (int i = 0, l = keyTable.length; i < l; i++) {
@@ -467,7 +481,7 @@ void write(DataOutputStream out) throws IOException {
 				length--;
 				String key = (String) keyTable[i];
 				out.writeUTF(key);
-				internedTypeLocators.add(key);
+				internedTypeLocators.put(key, new Integer(internedTypeLocators.elementSize));
 			}
 		}
 		if (JavaBuilder.DEBUG && length != 0)
@@ -477,7 +491,7 @@ void write(DataOutputStream out) throws IOException {
 /*
  * Type locators table
  * String		type name
- * int				interned locator id
+ * int			interned locator id
  */
 	out.writeInt(length = typeLocators.elementSize);
 	if (length > 0) {
@@ -487,7 +501,8 @@ void write(DataOutputStream out) throws IOException {
 			if (keyTable[i] != null) {
 				length--;
 				out.writeUTF((String) keyTable[i]);
-				out.writeInt(internedTypeLocators.indexOf(valueTable[i]));
+				Integer index = (Integer) internedTypeLocators.get(valueTable[i]);
+				out.writeInt(index.intValue());
 			}
 		}
 		if (JavaBuilder.DEBUG && length != 0)
@@ -496,10 +511,10 @@ void write(DataOutputStream out) throws IOException {
 
 /*
  * char[][][]	Interned qualified names
- * char[][]		Interned simple names
+ * char[][]	Interned simple names
  */
-	ArrayList internedQualifiedNames = new ArrayList(31);
-	ArrayList internedSimpleNames = new ArrayList(31);
+	SimpleLookupTable internedQualifiedNames = new SimpleLookupTable(31);
+	SimpleLookupTable internedSimpleNames = new SimpleLookupTable(31);
 	valueTable = references.valueTable;
 	for (int i = 0, l = valueTable.length; i < l; i++) {
 		if (valueTable[i] != null) {
@@ -507,39 +522,57 @@ void write(DataOutputStream out) throws IOException {
 			char[][][] qNames = collection.qualifiedNameReferences;
 			for (int j = 0, m = qNames.length; j < m; j++) {
 				char[][] qName = qNames[j];
-				if (!internedQualifiedNames.contains(qName)) { // remember the names have been interned
-					internedQualifiedNames.add(qName);
+				if (!internedQualifiedNames.containsKey(qName)) { // remember the names have been interned
+					internedQualifiedNames.put(qName, new Integer(internedQualifiedNames.elementSize));
 					for (int k = 0, n = qName.length; k < n; k++) {
 						char[] sName = qName[k];
-						if (!internedSimpleNames.contains(sName)) // remember the names have been interned
-							internedSimpleNames.add(sName);
+						if (!internedSimpleNames.containsKey(sName)) // remember the names have been interned
+							internedSimpleNames.put(sName, new Integer(internedSimpleNames.elementSize));
 					}
 				}
 			}
 			char[][] sNames = collection.simpleNameReferences;
 			for (int j = 0, m = sNames.length; j < m; j++) {
 				char[] sName = sNames[j];
-				if (!internedSimpleNames.contains(sName)) // remember the names have been interned
-					internedSimpleNames.add(sName);
+				if (!internedSimpleNames.containsKey(sName)) // remember the names have been interned
+					internedSimpleNames.put(sName, new Integer(internedSimpleNames.elementSize));
 			}
 		}
 	}
-	char[][] internedArray = new char[internedSimpleNames.size()][];
-	internedSimpleNames.toArray(internedArray);
+	char[][] internedArray = new char[internedSimpleNames.elementSize][];
+	Object[] simpleNames = internedSimpleNames.keyTable;
+	Object[] positions = internedSimpleNames.valueTable;
+	for (int i = positions.length; --i >= 0; ) {
+		if (positions[i] != null) {
+			int index = ((Integer) positions[i]).intValue();
+			internedArray[index] = (char[]) simpleNames[i];
+		}
+	}
 	writeNames(internedArray, out);
 	// now write the interned qualified names as arrays of interned simple names
-	out.writeInt(length = internedQualifiedNames.size());
+	char[][][] internedQArray = new char[internedQualifiedNames.elementSize][][];
+	Object[] qualifiedNames = internedQualifiedNames.keyTable;
+	positions = internedQualifiedNames.valueTable;
+	for (int i = positions.length; --i >= 0; ) {
+		if (positions[i] != null) {
+			int index = ((Integer) positions[i]).intValue();
+			internedQArray[index] = (char[][]) qualifiedNames[i];
+		}
+	}
+	out.writeInt(length = internedQArray.length);
 	for (int i = 0; i < length; i++) {
-		char[][] qName = (char[][]) internedQualifiedNames.get(i);
+		char[][] qName = internedQArray[i];
 		int qLength = qName.length;
 		out.writeInt(qLength);
-		for (int j = 0; j < qLength; j++)
-			out.writeInt(internedSimpleNames.indexOf(qName[j]));
+		for (int j = 0; j < qLength; j++) {
+			Integer index = (Integer) internedSimpleNames.get(qName[j]);
+			out.writeInt(index.intValue());
+		}
 	}
 
 /*
  * References table
- * int			interned locator id
+ * int		interned locator id
  * ReferenceCollection
 */
 	out.writeInt(length = references.elementSize);
@@ -548,7 +581,8 @@ void write(DataOutputStream out) throws IOException {
 		for (int i = 0, l = keyTable.length; i < l; i++) {
 			if (keyTable[i] != null) {
 				length--;
-				out.writeInt(internedTypeLocators.indexOf(keyTable[i]));
+				Integer index = (Integer) internedTypeLocators.get(keyTable[i]);
+				out.writeInt(index.intValue());
 				ReferenceCollection collection = (ReferenceCollection) valueTable[i];
 				if (collection instanceof AdditionalTypeCollection) {
 					out.writeByte(1);
@@ -560,13 +594,17 @@ void write(DataOutputStream out) throws IOException {
 				char[][][] qNames = collection.qualifiedNameReferences;
 				int qLength = qNames.length;
 				out.writeInt(qLength);
-				for (int j = 0; j < qLength; j++)
-					out.writeInt(internedQualifiedNames.indexOf(qNames[j]));
+				for (int j = 0; j < qLength; j++) {
+					index = (Integer) internedQualifiedNames.get(qNames[j]);
+					out.writeInt(index.intValue());
+				}
 				char[][] sNames = collection.simpleNameReferences;
 				int sLength = sNames.length;
 				out.writeInt(sLength);
-				for (int j = 0; j < sLength; j++)
-					out.writeInt(internedSimpleNames.indexOf(sNames[j]));
+				for (int j = 0; j < sLength; j++) {
+					index = (Integer) internedSimpleNames.get(sNames[j]);
+					out.writeInt(index.intValue());
+				}
 			}
 		}
 		if (JavaBuilder.DEBUG && length != 0)
@@ -595,13 +633,14 @@ private void writeRestriction(AccessRuleSet accessRuleSet, DataOutputStream out)
 		AccessRule[] accessRules = accessRuleSet.getAccessRules();
 		int length = accessRules.length;
 		out.writeInt(length);
-		if (length != 0) { 
+		if (length != 0) {
 			for (int i = 0; i < length; i++) {
 				AccessRule accessRule = accessRules[i];
 				writeName(accessRule.pattern, out);
 				out.writeInt(accessRule.problemId);
 			}
-			out.writeUTF(accessRuleSet.messageTemplate);
+			for (int i = 0; i < AccessRuleSet.MESSAGE_TEMPLATES_LENGTH; i++)
+				out.writeUTF(accessRuleSet.messageTemplates[i]);
 		}
 	}
 }
