@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,26 +12,30 @@ package org.aspectj.org.eclipse.jdt.internal.core.search;
 
 import java.util.ArrayList;
 import java.util.HashSet;
+import java.util.Map;
 
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
-import org.aspectj.org.eclipse.jdt.core.*;
+import org.aspectj.org.eclipse.jdt.core.IClasspathContainer;
 import org.aspectj.org.eclipse.jdt.core.IClasspathEntry;
 import org.aspectj.org.eclipse.jdt.core.IJavaElement;
 import org.aspectj.org.eclipse.jdt.core.IJavaElementDelta;
 import org.aspectj.org.eclipse.jdt.core.IJavaModel;
 import org.aspectj.org.eclipse.jdt.core.IJavaProject;
 import org.aspectj.org.eclipse.jdt.core.IMember;
-import org.aspectj.org.eclipse.jdt.core.IOpenable;
 import org.aspectj.org.eclipse.jdt.core.IPackageFragmentRoot;
+import org.aspectj.org.eclipse.jdt.core.JavaCore;
 import org.aspectj.org.eclipse.jdt.core.JavaModelException;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.AccessRuleSet;
-import org.aspectj.org.eclipse.jdt.internal.core.*;
+import org.aspectj.org.eclipse.jdt.internal.core.ClasspathEntry;
 import org.aspectj.org.eclipse.jdt.internal.core.JavaElement;
+import org.aspectj.org.eclipse.jdt.internal.core.JavaModel;
 import org.aspectj.org.eclipse.jdt.internal.core.JavaModelManager;
 import org.aspectj.org.eclipse.jdt.internal.core.JavaProject;
+import org.aspectj.org.eclipse.jdt.internal.core.PackageFragment;
 import org.aspectj.org.eclipse.jdt.internal.core.util.Util;
 
 /**
@@ -42,18 +46,20 @@ public class JavaSearchScope extends AbstractSearchScope {
 	private ArrayList elements;
 
 	/* The paths of the resources in this search scope 
-	   (or the classpath entries' paths 
-	   if the resources are projects) */
-	private String[] paths;
-	private boolean[] pathWithSubFolders;
+	    (or the classpath entries' paths if the resources are projects) 
+	*/
+	private ArrayList projectPaths = new ArrayList(); // container paths projects 
+	private int[] projectIndexes; // Indexes of projects in list
+	private String[] containerPaths; // path to the container (e.g. /P/src, /P/lib.jar, c:\temp\mylib.jar)
+	private String[] relativePaths; // path relative to the container (e.g. x/y/Z.class, x/y, (empty))
+	private boolean[] isPkgPath; // in the case of packages, matches must be direct children of the folder
 	protected AccessRuleSet[] pathRestrictions;
-	private String[] containerPaths;
 	private int pathsCount;
 	private int threshold;
 	
 	private IPath[] enclosingProjectsAndJars;
-	public final static AccessRuleSet NOT_ENCLOSED = new AccessRuleSet(null);
-	
+	public final static AccessRuleSet NOT_ENCLOSED = new AccessRuleSet(null, null);
+
 public JavaSearchScope() {
 	this(5);
 }
@@ -105,7 +111,7 @@ void add(JavaProject javaProject, IPath pathToAdd, int includeMask, HashSet visi
 	String projectPathString = projectPath.toString();
 	this.addEnclosingProjectOrJar(projectPath);
 
-	IClasspathEntry[] entries = javaProject.getResolvedClasspath(true/*ignoreUnresolvedEntry*/, false/*don't generateMarkerOnError*/, false/*don't returnResolutionInProgress*/);
+	IClasspathEntry[] entries = javaProject.getResolvedClasspath();
 	IJavaModel model = javaProject.getJavaModel();
 	JavaModelManager.PerProjectInfo perProjectInfo = javaProject.getPerProjectInfo();
 	for (int i = 0, length = entries.length; i < length; i++) {
@@ -123,8 +129,9 @@ void add(JavaProject javaProject, IPath pathToAdd, int includeMask, HashSet visi
 		switch (entry.getEntryKind()) {
 			case IClasspathEntry.CPE_LIBRARY:
 				IClasspathEntry rawEntry = null;
-				if (perProjectInfo != null && perProjectInfo.resolvedPathToRawEntries != null) {
-					rawEntry = (IClasspathEntry) perProjectInfo.resolvedPathToRawEntries.get(entry.getPath());
+				Map rootPathToRawEntries = perProjectInfo.rootPathToRawEntries;
+				if (rootPathToRawEntries != null) {
+					rawEntry = (IClasspathEntry) rootPathToRawEntries.get(entry.getPath());
 				}
 				if (rawEntry == null) break;
 				switch (rawEntry.getEntryKind()) {
@@ -134,7 +141,7 @@ void add(JavaProject javaProject, IPath pathToAdd, int includeMask, HashSet visi
 							IPath path = entry.getPath();
 							if (pathToAdd == null || pathToAdd.equals(path)) {
 								String pathToString = path.getDevice() == null ? path.toString() : path.toOSString();
-								add("", pathToString, true, access); //$NON-NLS-1$
+								add(projectPath.toString(), "", pathToString, false/*not a package*/, access); //$NON-NLS-1$
 								addEnclosingProjectOrJar(path);
 							}
 						}
@@ -147,7 +154,7 @@ void add(JavaProject javaProject, IPath pathToAdd, int includeMask, HashSet visi
 							IPath path = entry.getPath();
 							if (pathToAdd == null || pathToAdd.equals(path)) {
 								String pathToString = path.getDevice() == null ? path.toString() : path.toOSString();
-								add("", pathToString, true, access); //$NON-NLS-1$
+								add(projectPath.toString(), "", pathToString, false/*not a package*/, access); //$NON-NLS-1$
 								addEnclosingProjectOrJar(path);
 							}
 						}
@@ -166,7 +173,7 @@ void add(JavaProject javaProject, IPath pathToAdd, int includeMask, HashSet visi
 				if ((includeMask & SOURCES) != 0) {
 					IPath path = entry.getPath();
 					if (pathToAdd == null || pathToAdd.equals(path)) {
-						add(Util.relativePath(path,1/*remove project segment*/), projectPathString, true, access);
+						add(projectPath.toString(), Util.relativePath(path,1/*remove project segment*/), projectPathString, false/*not a package*/, access);
 					}
 				}
 				break;
@@ -195,27 +202,34 @@ public void add(IJavaElement element) throws JavaModelException {
 			containerPath = root.getKind() == IPackageFragmentRoot.K_SOURCE ? root.getParent().getPath() : rootPath;
 			containerPathToString = containerPath.getDevice() == null ? containerPath.toString() : containerPath.toOSString();
 			IResource rootResource = root.getResource();
+			String projectPath = root.getJavaProject().getPath().toString();
 			if (rootResource != null && rootResource.isAccessible()) {
 				String relativePath = Util.relativePath(rootResource.getFullPath(), containerPath.segmentCount());
-				add(relativePath, containerPathToString, true, null);
+				add(projectPath, relativePath, containerPathToString, false/*not a package*/, null);
 			} else {
-				add("", containerPathToString, true, null); //$NON-NLS-1$
+				add(projectPath, "", containerPathToString, false/*not a package*/, null); //$NON-NLS-1$
 			}
 			break;
 		case IJavaElement.PACKAGE_FRAGMENT:
 			root = (IPackageFragmentRoot)element.getParent();
+			projectPath = root.getJavaProject().getPath().toString();
 			if (root.isArchive()) {
 				String relativePath = Util.concatWith(((PackageFragment) element).names, '/');
 				containerPath = root.getPath();
 				containerPathToString = containerPath.getDevice() == null ? containerPath.toString() : containerPath.toOSString();
-				add(relativePath, containerPathToString, false, null);
+				add(projectPath, relativePath, containerPathToString, true/*package*/, null);
 			} else {
 				IResource resource = element.getResource();
-				if (resource != null && resource.isAccessible()) {
-					containerPath = root.getKind() == IPackageFragmentRoot.K_SOURCE ? root.getParent().getPath() : root.getPath();
+				if (resource != null) {
+					if (resource.isAccessible()) {
+						containerPath = root.getKind() == IPackageFragmentRoot.K_SOURCE ? root.getParent().getPath() : root.getPath();
+					} else {
+						// for working copies, get resource container full path
+						containerPath = resource.getParent().getFullPath();
+					}
 					containerPathToString = containerPath.getDevice() == null ? containerPath.toString() : containerPath.toOSString();
 					String relativePath = Util.relativePath(resource.getFullPath(), containerPath.segmentCount());
-					add(relativePath, containerPathToString, false, null);
+					add(projectPath, relativePath, containerPathToString, true/*package*/, null);
 				}
 			}
 			break;
@@ -228,6 +242,7 @@ public void add(IJavaElement element) throws JavaModelException {
 				this.elements.add(element);
 			}
 			root = (IPackageFragmentRoot) element.getAncestor(IJavaElement.PACKAGE_FRAGMENT_ROOT);
+			projectPath = root.getJavaProject().getPath().toString();
 			String relativePath;
 			if (root.getKind() == IPackageFragmentRoot.K_SOURCE) {
 				containerPath = root.getParent().getPath();
@@ -237,7 +252,7 @@ public void add(IJavaElement element) throws JavaModelException {
 				relativePath = getPath(element, true/*relative path*/).toString();
 			}
 			containerPathToString = containerPath.getDevice() == null ? containerPath.toString() : containerPath.toOSString();
-			add(relativePath, containerPathToString, true, null);
+			add(projectPath, relativePath, containerPathToString, false/*not a package*/, null);
 	}
 	
 	if (containerPath != null)
@@ -248,95 +263,147 @@ public void add(IJavaElement element) throws JavaModelException {
  * Adds the given path to this search scope. Remember if subfolders need to be included
  * and associated access restriction as well.
  */
-private void add(String relativePath, String containerPath, boolean withSubFolders, AccessRuleSet access) {
-	int index = (containerPath.hashCode() & 0x7FFFFFFF) % this.paths.length;
-	String currentPath, currentContainerPath;
-	while ((currentPath = this.paths[index]) != null && (currentContainerPath = this.containerPaths[index]) != null) {
-		if (currentPath.equals(relativePath) && currentContainerPath.equals(containerPath))
+private void add(String projectPath, String relativePath, String containerPath, boolean isPackage, AccessRuleSet access) {
+	// normalize containerPath and relativePath
+	containerPath = normalize(containerPath);
+	relativePath = normalize(relativePath);
+	int length = this.containerPaths.length,
+		index = (containerPath.hashCode()& 0x7FFFFFFF) % length;
+	String currentRelativePath, currentContainerPath;
+	while ((currentRelativePath = this.relativePaths[index]) != null && (currentContainerPath = this.containerPaths[index]) != null) {
+		if (currentRelativePath.equals(relativePath) && currentContainerPath.equals(containerPath))
 			return;
-		index = (index + 1) % this.paths.length;
+		if (++index == length) {
+			index = 0;
+		}
 	}
-	this.paths[index] = relativePath;
+	int idx = this.projectPaths.indexOf(projectPath);
+	if (idx == -1) {
+		// store project in separated list to minimize memory footprint
+		this.projectPaths.add(projectPath);
+		idx = this.projectPaths.indexOf(projectPath);
+	}
+	this.projectIndexes[index] = idx;
+	this.relativePaths[index] = relativePath;
 	this.containerPaths[index] = containerPath;
-	this.pathWithSubFolders[index] = withSubFolders;
+	this.isPkgPath[index] = isPackage;
 	if (this.pathRestrictions != null)
 		this.pathRestrictions[index] = access;
 	else if (access != null) {
-		this.pathRestrictions = new AccessRuleSet[this.paths.length];
+		this.pathRestrictions = new AccessRuleSet[this.relativePaths.length];
 		this.pathRestrictions[index] = access;
 	}
 
 	// assumes the threshold is never equal to the size of the table
 	if (++this.pathsCount > this.threshold)
 		rehash();
-		
 }
 
-/* (non-Javadoc)
+/* 
+ * E.g.
+ * 
+ * 1. /P/src/pkg/X.java
+ * 2. /P/src/pkg
+ * 3. /P/lib.jar|org.aspectj.org.eclipse.jdt/core/IJavaElement.class
+ * 4. /home/mylib.jar|x/y/z/X.class
+ * 5. c:\temp\mylib.jar|x/y/Y.class
+ * 
  * @see IJavaSearchScope#encloses(String)
  */
 public boolean encloses(String resourcePathString) {
 	int separatorIndex = resourcePathString.indexOf(JAR_FILE_ENTRY_SEPARATOR);
 	if (separatorIndex != -1) {
-		return indexOf(resourcePathString.substring(separatorIndex+1), resourcePathString.substring(0, separatorIndex)) >= 0;
+		// internal or external jar (case 3, 4, or 5)
+		String jarPath = resourcePathString.substring(0, separatorIndex);
+		String relativePath = resourcePathString.substring(separatorIndex+1);
+		return indexOf(jarPath, relativePath) >= 0;
 	}
-	return indexOf(resourcePathString, null) >= 0;
+	// resource in workspace (case 1 or 2)
+	return indexOf(resourcePathString) >= 0;
 }
 
 /**
  * Returns paths list index of given path or -1 if not found.
+ * NOTE: Use indexOf(String, String) for path inside jars
+ * 
+ * @param fullPath the full path of the resource, e.g.
+ *   1. /P/src/pkg/X.java
+ *   2. /P/src/pkg
  */
-private int indexOf(String relativePath, String containerPath) {
-	if (containerPath != null) {
-		// if container path is known, use the hash to get faster comparison
-		int index = (containerPath.hashCode()& 0x7FFFFFFF) % this.paths.length;
-		String currentContainerPath;
-		while ((currentContainerPath = this.containerPaths[index]) != null) {
-			if (currentContainerPath.equals(containerPath)) {
-				String scopePath = this.paths[index];
-				if (encloses(scopePath, relativePath, index))
-					return index;
-			}
-			index = (index + 1) % this.paths.length;
-		}
-		return -1;
-	}
-	
+private int indexOf(String fullPath) {
+	// cannot guess the index of the container path
 	// fallback to sequentially looking at all known paths
-	for (int i = 0, length = this.paths.length; i < length; i++) {
-		String scopePath = this.paths[i];
-		if (scopePath == null) continue;
-		if (encloses(this.containerPaths[i] + '/' + scopePath, relativePath, i))
+	for (int i = 0, length = this.relativePaths.length; i < length; i++) {
+		String currentRelativePath = this.relativePaths[i];
+		if (currentRelativePath == null) continue;
+		String currentContainerPath = this.containerPaths[i];
+		String currentFullPath = currentRelativePath.length() == 0 ? currentContainerPath : (currentContainerPath + '/' + currentRelativePath);
+		if (encloses(currentFullPath, fullPath, i))
 			return i;
 	}
 	return -1;
 }
 
-private boolean encloses(String scopePath, String path, int index) {
-	if (this.pathWithSubFolders[index]) {
-		// TODO (frederic) apply similar change also if not looking at subfolders
-		int pathLength = path.length();
-		int scopeLength = scopePath.length();
-		if (pathLength < scopeLength) {
-			return false;
+/**
+ * Returns paths list index of given path or -1 if not found.
+ * @param containerPath the path of the container, e.g.
+ *   1. /P/src
+ *   2. /P
+ *   3. /P/lib.jar
+ *   4. /home/mylib.jar
+ *   5. c:\temp\mylib.jar
+ * @param relativePath the forward slash path relatively to the container, e.g.
+ *   1. x/y/Z.class
+ *   2. x/y
+ *   3. X.java
+ *   4. (empty)
+ */
+private int indexOf(String containerPath, String relativePath) {
+	// use the hash to get faster comparison
+	int length = this.containerPaths.length,
+		index = (containerPath.hashCode()& 0x7FFFFFFF) % length;
+	String currentContainerPath;
+	while ((currentContainerPath = this.containerPaths[index]) != null) {
+		if (currentContainerPath.equals(containerPath)) {
+			String currentRelativePath = this.relativePaths[index];
+			if (encloses(currentRelativePath, relativePath, index))
+				return index;
 		}
-		if (scopeLength == 0) {
-			return true;
-		}
-		if (pathLength == scopeLength) {
-			return path.equals(scopePath);
-		}
-		if (path.startsWith(scopePath)) {
-			if (scopePath.charAt(scopeLength-1) == '/') scopeLength--;
-			return path.charAt(scopeLength) == '/';
+		if (++index == length) {
+			index = 0;
 		}
+	}
+	return -1;
+}
+
+/*
+ * Returns whether the enclosing path encloses the given path (or is equal to it)
+ */
+private boolean encloses(String enclosingPath, String path, int index) {
+	// normalize given path as it can come from outside
+	path = normalize(path);
+	
+	int pathLength = path.length();
+	int enclosingLength = enclosingPath.length();
+	if (pathLength < enclosingLength) {
+		return false;
+	}
+	if (enclosingLength == 0) {
+		return true;
+	}
+	if (pathLength == enclosingLength) {
+		return path.equals(enclosingPath);
+	}
+	if (!this.isPkgPath[index]) {
+		return path.startsWith(enclosingPath)
+			&& path.charAt(enclosingLength) == '/';
 	} else {
-		// if not looking at subfolders, this scope encloses the given path 
-		// if this path is a direct child of the scope's ressource
-		// or if this path is the scope's resource (see bug 13919 Declaration for package not found if scope is not project)
-		if (path.startsWith(scopePath) 
-			&& ((scopePath.length() == path.lastIndexOf('/'))
-				|| (scopePath.length() == path.length()))) {
+		// if looking at a package, this scope encloses the given path 
+		// if the given path is a direct child of the folder
+		// or if the given path path is the folder path (see bug 13919 Declaration for package not found if scope is not project)
+		if (path.startsWith(enclosingPath) 
+			&& ((enclosingPath.length() == path.lastIndexOf('/'))
+				|| (enclosingPath.length() == path.length()))) {
 			return true;
 		}
 	}
@@ -361,12 +428,15 @@ public boolean encloses(IJavaElement element) {
 	}
 	IPackageFragmentRoot root = (IPackageFragmentRoot) element.getAncestor(IJavaElement.PACKAGE_FRAGMENT_ROOT);
 	if (root != null && root.isArchive()) {
+		// external or internal jar
 		IPath rootPath = root.getPath();
 		String rootPathToString = rootPath.getDevice() == null ? rootPath.toString() : rootPath.toOSString();
 		IPath relativePath = getPath(element, true/*relative path*/);
-		return indexOf(relativePath.toString(), rootPathToString) >= 0;
+		return indexOf(rootPathToString, relativePath.toString()) >= 0;
 	}
-	return this.indexOf(getPath(element, false/*full path*/).toString(), null) >= 0;
+	// resource in workspace
+	String fullResourcePathString = getPath(element, false/*full path*/).toString();
+	return indexOf(fullResourcePathString) >= 0;
 }
 
 /* (non-Javadoc)
@@ -376,23 +446,24 @@ public IPath[] enclosingProjectsAndJars() {
 	return this.enclosingProjectsAndJars;
 }
 private IPath getPath(IJavaElement element, boolean relativeToRoot) {
-	if (element instanceof IPackageFragmentRoot) {
-		if (relativeToRoot)
+	switch (element.getElementType()) {
+		case IJavaElement.JAVA_MODEL:
 			return Path.EMPTY;
-		return ((IPackageFragmentRoot)element).getPath();
+		case IJavaElement.JAVA_PROJECT:
+			return element.getPath();
+		case IJavaElement.PACKAGE_FRAGMENT_ROOT:
+			if (relativeToRoot)
+				return Path.EMPTY;
+			return element.getPath();
+		case IJavaElement.PACKAGE_FRAGMENT:
+			String relativePath = Util.concatWith(((PackageFragment) element).names, '/');
+			return getPath(element.getParent(), relativeToRoot).append(new Path(relativePath));
+		case IJavaElement.COMPILATION_UNIT:
+		case IJavaElement.CLASS_FILE:
+			return getPath(element.getParent(), relativeToRoot).append(new Path(element.getElementName()));
+		default:
+			return getPath(element.getParent(), relativeToRoot);
 	}
-	IJavaElement parent = element.getParent();
-	IPath parentPath = parent == null ? null : getPath(parent, relativeToRoot);
-	IPath childPath;
-	if (element instanceof PackageFragment) {
-		String relativePath = Util.concatWith(((PackageFragment) element).names, '/');
-		childPath = new Path(relativePath);
-	} else if (element instanceof IOpenable) {
-		childPath = new Path(element.getElementName());
-	} else {
-		return parentPath;
-	}
-	return parentPath == null ? childPath : parentPath.append(childPath);
 }
 
 /**
@@ -402,7 +473,7 @@ private IPath getPath(IJavaElement element, boolean relativeToRoot) {
  * 	Returns specific uninit access rule set when scope does not enclose the given path.
  */
 public AccessRuleSet getAccessRuleSet(String relativePath, String containerPath) {
-	int index = indexOf(relativePath, containerPath);
+	int index = indexOf(containerPath, relativePath);
 	if (index == -1) {
 		// this search scope does not enclose given path
 		return NOT_ENCLOSED;
@@ -418,13 +489,29 @@ protected void initialize(int size) {
 	int extraRoom = (int) (size * 1.75f);
 	if (this.threshold == extraRoom)
 		extraRoom++;
-	this.paths = new String[extraRoom];
+	this.relativePaths = new String[extraRoom];
 	this.containerPaths = new String[extraRoom];
-	this.pathWithSubFolders = new boolean[extraRoom];
+	this.projectPaths = new ArrayList();
+	this.projectIndexes = new int[extraRoom];
+	this.isPkgPath = new boolean[extraRoom];
 	this.pathRestrictions = null; // null to optimize case where no access rules are used
 
 	this.enclosingProjectsAndJars = new IPath[0];
 }
+
+/*
+ * Removes trailing slashes from the given path
+ */
+private String normalize(String path) {
+	int pathLength = path.length();
+	int index = pathLength-1;
+	while (index >= 0 && path.charAt(index) == '/')
+		index--;
+	if (index != pathLength-1)
+		return path.substring(0, index + 1);
+	return path;
+}
+
 /*
  * @see AbstractSearchScope#processDelta(IJavaElementDelta)
  */
@@ -453,13 +540,13 @@ public void processDelta(IJavaElementDelta delta) {
 						}
 						int toRemove = -1;
 						for (int i = 0; i < this.pathsCount; i++) {
-							if (this.paths[i].equals(path)) {
+							if (this.relativePaths[i].equals(path)) { // TODO (jerome) this compares String and IPath !
 								toRemove = i;
 								break;
 							}
 						}
 						if (toRemove != -1) {
-							this.paths[toRemove] = null;
+							this.relativePaths[toRemove] = null;
 							rehash();
 						}
 				}
@@ -468,16 +555,63 @@ public void processDelta(IJavaElementDelta delta) {
 	}
 }
 
+/**
+ * Returns the package fragment root corresponding to a given resource path.
+ * 
+ * @param resourcePathString path of expected package fragment root.
+ * @return the {@link IPackageFragmentRoot package fragment root} which path
+ * 	match the given one or <code>null</code> if none was found.
+ */
+public IPackageFragmentRoot packageFragmentRoot(String resourcePathString) {
+	int index = -1;
+	int separatorIndex = resourcePathString.indexOf(JAR_FILE_ENTRY_SEPARATOR);
+	boolean isJarFile = separatorIndex != -1;
+	if (isJarFile) {
+		// internal or external jar (case 3, 4, or 5)
+		String jarPath = resourcePathString.substring(0, separatorIndex);
+		String relativePath = resourcePathString.substring(separatorIndex+1);
+		index = indexOf(jarPath, relativePath);
+	} else {
+		// resource in workspace (case 1 or 2)
+		index = indexOf(resourcePathString);
+	}
+	if (index >= 0) {
+		int idx = projectIndexes[index];
+		String projectPath = idx == -1 ? null : (String) this.projectPaths.get(idx);
+		if (projectPath != null) {
+			IJavaProject project =JavaCore.create(ResourcesPlugin.getWorkspace().getRoot().getProject(projectPath));
+			Object target = JavaModel.getTarget(ResourcesPlugin.getWorkspace().getRoot(), new Path(this.containerPaths[index]+'/'+this.relativePaths[index]), false);
+			if (target instanceof IProject) {
+				return project.getPackageFragmentRoot((IProject) target);
+			}
+			if (target instanceof IResource) {
+				IJavaElement element = JavaCore.create((IResource)target);
+				return (IPackageFragmentRoot) element.getAncestor(IJavaElement.PACKAGE_FRAGMENT_ROOT);
+			}
+			if (isJarFile) {
+				return project.getPackageFragmentRoot(this.containerPaths[index]);
+			}
+		}
+	}
+	return null;
+}
+
 private void rehash() {
 	JavaSearchScope newScope = new JavaSearchScope(this.pathsCount * 2);		// double the number of expected elements
+	newScope.projectPaths.ensureCapacity(this.projectPaths.size());
 	String currentPath;
-	for (int i = this.paths.length; --i >= 0;)
-		if ((currentPath = this.paths[i]) != null)
-			newScope.add(currentPath, this.containerPaths[i], this.pathWithSubFolders[i], this.pathRestrictions == null ? null : this.pathRestrictions[i]);
+	for (int i = this.relativePaths.length; --i >= 0;)
+		if ((currentPath = this.relativePaths[i]) != null) {
+			int idx = this.projectIndexes[i];
+			String projectPath = idx == -1 ? null : (String)this.projectPaths.get(idx);
+			newScope.add(projectPath, currentPath, this.containerPaths[i], this.isPkgPath[i], this.pathRestrictions == null ? null : this.pathRestrictions[i]);
+		}
 
-	this.paths = newScope.paths;
+	this.relativePaths = newScope.relativePaths;
 	this.containerPaths = newScope.containerPaths;
-	this.pathWithSubFolders = newScope.pathWithSubFolders;
+	this.projectPaths = newScope.projectPaths;
+	this.projectIndexes = newScope.projectIndexes;
+	this.isPkgPath = newScope.isPkgPath;
 	this.pathRestrictions = newScope.pathRestrictions;
 	this.threshold = newScope.threshold;
 }
@@ -497,8 +631,8 @@ public String toString() {
 			result.append("[empty scope]"); //$NON-NLS-1$
 		} else {
 			result.append("["); //$NON-NLS-1$
-			for (int i = 0; i < this.paths.length; i++) {
-				String path = this.paths[i];
+			for (int i = 0; i < this.relativePaths.length; i++) {
+				String path = this.relativePaths[i];
 				if (path == null) continue;
 				result.append("\n\t"); //$NON-NLS-1$
 				result.append(this.containerPaths[i]);
