@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2006 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,9 +11,8 @@
 package org.aspectj.org.eclipse.jdt.internal.core.search.matching;
 
 import org.aspectj.org.eclipse.jdt.internal.core.index.*;
-import org.aspectj.org.eclipse.jdt.internal.core.search.indexing.IIndexConstants;
 
-public class PackageDeclarationPattern extends JavaSearchPattern implements IIndexConstants {
+public class PackageDeclarationPattern extends JavaSearchPattern {
 
 protected char[] pkgName;
 
