@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -19,7 +19,7 @@ package org.aspectj.org.eclipse.jdt.internal.formatter.comment;
 public interface IHtmlTagDelimiters {
 
 	/** Html tag close prefix */
-	public static final String HTML_CLOSE_PREFIX= "</"; //$NON-NLS-1$
+	public static final char[] HTML_CLOSE_PREFIX= "</".toCharArray(); //$NON-NLS-1$
 
 	/** Html tag postfix */
 	public static final char HTML_TAG_POSTFIX= '>';
