@@ -11,11 +11,13 @@
  *     Xerox/PARC     initial implementation 
  * ******************************************************************/
 
-
 package org.aspectj.ajde.ui;
 
 import java.io.IOException;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+import java.util.StringTokenizer;
 
 /**
  * TODO: we have schitzophrenia between BuildConfigNode(s) and IProgramElement(s), fix.
@@ -26,14 +28,14 @@ public class BuildConfigModel {
 
 	private BuildConfigNode root = null;
 
-	private String sourceFile; 
-	 
+	private String sourceFile;
+
 	public BuildConfigModel(String sourceFile) {
 		this.sourceFile = sourceFile;
 	}
 
 	/**
-	 * @param	path	java.io.File.separator delimited path
+	 * @param path java.io.File.separator delimited path
 	 * @return corresponding node if the path is found, the root otherwise
 	 */
 	public BuildConfigNode getNodeForPath(String path) {
@@ -47,9 +49,9 @@ public class BuildConfigModel {
 	}
 
 	private BuildConfigNode searchUpPaths(String path) {
-		for (Iterator it = root.getChildren().iterator(); it.hasNext(); ) {
-			BuildConfigNode node = (BuildConfigNode)it.next();
-			if (node.getName().equals(path)) return node;	
+		for (BuildConfigNode node : root.getChildren()) {
+			if (node.getName().equals(path))
+				return node;
 		}
 		return null;
 	}
@@ -57,39 +59,37 @@ public class BuildConfigModel {
 	private BuildConfigNode getNodeForPathHelper(StringTokenizer st, BuildConfigNode node) {
 		BuildConfigNode parent = node;
 		while (st.hasMoreElements()) {
-			String pathItem = (String)st.nextElement();
-			for (Iterator it = node.getChildren().iterator(); it.hasNext(); ) {
-				node = (BuildConfigNode)it.next();
+			String pathItem = (String) st.nextElement();
+			for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
+				node = (BuildConfigNode) it.next();
 				String childName = node.getName();
 				if (childName.equals(pathItem)) {
 					return getNodeForPathHelper(st, node);
-				} 
+				}
 			}
 		}
-		return parent;	
+		return parent;
 	}
-	
-	public List getActiveNodes(BuildConfigNode.Kind kind) {
-		List nodes = new ArrayList();
+
+	public List<BuildConfigNode> getActiveNodes(BuildConfigNode.Kind kind) {
+		List<BuildConfigNode> nodes = new ArrayList<BuildConfigNode>();
 		getActiveNodesHelper(root, kind, nodes);
 		return nodes;
 	}
 
-	private void getActiveNodesHelper(BuildConfigNode node, BuildConfigNode.Kind kind, List nodes) {
-		for (Iterator it = node.getChildren().iterator(); it.hasNext(); ) {
-			BuildConfigNode currNode = (BuildConfigNode)it.next();
-			if (currNode.getBuildConfigNodeKind().equals(kind)
-				&& currNode.isActive()) {
+	private void getActiveNodesHelper(BuildConfigNode node, BuildConfigNode.Kind kind, List<BuildConfigNode> nodes) {
+		for (BuildConfigNode currNode : node.getChildren()) {
+			if (currNode.getBuildConfigNodeKind().equals(kind) && currNode.isActive()) {
 				nodes.add(currNode);
 			}
 			getActiveNodesHelper(currNode, kind, nodes);
-		}			
+		}
 	}
-	
+
 	public String getSourceFile() {
 		return sourceFile;
 	}
-	
+
 	public void setSourceFile(String sourceFile) {
 		this.sourceFile = sourceFile;
 	}
@@ -101,53 +101,45 @@ public class BuildConfigModel {
 	public void setRoot(BuildConfigNode node) {
 		root = node;
 	}
-	
+
 	public BuildConfigNode findNodeForSourceLine(String sourceFilePath, int lineNumber) {
 		BuildConfigNode node = findNodeForSourceLineHelper(root, sourceFilePath, lineNumber);
-		return node;	
+		return node;
 	}
-	
+
 	private BuildConfigNode findNodeForSourceLineHelper(BuildConfigNode node, String sourceFilePath, int lineNumber) {
-		if (matches(node, sourceFilePath, lineNumber) 
-			&& !hasMoreSpecificChild(node, sourceFilePath, lineNumber)) {
-			return node;	
-		} 
-		
+		if (matches(node, sourceFilePath, lineNumber) && !hasMoreSpecificChild(node, sourceFilePath, lineNumber)) {
+			return node;
+		}
+
 		if (node != null && node.getChildren() != null) {
-			for (Iterator it = node.getChildren().iterator(); it.hasNext(); ) {
-				BuildConfigNode foundNode = findNodeForSourceLineHelper(
-					(BuildConfigNode)it.next(), 
-					sourceFilePath, 
-					lineNumber); 		
-				if (foundNode != null) return foundNode;
+			for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
+				BuildConfigNode foundNode = findNodeForSourceLineHelper((BuildConfigNode) it.next(), sourceFilePath, lineNumber);
+				if (foundNode != null)
+					return foundNode;
 			}
 		}
-		return null;		
+		return null;
 	}
 
 	private boolean matches(BuildConfigNode node, String sourceFilePath, int lineNumber) {
-		try {			
-			return node != null 
-				&& node.getSourceLocation() != null
-				&& node.getSourceLocation().getSourceFile().getCanonicalPath().equals(sourceFilePath)
-				&& ((node.getSourceLocation().getLine() <= lineNumber
-					&& node.getSourceLocation().getEndLine() >= lineNumber)
-					||
-					(lineNumber <= 1)	
-				);
-		} catch (IOException ioe) { 
+		try {
+			return node != null
+					&& node.getSourceLocation() != null
+					&& node.getSourceLocation().getSourceFile().getCanonicalPath().equals(sourceFilePath)
+					&& ((node.getSourceLocation().getLine() <= lineNumber && node.getSourceLocation().getEndLine() >= lineNumber) || (lineNumber <= 1));
+		} catch (IOException ioe) {
 			return false;
-		} 
+		}
 	}
-	
+
 	private boolean hasMoreSpecificChild(BuildConfigNode node, String sourceFilePath, int lineNumber) {
-		for (Iterator it = node.getChildren().iterator(); it.hasNext(); ) {
-			BuildConfigNode child = (BuildConfigNode)it.next();
-			if (matches(child, sourceFilePath, lineNumber)) return true;
+		for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
+			BuildConfigNode child = (BuildConfigNode) it.next();
+			if (matches(child, sourceFilePath, lineNumber))
+				return true;
 		}
 		return false;
 	}
 
 }
-
-
