@@ -19,6 +19,7 @@ import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.security.ProtectionDomain;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.HashMap;
@@ -88,7 +89,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 	private StringBuffer namespace;
 	private IWeavingContext weavingContext;
 
-	private List concreteAspects = new ArrayList();
+	private List<ConcreteAspectCodeGen> concreteAspects = new ArrayList<ConcreteAspectCodeGen>();
 
 	private static Trace trace = TraceFactory.getTraceFactory().getTrace(ClassLoaderWeavingAdaptor.class);
 
@@ -137,8 +138,12 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 			} catch (Throwable throwable) {
 				throwable.printStackTrace();
 			}
+			if (activeProtectionDomain != null) {
+				defineClass(loaderRef.getClassLoader(), name, bytes, activeProtectionDomain);
+			} else {
+				defineClass(loaderRef.getClassLoader(), name, bytes); // could be done lazily using the hook
 
-			defineClass(loaderRef.getClassLoader(), name, bytes); // could be done lazily using the hook
+			}
 		}
 	}
 
@@ -258,7 +263,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 							definitions.add(DocumentParser.parse(xml));
 							seenBefore.add(xml);
 						} else {
-							warn("ignoring duplicate definition: " + xml);
+							debug("ignoring duplicate definition: " + xml);
 						}
 					}
 				}
@@ -314,10 +319,9 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 	 * @param loader
 	 * @param definitions
 	 */
-	private void registerOptions(final BcelWeaver weaver, final ClassLoader loader, final List definitions) {
+	private void registerOptions(final BcelWeaver weaver, final ClassLoader loader, final List<Definition> definitions) {
 		StringBuffer allOptions = new StringBuffer();
-		for (Iterator iterator = definitions.iterator(); iterator.hasNext();) {
-			Definition definition = (Definition) iterator.next();
+		for (Definition definition : definitions) {
 			allOptions.append(definition.getWeaverOptions()).append(' ');
 		}
 
@@ -532,8 +536,7 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 		}
 		boolean success = true;
 
-		for (Iterator iterator = concreteAspects.iterator(); iterator.hasNext();) {
-			ConcreteAspectCodeGen gen = (ConcreteAspectCodeGen) iterator.next();
+		for (ConcreteAspectCodeGen gen : concreteAspects) {
 			String name = gen.getClassName();
 			byte[] bytes = gen.getBytes();
 
@@ -560,13 +563,12 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 	 * @param loader
 	 * @param definitions
 	 */
-	private void registerIncludeExclude(final BcelWeaver weaver, final ClassLoader loader, final List definitions) {
+	private void registerIncludeExclude(final BcelWeaver weaver, final ClassLoader loader, final List<Definition> definitions) {
 		String fastMatchInfo = null;
-		for (Iterator iterator = definitions.iterator(); iterator.hasNext();) {
-			Definition definition = (Definition) iterator.next();
-			for (Iterator iterator1 = definition.getIncludePatterns().iterator(); iterator1.hasNext();) {
+		for (Definition definition : definitions) {
+			for (Iterator<String> iterator1 = definition.getIncludePatterns().iterator(); iterator1.hasNext();) {
 				hasIncludes = true;
-				String include = (String) iterator1.next();
+				String include = iterator1.next();
 				fastMatchInfo = looksLikeStartsWith(include);
 				if (fastMatchInfo != null) {
 					m_includeStartsWith.add(fastMatchInfo);
@@ -579,9 +581,9 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 					includeTypePattern.add(includePattern);
 				}
 			}
-			for (Iterator iterator1 = definition.getExcludePatterns().iterator(); iterator1.hasNext();) {
+			for (Iterator<String> iterator1 = definition.getExcludePatterns().iterator(); iterator1.hasNext();) {
 				hasExcludes = true;
-				String exclude = (String) iterator1.next();
+				String exclude = iterator1.next();
 				fastMatchInfo = looksLikeStartsWith(exclude);
 				if (fastMatchInfo != null) {
 					excludeStartsWith.add(fastMatchInfo);
@@ -701,11 +703,10 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 	 * @param loader
 	 * @param definitions
 	 */
-	private void registerDump(final BcelWeaver weaver, final ClassLoader loader, final List definitions) {
-		for (Iterator iterator = definitions.iterator(); iterator.hasNext();) {
-			Definition definition = (Definition) iterator.next();
-			for (Iterator iterator1 = definition.getDumpPatterns().iterator(); iterator1.hasNext();) {
-				String dump = (String) iterator1.next();
+	private void registerDump(final BcelWeaver weaver, final ClassLoader loader, final List<Definition> definitions) {
+		for (Definition definition : definitions) {
+			for (Iterator<String> iterator1 = definition.getDumpPatterns().iterator(); iterator1.hasNext();) {
+				String dump = iterator1.next();
 				TypePattern pattern = new PatternParser(dump).parseTypePattern();
 				m_dumpTypePattern.add(pattern);
 			}
@@ -977,6 +978,9 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 		generatedClasses = new HashMap();
 	}
 
+	private Method defineClassMethod;
+	private Method defineClassWithProtectionDomainMethod;
+
 	private void defineClass(ClassLoader loader, String name, byte[] bytes) {
 		if (trace.isTraceEnabled()) {
 			trace.enter("defineClass", this, new Object[] { loader, name, bytes });
@@ -985,11 +989,45 @@ public class ClassLoaderWeavingAdaptor extends WeavingAdaptor {
 		debug("generating class '" + name + "'");
 
 		try {
-			// TODO av protection domain, and optimize
-			Method defineClass = ClassLoader.class.getDeclaredMethod("defineClass", new Class[] { String.class, bytes.getClass(),
-					int.class, int.class });
-			defineClass.setAccessible(true);
-			clazz = defineClass.invoke(loader, new Object[] { name, bytes, new Integer(0), new Integer(bytes.length) });
+			if (defineClassMethod == null) {
+				defineClassMethod = ClassLoader.class.getDeclaredMethod("defineClass", new Class[] { String.class,
+						bytes.getClass(), int.class, int.class });
+			}
+			defineClassMethod.setAccessible(true);
+			clazz = defineClassMethod.invoke(loader, new Object[] { name, bytes, new Integer(0), new Integer(bytes.length) });
+		} catch (InvocationTargetException e) {
+			if (e.getTargetException() instanceof LinkageError) {
+				warn("define generated class failed", e.getTargetException());
+				// is already defined (happens for X$ajcMightHaveAspect interfaces since aspects are reweaved)
+				// TODO maw I don't think this is OK and
+			} else {
+				warn("define generated class failed", e.getTargetException());
+			}
+		} catch (Exception e) {
+			warn("define generated class failed", e);
+		}
+
+		if (trace.isTraceEnabled()) {
+			trace.exit("defineClass", clazz);
+		}
+	}
+
+	private void defineClass(ClassLoader loader, String name, byte[] bytes, ProtectionDomain protectionDomain) {
+		if (trace.isTraceEnabled()) {
+			trace.enter("defineClass", this, new Object[] { loader, name, bytes, protectionDomain });
+		}
+		Object clazz = null;
+		debug("generating class '" + name + "'");
+
+		try {
+			// System.out.println(">> Defining with protection domain " + name + " pd=" + protectionDomain);
+			if (defineClassWithProtectionDomainMethod == null) {
+				defineClassWithProtectionDomainMethod = ClassLoader.class.getDeclaredMethod("defineClass", new Class[] {
+						String.class, bytes.getClass(), int.class, int.class, ProtectionDomain.class });
+			}
+			defineClassWithProtectionDomainMethod.setAccessible(true);
+			clazz = defineClassWithProtectionDomainMethod.invoke(loader, new Object[] { name, bytes, Integer.valueOf(0),
+					new Integer(bytes.length), protectionDomain });
 		} catch (InvocationTargetException e) {
 			if (e.getTargetException() instanceof LinkageError) {
 				warn("define generated class failed", e.getTargetException());
