@@ -20,6 +20,9 @@ import java.util.Map;
 
 import org.aspectj.ajdt.internal.compiler.ast.AddAtAspectJAnnotationsVisitor;
 import org.aspectj.ajdt.internal.compiler.ast.AspectDeclaration;
+import org.aspectj.ajdt.internal.compiler.ast.InterTypeConstructorDeclaration;
+import org.aspectj.ajdt.internal.compiler.ast.InterTypeFieldDeclaration;
+import org.aspectj.ajdt.internal.compiler.ast.InterTypeMethodDeclaration;
 import org.aspectj.ajdt.internal.compiler.ast.ValidateAtAspectJAnnotationsVisitor;
 import org.aspectj.ajdt.internal.compiler.lookup.EclipseFactory;
 import org.aspectj.ajdt.internal.core.builder.AjState;
@@ -31,6 +34,7 @@ import org.aspectj.bridge.context.CompilationAndWeavingContext;
 import org.aspectj.bridge.context.ContextToken;
 import org.aspectj.org.eclipse.jdt.internal.compiler.CompilationResult;
 import org.aspectj.org.eclipse.jdt.internal.compiler.Compiler;
+import org.aspectj.org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Annotation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
@@ -39,6 +43,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
 import org.aspectj.weaver.bcel.BcelWeaver;
 import org.aspectj.weaver.bcel.BcelWorld;
+import org.aspectj.weaver.bcel.UnwovenClassFile;
 
 /**
  * Adapts standard JDT Compiler to add in AspectJ specific behaviours. This version implements pipelining - where files are compiled
@@ -96,13 +101,14 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	private boolean isXTerminateAfterCompilation;
 	private boolean proceedOnError;
 	private boolean inJava5Mode;
+	private boolean makeReflectable;
 	private boolean noAtAspectJAnnotationProcessing;
 	private IIntermediateResultsRequestor intermediateResultsRequestor;
 	private IProgressListener progressListener;
 	private IOutputClassFileNameProvider outputFileNameProvider;
 	private IBinarySourceProvider binarySourceProvider;
 	private WeaverMessageHandler weaverMessageHandler;
-	private Map /* fileName > List<UnwovenClassFile> */binarySourceSetForFullWeave = new HashMap();
+	private Map<String, List<UnwovenClassFile>> binarySourceSetForFullWeave = new HashMap<String, List<UnwovenClassFile>>();
 
 	private ContextToken processingToken = null;
 	private ContextToken resolvingToken = null;
@@ -112,7 +118,7 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	private AjState incrementalCompilationState;
 
 	// Maintains a list of whats weaving - whilst the pipeline is stalled, this accumulates aspects.
-	List /* InterimResult */resultsPendingWeave = new ArrayList();
+	List<InterimCompilationResult> resultsPendingWeave = new ArrayList<InterimCompilationResult>();
 
 	// pipelining info
 	private boolean pipelineStalled = true;
@@ -138,7 +144,7 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 			EclipseFactory eFactory, IIntermediateResultsRequestor intRequestor, IProgressListener progressListener,
 			IOutputClassFileNameProvider outputFileNameProvider, IBinarySourceProvider binarySourceProvider,
 			Map fullBinarySourceEntries, /* fileName |-> List<UnwovenClassFile> */
-			boolean isXterminateAfterCompilation, boolean proceedOnError, boolean noAtAspectJProcessing,
+			boolean isXterminateAfterCompilation, boolean proceedOnError, boolean noAtAspectJProcessing, boolean makeReflectable,
 			AjState incrementalCompilationState) {
 		this.compiler = compiler;
 		this.isBatchCompile = isBatchCompile;
@@ -152,11 +158,13 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 		this.binarySourceSetForFullWeave = fullBinarySourceEntries;
 		this.eWorld = eFactory;
 		this.inJava5Mode = false;
+		this.makeReflectable = makeReflectable;
 		this.noAtAspectJAnnotationProcessing = noAtAspectJProcessing;
 		this.incrementalCompilationState = incrementalCompilationState;
 
-		if (compiler.options.complianceLevel >= ClassFileConstants.JDK1_5)
+		if (compiler.options.complianceLevel >= ClassFileConstants.JDK1_5) {
 			inJava5Mode = true;
+		}
 		IMessageHandler msgHandler = world.getMessageHandler();
 		// Do we need to reset the message handler or create a new one? (This saves a ton of memory lost on incremental compiles...)
 		if (msgHandler instanceof WeaverMessageHandler) {
@@ -178,8 +186,9 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	 * weaving.
 	 */
 	public void afterDietParsing(CompilationUnitDeclaration[] units) {
-		if (debugPipeline)
+		if (debugPipeline) {
 			System.err.println("> afterDietParsing: there are " + (units == null ? 0 : units.length) + " units to sort");
+		}
 
 		if (!reportedErrors && units != null) {
 			for (int i = 0; i < units.length; i++) {
@@ -191,47 +200,53 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 		}
 
 		// Break the units into two lists...
-		List aspects = new ArrayList();
-		List nonaspects = new ArrayList();
+		List<CompilationUnitDeclaration> aspects = new ArrayList<CompilationUnitDeclaration>();
+		List<CompilationUnitDeclaration> nonaspects = new ArrayList<CompilationUnitDeclaration>();
 		for (int i = 0; i < units.length; i++) {
-			if (containsAnAspect(units[i]))
+			if (containsAnAspect(units[i])) {
 				aspects.add(units[i]);
-			else
+			} else {
 				nonaspects.add(units[i]);
+			}
 		}
 
 		// ...and put them back together, aspects first
 		int posn = 0;
-		for (Iterator iter = aspects.iterator(); iter.hasNext();) {
-			units[posn++] = (CompilationUnitDeclaration) iter.next();
+		for (CompilationUnitDeclaration aspect : aspects) {
+			units[posn++] = aspect;
 		}
-		for (Iterator iter = nonaspects.iterator(); iter.hasNext();) {
-			units[posn++] = (CompilationUnitDeclaration) iter.next();
+		for (CompilationUnitDeclaration nonaspect : nonaspects) {
+			units[posn++] = nonaspect;
 		}
 
 		// Work out how long to stall the pipeline
 		toWaitFor = aspects.size();
-		if (debugPipeline)
+		if (debugPipeline) {
 			System.err.println("< afterDietParsing: stalling pipeline for " + toWaitFor + " source files");
+		}
 
 		// TESTING
 		if (pipelineTesting) {
-			if (pipelineOutput == null)
+			if (pipelineOutput == null) {
 				pipelineOutput = new Hashtable();
+			}
 			pipelineOutput.put("filesContainingAspects", new Integer(toWaitFor).toString());
 			StringBuffer order = new StringBuffer();
 			order.append("[");
 			for (int i = 0; i < units.length; i++) {
-				if (i != 0)
+				if (i != 0) {
 					order.append(",");
+				}
 				CompilationUnitDeclaration declaration = units[i];
 				String filename = new String(declaration.getFileName());
 				int idx = filename.lastIndexOf('/');
-				if (idx > 0)
+				if (idx > 0) {
 					filename = filename.substring(idx + 1);
+				}
 				idx = filename.lastIndexOf('\\');
-				if (idx > 0)
+				if (idx > 0) {
 					filename = filename.substring(idx + 1);
+				}
 				order.append(filename);
 			}
 			order.append("]");
@@ -240,21 +255,22 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	}
 
 	public void beforeCompiling(ICompilationUnit[] sourceUnits) {
-		resultsPendingWeave = new ArrayList();
+		resultsPendingWeave = new ArrayList<InterimCompilationResult>();
 		reportedErrors = false;
 		droppingBackToFullBuild = false;
 	}
 
 	public void beforeProcessing(CompilationUnitDeclaration unit) {
-		if (debugPipeline)
+		if (debugPipeline) {
 			System.err.println("compiling " + new String(unit.getFileName()));
+		}
 		eWorld.showMessage(IMessage.INFO, "compiling " + new String(unit.getFileName()), null, null);
 		processingToken = CompilationAndWeavingContext.enteringPhase(CompilationAndWeavingContext.PROCESSING_COMPILATION_UNIT, unit
 				.getFileName());
 		if (inJava5Mode && !noAtAspectJAnnotationProcessing) {
 			ContextToken tok = CompilationAndWeavingContext.enteringPhase(
 					CompilationAndWeavingContext.ADDING_AT_ASPECTJ_ANNOTATIONS, unit.getFileName());
-			AddAtAspectJAnnotationsVisitor atAspectJVisitor = new AddAtAspectJAnnotationsVisitor(unit);
+			AddAtAspectJAnnotationsVisitor atAspectJVisitor = new AddAtAspectJAnnotationsVisitor(unit, makeReflectable);
 			unit.traverse(atAspectJVisitor, unit.scope);
 			CompilationAndWeavingContext.leavingPhase(tok);
 		}
@@ -266,8 +282,9 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	}
 
 	public void afterResolving(CompilationUnitDeclaration unit) {
-		if (resolvingToken != null)
+		if (resolvingToken != null) {
 			CompilationAndWeavingContext.leavingPhase(resolvingToken);
+		}
 	}
 
 	public void beforeAnalysing(CompilationUnitDeclaration unit) {
@@ -280,18 +297,72 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	}
 
 	public void afterAnalysing(CompilationUnitDeclaration unit) {
-		if (analysingToken != null)
+		if (analysingToken != null) {
 			CompilationAndWeavingContext.leavingPhase(analysingToken);
+		}
 	}
 
 	public void beforeGenerating(CompilationUnitDeclaration unit) {
 		generatingToken = CompilationAndWeavingContext.enteringPhase(
 				CompilationAndWeavingContext.GENERATING_UNWOVEN_CODE_FOR_COMPILATION_UNIT, unit.getFileName());
+		if (eWorld.pushinCollector != null) {
+			if (unit.types != null && unit.types.length > 0) {
+				for (int t = 0; t < unit.types.length; t++) {
+					TypeDeclaration type = unit.types[t];
+					if (type.methods != null) {
+						for (int m = 0; m < type.methods.length; m++) {
+							AbstractMethodDeclaration md = type.methods[m];
+							if (md instanceof InterTypeMethodDeclaration) {
+								InterTypeMethodDeclaration itmd = ((InterTypeMethodDeclaration) md);
+								ITDMethodPrinter printer = new ITDMethodPrinter(itmd, md.scope);
+								String s = printer.print();
+								eWorld.pushinCollector.recordInterTypeMethodDeclarationCode(md, s, getDeclarationLineNumber(md));
+							} else if (md instanceof InterTypeFieldDeclaration) {
+								ITDFieldPrinter printer = new ITDFieldPrinter(((InterTypeFieldDeclaration) md), md.scope);
+								String s = printer.print();
+								eWorld.pushinCollector.recordInterTypeFieldDeclarationCode(md, s, getDeclarationLineNumber(md));
+							} else if (md instanceof InterTypeConstructorDeclaration) {
+								ITDConstructorPrinter printer = new ITDConstructorPrinter(((InterTypeConstructorDeclaration) md),
+										md.scope);
+								String s = printer.print();
+								eWorld.pushinCollector.recordInterTypeConstructorDeclarationCode(md, s,
+										getDeclarationLineNumber(md));
+								// } else if (md instanceof DeclareAnnotationDeclaration) {
+								// DeclareAnnotationDeclaration dad = (DeclareAnnotationDeclaration) md;
+								// String value = new DeclareAnnotationsPrinter(dad, dad.scope).print();
+								// eWorld.pushinCollector.recordDeclareAnnotationDeclarationCode(md, value);
+							}
+						}
+					}
+				}
+			}
+			eWorld.pushinCollector.setOutputFileNameProvider(outputFileNameProvider);
+		}
+	}
+
+	/**
+	 * @return the line number for this declaration in the source code
+	 */
+	private int getDeclarationLineNumber(AbstractMethodDeclaration md) {
+		int sourceStart = md.sourceStart;
+		int[] separators = md.compilationResult.lineSeparatorPositions;
+		int declarationStartLine = 1;
+		for (int i = 0; i < separators.length; i++) {
+			if (sourceStart < separators[i]) {
+				break;
+			}
+			declarationStartLine++;
+		}
+		return declarationStartLine;
 	}
 
 	public void afterGenerating(CompilationUnitDeclaration unit) {
-		if (generatingToken != null)
+		if (generatingToken != null) {
 			CompilationAndWeavingContext.leavingPhase(generatingToken);
+		}
+		if (eWorld.pushinCollector != null) {
+			eWorld.pushinCollector.dump(unit);
+		}
 	}
 
 	public void afterCompiling(CompilationUnitDeclaration[] units) {
@@ -301,8 +372,9 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 				// acceptResult(unit.compilationResult);
 				// } else {
 				try {
-					if (weaveQueuedEntries())
+					if (weaveQueuedEntries()) {
 						droppingBackToFullBuild = true;
+					}
 				} catch (IOException ex) {
 					AbortCompilation ac = new AbortCompilation(null, ex);
 					throw ac;
@@ -331,8 +403,9 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 			// AbortCompilation ac = new AbortCompilation(null,ex);
 			// throw ac;
 		} catch (RuntimeException rEx) {
-			if (rEx instanceof AbortCompilation)
+			if (rEx instanceof AbortCompilation) {
 				throw rEx; // Don't wrap AbortCompilation exceptions!
+			}
 
 			// This will be unwrapped in Compiler.handleInternalException() and the nested
 			// RuntimeException thrown back to the original caller - which is AspectJ
@@ -345,8 +418,9 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 		CompilationAndWeavingContext.leavingPhase(processingToken);
 		eWorld.finishedCompilationUnit(unit);
 		InterimCompilationResult intRes = new InterimCompilationResult(unit.compilationResult, outputFileNameProvider);
-		if (unit.compilationResult.hasErrors())
+		if (unit.compilationResult.hasErrors()) {
 			reportedErrors = true;
+		}
 
 		if (intermediateResultsRequestor != null) {
 			intermediateResultsRequestor.acceptResult(intRes);
@@ -362,14 +436,17 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	private void queueForWeaving(InterimCompilationResult intRes) {
 		resultsPendingWeave.add(intRes);
 		if (pipelineStalled) {
-			if (resultsPendingWeave.size() >= toWaitFor)
+			if (resultsPendingWeave.size() >= toWaitFor) {
 				pipelineStalled = false;
+			}
 		}
-		if (pipelineStalled)
+		if (pipelineStalled) {
 			return;
+		}
 		try {
-			if (weaveQueuedEntries())
+			if (weaveQueuedEntries()) {
 				droppingBackToFullBuild = true;
+			}
 		} catch (IOException ex) {
 			AbortCompilation ac = new AbortCompilation(null, ex);
 			throw ac;
@@ -396,12 +473,12 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	// helper methods...
 	// ==================================================================================
 
-	private List getBinarySourcesFrom(Map binarySourceEntries) {
+	private List<InterimCompilationResult> getBinarySourcesFrom(Map<String, List<UnwovenClassFile>> binarySourceEntries) {
 		// Map is fileName |-> List<UnwovenClassFile>
-		List ret = new ArrayList();
-		for (Iterator binIter = binarySourceEntries.keySet().iterator(); binIter.hasNext();) {
-			String sourceFileName = (String) binIter.next();
-			List unwovenClassFiles = (List) binarySourceEntries.get(sourceFileName);
+		List<InterimCompilationResult> ret = new ArrayList<InterimCompilationResult>();
+		for (Iterator<String> binIter = binarySourceEntries.keySet().iterator(); binIter.hasNext();) {
+			String sourceFileName = binIter.next();
+			List<UnwovenClassFile> unwovenClassFiles = binarySourceEntries.get(sourceFileName);
 			// XXX - see bugs 57432,58679 - final parameter on next call should be "compiler.options.maxProblemsPerUnit"
 			CompilationResult result = new CompilationResult(sourceFileName.toCharArray(), 0, 0, Integer.MAX_VALUE);
 			result.noSourceAvailable();
@@ -420,29 +497,33 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 
 	/** Return true if we've decided to drop back to a full build (too much has changed) */
 	private boolean weaveQueuedEntries() throws IOException {
-		if (debugPipeline)
+		if (debugPipeline) {
 			System.err.println(">.weaveQueuedEntries()");
+		}
 		for (Iterator iter = resultsPendingWeave.iterator(); iter.hasNext();) {
 			InterimCompilationResult iresult = (InterimCompilationResult) iter.next();
 			for (int i = 0; i < iresult.unwovenClassFiles().length; i++) {
-				weaver.addClassFile(iresult.unwovenClassFiles()[i],false);
+				weaver.addClassFile(iresult.unwovenClassFiles()[i], false);
 			}
 		}
 		ensureWeaverInitialized(); // by doing this only once, are we saying needToReweaveWorld can't change once the aspects have
 		// been stuffed into the weaver?
-		if (weaver.needToReweaveWorld() && !isBatchCompile)
+		if (weaver.needToReweaveWorld() && !isBatchCompile) {
 			return true;
+		}
 		weaver.weave(new WeaverAdapter(this, weaverMessageHandler, progressListener));
 		resultsPendingWeave.clear(); // dont need to do those again
 		this.eWorld.minicleanup();
-		if (debugPipeline)
+		if (debugPipeline) {
 			System.err.println("<.weaveQueuedEntries()");
+		}
 		return false;
 	}
 
 	private void ensureWeaverInitialized() {
-		if (weaverInitialized)
+		if (weaverInitialized) {
 			return;
+		}
 		weaverInitialized = true;
 		weaver.setIsBatchWeave(isBatchCompile);
 		weaver.prepareForWeave();
@@ -495,18 +576,23 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	// }
 
 	private void postWeave() {
-		if (debugPipeline)
+		if (debugPipeline) {
 			System.err.println("> postWeave()");
+		}
 		IMessageHandler imh = weaver.getWorld().getMessageHandler();
-		if (imh instanceof WeaverMessageHandler)
+		if (imh instanceof WeaverMessageHandler) {
 			((WeaverMessageHandler) imh).setCurrentResult(null);
-		if (!droppingBackToFullBuild)
+		}
+		if (!droppingBackToFullBuild) {
 			weaver.allWeavingComplete();
+		}
 		weaver.tidyUp();
-		if (imh instanceof WeaverMessageHandler)
+		if (imh instanceof WeaverMessageHandler) {
 			((WeaverMessageHandler) imh).resetCompiler(null);
-		if (debugPipeline)
+		}
+		if (debugPipeline) {
 			System.err.println("< postWeave()");
+		}
 	}
 
 	/**
@@ -518,13 +604,15 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 		if (typeDecls != null) {
 			for (int i = 0; i < typeDecls.length; i++) { // loop through top level types in the file
 				TypeDeclaration declaration = typeDecls[i];
-				if (isAspect(declaration))
+				if (isAspect(declaration)) {
 					return true;
+				}
 				if (declaration.memberTypes != null) {
 					TypeDeclaration[] memberTypes = declaration.memberTypes;
 					for (int j = 0; j < memberTypes.length; j++) { // loop through inner types
-						if (containsAnAspect(memberTypes[j]))
+						if (containsAnAspect(memberTypes[j])) {
 							return true;
+						}
 					}
 				}
 			}
@@ -533,13 +621,15 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	}
 
 	private boolean containsAnAspect(TypeDeclaration tDecl) {
-		if (isAspect(tDecl))
+		if (isAspect(tDecl)) {
 			return true;
+		}
 		if (tDecl.memberTypes != null) {
 			TypeDeclaration[] memberTypes = tDecl.memberTypes;
 			for (int j = 0; j < memberTypes.length; j++) { // loop through inner types
-				if (containsAnAspect(memberTypes[j]))
+				if (containsAnAspect(memberTypes[j])) {
 					return true;
+				}
 			}
 		}
 		return false;
@@ -549,21 +639,24 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 
 	private boolean isAspect(TypeDeclaration declaration) {
 		// avoid an NPE when something else is wrong in this system ... the real problem will be reported elsewhere
-		if (declaration.staticInitializerScope == null)
+		if (declaration.staticInitializerScope == null) {
 			return false;
-		if (declaration instanceof AspectDeclaration)
+		}
+		if (declaration instanceof AspectDeclaration) {
 			return true; // code style
-		else if (declaration.annotations != null) { // check for annotation style
+		} else if (declaration.annotations != null) { // check for annotation style
 			for (int index = 0; index < declaration.annotations.length; index++) {
 				TypeDeclaration
 						.resolveAnnotations(declaration.staticInitializerScope, declaration.annotations, declaration.binding); // force
 				// annotation
 				// resolution
 				Annotation a = declaration.annotations[index];
-				if (a.resolvedType == null)
+				if (a.resolvedType == null) {
 					continue; // another problem is being reported, so don't crash here
-				if (CharOperation.equals(a.resolvedType.signature(), aspectSig))
+				}
+				if (CharOperation.equals(a.resolvedType.signature(), aspectSig)) {
 					return true;
+				}
 			}
 		}
 		return false;
@@ -574,7 +667,7 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	 * SECRET: FOR TESTING - this can be used to collect information that tests can verify.
 	 */
 	public static boolean pipelineTesting = false;
-	public static Hashtable pipelineOutput = null;
+	public static Hashtable<String, String> pipelineOutput = null;
 
 	// Keys into pipelineOutput:
 	// compileOrder "[XXX,YYY]" a list of the order in which files will be woven (aspects should be first)
@@ -582,14 +675,15 @@ public class AjPipeliningCompilerAdapter extends AbstractCompilerAdapter {
 	// 
 
 	public static String getPipelineDebugOutput(String key) {
-		if (pipelineOutput == null)
+		if (pipelineOutput == null) {
 			return "";
-		return (String) pipelineOutput.get(key);
+		}
+		return pipelineOutput.get(key);
 	}
 
 	private final static boolean debugPipeline = false;
 
-	public List getResultsPendingWeave() {
+	public List<InterimCompilationResult> getResultsPendingWeave() {
 		return resultsPendingWeave;
 	}
 
