@@ -16,6 +16,9 @@ import java.lang.reflect.Modifier;
 
 import org.aspectj.ajdt.internal.compiler.lookup.EclipseFactory;
 import org.aspectj.ajdt.internal.compiler.lookup.EclipseTypeMunger;
+import org.aspectj.ajdt.internal.compiler.lookup.PrivilegedFieldBinding;
+import org.aspectj.ajdt.internal.compiler.lookup.PrivilegedHandler;
+import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ClassFile;
 import org.aspectj.org.eclipse.jdt.internal.compiler.CompilationResult;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ast.Argument;
@@ -31,6 +34,9 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ClassScope;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Scope;
+import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.Parser;
 import org.aspectj.weaver.AjAttribute;
@@ -76,8 +82,9 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 
 	public void resolveOnType(ClassScope classScope) {
 		super.resolveOnType(classScope);
-		if (ignoreFurtherInvestigation)
+		if (ignoreFurtherInvestigation) {
 			return;
+		}
 		if (Modifier.isStatic(declaredModifiers) && onTypeBinding.isInterface()) {
 			scope.problemReporter().signalError(sourceStart, sourceEnd, "static inter-type field on interface not supported");
 			ignoreFurtherInvestigation = true;
@@ -91,17 +98,19 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 	}
 
 	public void resolve(ClassScope upperScope) {
-		if (munger == null)
+		if (munger == null) {
 			ignoreFurtherInvestigation = true;
-		if (ignoreFurtherInvestigation)
+		}
+		if (ignoreFurtherInvestigation) {
 			return;
+		}
 
 		EclipseFactory world = EclipseFactory.fromScopeLookupEnvironment(upperScope);
 		ResolvedMember sig = munger.getSignature();
 		UnresolvedType aspectType = world.fromBinding(upperScope.referenceContext.binding);
 
-		if (sig.getReturnType() == ResolvedType.VOID
-				|| (sig.getReturnType().isArray() && (sig.getReturnType().getComponentType() == ResolvedType.VOID))) {
+		if (sig.getReturnType().equals(UnresolvedType.VOID)
+				|| (sig.getReturnType().isArray() && (sig.getReturnType().getComponentType().equals(UnresolvedType.VOID)))) {
 			upperScope.problemReporter().signalError(sourceStart, sourceEnd, "field type can not be void");
 		}
 
@@ -157,28 +166,29 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 		if (initialization == null) {
 			this.statements = new Statement[] { new ReturnStatement(null, 0, 0), };
 		} else if (!onTypeBinding.isInterface()) {
-			MethodBinding writeMethod = world.makeMethodBinding(AjcMemberMaker.interFieldSetDispatcher(sig, aspectType), munger
-					.getTypeVariableAliases());
+			MethodBinding writeMethod = world.makeMethodBinding(AjcMemberMaker.interFieldSetDispatcher(sig, aspectType),
+					munger.getTypeVariableAliases());
 			// For the body of an intertype field initalizer, generate a call to the inter field set dispatcher
 			// method as that casts the shadow of a field set join point.
 			if (Modifier.isStatic(declaredModifiers)) {
-				this.statements = new Statement[] { new KnownMessageSend(writeMethod, AstUtil
-						.makeNameReference(writeMethod.declaringClass), new Expression[] { initialization }), };
+				this.statements = new Statement[] { new KnownMessageSend(writeMethod,
+						AstUtil.makeNameReference(writeMethod.declaringClass), new Expression[] { initialization }), };
 			} else {
-				this.statements = new Statement[] { new KnownMessageSend(writeMethod, AstUtil
-						.makeNameReference(writeMethod.declaringClass), new Expression[] {
-						AstUtil.makeLocalVariableReference(arguments[0].binding), initialization }), };
+				this.statements = new Statement[] { new KnownMessageSend(writeMethod,
+						AstUtil.makeNameReference(writeMethod.declaringClass), new Expression[] {
+								AstUtil.makeLocalVariableReference(arguments[0].binding), initialization }), };
 			}
 		} else {
 			// XXX something is broken about this logic. Can we write to static interface fields?
-			MethodBinding writeMethod = world.makeMethodBinding(AjcMemberMaker.interFieldInterfaceSetter(sig, sig
-					.getDeclaringType().resolve(world.getWorld()), aspectType), munger.getTypeVariableAliases());
+			MethodBinding writeMethod = world.makeMethodBinding(
+					AjcMemberMaker.interFieldInterfaceSetter(sig, sig.getDeclaringType().resolve(world.getWorld()), aspectType),
+					munger.getTypeVariableAliases());
 			if (Modifier.isStatic(declaredModifiers)) {
-				this.statements = new Statement[] { new KnownMessageSend(writeMethod, AstUtil
-						.makeNameReference(writeMethod.declaringClass), new Expression[] { initialization }), };
+				this.statements = new Statement[] { new KnownMessageSend(writeMethod,
+						AstUtil.makeNameReference(writeMethod.declaringClass), new Expression[] { initialization }), };
 			} else {
-				this.statements = new Statement[] { new KnownMessageSend(writeMethod, AstUtil
-						.makeLocalVariableReference(arguments[0].binding), new Expression[] { initialization }), };
+				this.statements = new Statement[] { new KnownMessageSend(writeMethod,
+						AstUtil.makeLocalVariableReference(arguments[0].binding), new Expression[] { initialization }), };
 			}
 		}
 
@@ -226,17 +236,21 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 		EclipseFactory world = EclipseFactory.fromScopeLookupEnvironment(classScope);
 		resolveOnType(classScope);
 
-		if (ignoreFurtherInvestigation)
+		if (ignoreFurtherInvestigation) {
 			return null;
+		}
 
 		binding = classScope.referenceContext.binding.resolveTypesFor(binding);
-		if (ignoreFurtherInvestigation)
+		if (ignoreFurtherInvestigation) {
 			return null;
+		}
 
-		if (isTargetAnnotation(classScope, "field"))
+		if (isTargetAnnotation(classScope, "field")) {
 			return null; // Error message output in isTargetAnnotation
-		if (isTargetEnum(classScope, "field"))
+		}
+		if (isTargetEnum(classScope, "field")) {
 			return null; // Error message output in isTargetEnum
+		}
 
 		if (!Modifier.isStatic(declaredModifiers)) {
 			super.binding.parameters = new TypeBinding[] { onTypeBinding, };
@@ -249,8 +263,9 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 			declaringType = declaringType.getGenericType();
 		}
 
-		if (interTypeScope == null)
+		if (interTypeScope == null) {
 			return null; // We encountered a problem building the scope, don't continue - error already reported
+		}
 
 		// Build a half correct resolvedmember (makeResolvedMember understands tvars) then build a fully correct sig from it
 		ResolvedMember sigtemp = world.makeResolvedMemberForITD(binding, onTypeBinding, interTypeScope.getRecoveryAliases());
@@ -277,8 +292,9 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 	}
 
 	public void generateCode(ClassScope classScope, ClassFile classFile) {
-		if (ignoreFurtherInvestigation)
+		if (ignoreFurtherInvestigation) {
 			return;
+		}
 
 		classFile.extraAttributes.add(new EclipseAttributeAdapter(makeAttribute()));
 		super.generateCode(classScope, classFile);
@@ -299,39 +315,59 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 			ClassFile classFile, boolean isGetter) {
 		MethodBinding binding;
 		if (isGetter) {
-			binding = world.makeMethodBinding(AjcMemberMaker.interFieldGetDispatcher(sig, aspectType), munger
-					.getTypeVariableAliases(), munger.getSignature().getDeclaringType());
+			binding = world.makeMethodBinding(AjcMemberMaker.interFieldGetDispatcher(sig, aspectType),
+					munger.getTypeVariableAliases(), munger.getSignature().getDeclaringType());
 		} else {
-			binding = world.makeMethodBinding(AjcMemberMaker.interFieldSetDispatcher(sig, aspectType), munger
-					.getTypeVariableAliases(), munger.getSignature().getDeclaringType());
+			binding = world.makeMethodBinding(AjcMemberMaker.interFieldSetDispatcher(sig, aspectType),
+					munger.getTypeVariableAliases(), munger.getSignature().getDeclaringType());
 		}
 		classFile.generateMethodInfoHeader(binding);
 		int methodAttributeOffset = classFile.contentsOffset;
-		int attributeNumber = classFile.generateMethodInfoAttribute(binding, false, makeEffectiveSignatureAttribute(sig,
-				isGetter ? Shadow.FieldGet : Shadow.FieldSet, false));
+		int attributeNumber = classFile.generateMethodInfoAttribute(binding, false,
+				makeEffectiveSignatureAttribute(sig, isGetter ? Shadow.FieldGet : Shadow.FieldSet, false));
 		int codeAttributeOffset = classFile.contentsOffset;
 		classFile.generateCodeAttributeHeader();
 		CodeStream codeStream = classFile.codeStream;
 		codeStream.reset(this, classFile);
 
-		FieldBinding classField = world.makeFieldBinding(AjcMemberMaker.interFieldClassField(sig, aspectType), munger
-				.getTypeVariableAliases());
+		NewFieldTypeMunger fieldMunger = (NewFieldTypeMunger) munger;
+
+		// Force use of version 1 if there is a field with that name on the type already
+		if (world.getItdVersion() == 1) {
+			fieldMunger.version = NewFieldTypeMunger.VersionOne;
+		} else {
+			if (!onTypeBinding.isInterface()) {
+				FieldBinding[] existingFields = onTypeBinding.fields();
+				for (int f = 0; f < existingFields.length; f++) {
+					FieldBinding fieldBinding = existingFields[f];
+					if (CharOperation.equals(fieldBinding.name, sig.getName().toCharArray())) {
+						fieldMunger.version = NewFieldTypeMunger.VersionOne;
+					}
+				}
+			}
+		}
+
+		FieldBinding classField = world.makeFieldBinding(
+				AjcMemberMaker.interFieldClassField(sig, aspectType, fieldMunger.version == NewFieldTypeMunger.VersionTwo),
+				munger.getTypeVariableAliases());
 
 		codeStream.initializeMaxLocals(binding);
 		if (isGetter) {
 			if (onTypeBinding.isInterface()) {
 				UnresolvedType declaringTX = sig.getDeclaringType();
 				ResolvedType declaringRTX = world.getWorld().resolve(declaringTX, munger.getSourceLocation());
-				MethodBinding readMethod = world.makeMethodBinding(AjcMemberMaker.interFieldInterfaceGetter(sig, declaringRTX,
-						aspectType), munger.getTypeVariableAliases());
+				MethodBinding readMethod = world.makeMethodBinding(
+						AjcMemberMaker.interFieldInterfaceGetter(sig, declaringRTX, aspectType), munger.getTypeVariableAliases());
 				generateInterfaceReadBody(binding, readMethod, codeStream);
 			} else {
 				generateClassReadBody(binding, classField, codeStream);
 			}
 		} else {
 			if (onTypeBinding.isInterface()) {
-				MethodBinding writeMethod = world.makeMethodBinding(AjcMemberMaker.interFieldInterfaceSetter(sig, world.getWorld()
-						.resolve(sig.getDeclaringType(), munger.getSourceLocation()), aspectType), munger.getTypeVariableAliases());
+				MethodBinding writeMethod = world.makeMethodBinding(
+						AjcMemberMaker.interFieldInterfaceSetter(sig,
+								world.getWorld().resolve(sig.getDeclaringType(), munger.getSourceLocation()), aspectType),
+						munger.getTypeVariableAliases());
 				generateInterfaceWriteBody(binding, writeMethod, codeStream);
 			} else {
 				generateClassWriteBody(binding, classField, codeStream);
@@ -356,6 +392,28 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 	}
 
 	private void generateClassReadBody(MethodBinding binding, FieldBinding field, CodeStream codeStream) {
+		if (field.isPrivate() || !field.canBeSeenBy(binding.declaringClass.fPackage)) {
+
+			PrivilegedHandler handler = (PrivilegedHandler) Scope.findPrivilegedHandler(binding.declaringClass);
+			if (handler == null) {
+				// one is now required!
+				ReferenceBinding typebinding = binding.declaringClass;
+				if (typebinding instanceof ReferenceBinding) {
+					SourceTypeBinding sourceBinding = (SourceTypeBinding) typebinding;
+					handler = new PrivilegedHandler((AspectDeclaration) sourceBinding.scope.referenceContext);
+					sourceBinding.privilegedHandler = handler;
+				}
+			}
+			PrivilegedFieldBinding fBinding = (PrivilegedFieldBinding) handler.getPrivilegedAccessField(field, null);
+
+			if (field.isStatic()) {
+				codeStream.invokestatic(fBinding.reader);
+			} else {
+				codeStream.aload_0();
+				codeStream.invokestatic(fBinding.reader);
+			}
+			return;
+		}
 		if (field.isStatic()) {
 			codeStream.getstatic(field);
 		} else {
@@ -365,6 +423,20 @@ public class InterTypeFieldDeclaration extends InterTypeDeclaration {
 	}
 
 	private void generateClassWriteBody(MethodBinding binding, FieldBinding field, CodeStream codeStream) {
+		if (field.isPrivate() || !field.canBeSeenBy(binding.declaringClass.fPackage)) {
+			PrivilegedFieldBinding fBinding = (PrivilegedFieldBinding) Scope.findPrivilegedHandler(binding.declaringClass)
+					.getPrivilegedAccessField(field, null);
+
+			if (field.isStatic()) {
+				codeStream.load(field.type, 0);
+				codeStream.invokestatic(fBinding.writer);
+			} else {
+				codeStream.aload_0();
+				codeStream.load(field.type, 1);
+				codeStream.invokestatic(fBinding.writer);
+			}
+			return;
+		}
 		if (field.isStatic()) {
 			codeStream.load(field.type, 0);
 			codeStream.putstatic(field);
