@@ -232,12 +232,12 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		// - weaving that brings new types in for processing (see
 		// pendingTypesToWeave.add() calls) after we thought
 		// we had the full list.
-		// 
+		//
 		// but these aren't common cases (he bravely said...)
 		boolean typeProcessingOrderIsImportant = declareParents.size() > 0 || declareAnnotationOnTypes.size() > 0; // DECAT
 
 		if (typeProcessingOrderIsImportant) {
-			typesToProcess = new ArrayList();
+			typesToProcess = new ArrayList<SourceTypeBinding>();
 			for (int i = lastCompletedUnitIndex + 1; i <= lastUnitIndex; i++) {
 				CompilationUnitScope cus = units[i].scope;
 				SourceTypeBinding[] stbs = cus.topLevelTypes;
@@ -247,10 +247,17 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 				}
 			}
 
+			List<SourceTypeBinding> stb2 = new ArrayList<SourceTypeBinding>();
+			stb2.addAll(typesToProcess);
+
 			while (typesToProcess.size() > 0) {
-				// A side effect of weaveIntertypes() is that the processed type
-				// is removed from the collection
-				weaveIntertypes(typesToProcess, typesToProcess.get(0), typeMungers, declareParents, declareAnnotationOnTypes);
+				// A side effect of weaveIntertypes() is that the processed type is removed from the collection
+				weaveIntertypes(typesToProcess, typesToProcess.get(0), typeMungers, declareParents, declareAnnotationOnTypes, 1);
+			}
+
+			while (stb2.size() > 0) {
+				// A side effect of weaveIntertypes() is that the processed type is removed from the collection
+				weaveIntertypes(stb2, stb2.get(0), typeMungers, declareParents, declareAnnotationOnTypes, 2);
 			}
 
 		} else {
@@ -383,16 +390,19 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 	 * compile... it stops recursing the first time it hits a type we aren't going to process during this compile. This could cause
 	 * problems if you supply 'pieces' of a hierarchy, i.e. the bottom and the top, but not the middle - but what the hell are you
 	 * doing if you do that?
+	 * 
+	 * @param mode 0=do everything, 1=do declare parents, 2=do ITDs
 	 */
 	private void weaveIntertypes(List<SourceTypeBinding> typesToProcess, SourceTypeBinding typeToWeave,
 			List<ConcreteTypeMunger> typeMungers, List<DeclareParents> declareParents,
-			List<DeclareAnnotation> declareAnnotationOnTypes) {
+			List<DeclareAnnotation> declareAnnotationOnTypes, int mode) {
 		// Look at the supertype first
 		ReferenceBinding superType = typeToWeave.superclass();
 		if (typesToProcess.contains(superType) && superType instanceof SourceTypeBinding) {
 			// System.err.println("Recursing to supertype "+new
 			// String(superType.getFileName()));
-			weaveIntertypes(typesToProcess, (SourceTypeBinding) superType, typeMungers, declareParents, declareAnnotationOnTypes);
+			weaveIntertypes(typesToProcess, (SourceTypeBinding) superType, typeMungers, declareParents, declareAnnotationOnTypes,
+					mode);
 		}
 		// Then look at the superinterface list
 		ReferenceBinding[] interfaceTypes = typeToWeave.superInterfaces();
@@ -401,10 +411,11 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			if (typesToProcess.contains(binding) && binding instanceof SourceTypeBinding) {
 				// System.err.println("Recursing to superinterface "+new
 				// String(binding.getFileName()));
-				weaveIntertypes(typesToProcess, (SourceTypeBinding) binding, typeMungers, declareParents, declareAnnotationOnTypes);
+				weaveIntertypes(typesToProcess, (SourceTypeBinding) binding, typeMungers, declareParents, declareAnnotationOnTypes,
+						mode);
 			}
 		}
-		weaveInterTypeDeclarations(typeToWeave, typeMungers, declareParents, declareAnnotationOnTypes, false);
+		weaveInterTypeDeclarations(typeToWeave, typeMungers, declareParents, declareAnnotationOnTypes, false, mode);
 		typesToProcess.remove(typeToWeave);
 	}
 
@@ -532,8 +543,13 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		}
 
 		ReferenceBinding[] memberTypes = sourceType.memberTypes;
-		for (int i = 0, length = memberTypes.length; i < length; i++) {
-			buildInterTypeAndPerClause(((SourceTypeBinding) memberTypes[i]).scope);
+		if (memberTypes == null) {
+			System.err.println("Unexpectedly found null for memberTypes of " + sourceType.debugName());
+		}
+		if (memberTypes != null) {
+			for (int i = 0, length = memberTypes.length; i < length; i++) {
+				buildInterTypeAndPerClause(((SourceTypeBinding) memberTypes[i]).scope);
+			}
 		}
 	}
 
@@ -579,7 +595,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 	private void weaveInterTypeDeclarations(CompilationUnitScope unit, List<ConcreteTypeMunger> typeMungers,
 			List<DeclareParents> declareParents, List<DeclareAnnotation> declareAnnotationOnTypes) {
 		for (int i = 0, length = unit.topLevelTypes.length; i < length; i++) {
-			weaveInterTypeDeclarations(unit.topLevelTypes[i], typeMungers, declareParents, declareAnnotationOnTypes, false);
+			weaveInterTypeDeclarations(unit.topLevelTypes[i], typeMungers, declareParents, declareAnnotationOnTypes, false, 0);
 		}
 	}
 
@@ -588,7 +604,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			if (!pendingTypesToWeave.contains(sourceType)) {
 				pendingTypesToWeave.add(sourceType);
 
-// inner type ITD support - may need this for some incremental cases...
+				// inner type ITD support - may need this for some incremental cases...
 				// List<ConcreteTypeMunger> ctms = factory.getWorld().getCrosscuttingMembersSet().getTypeMungersOfKind(
 				// ResolvedTypeMunger.InnerClass);
 				// // List<ConcreteTypeMunger> innerTypeMungers = new ArrayList<ConcreteTypeMunger>();
@@ -633,13 +649,16 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 
 			}
 		} else {
-			weaveInterTypeDeclarations(sourceType, factory.getTypeMungers(), factory.getDeclareParents(), factory
-					.getDeclareAnnotationOnTypes(), true);
+			weaveInterTypeDeclarations(sourceType, factory.getTypeMungers(), factory.getDeclareParents(),
+					factory.getDeclareAnnotationOnTypes(), true, 0);
 		}
 	}
 
+	/**
+	 * @param mode 0=do everything, 1=do declare parents, 2=do ITDs
+	 */
 	private void weaveInterTypeDeclarations(SourceTypeBinding sourceType, List<ConcreteTypeMunger> typeMungers,
-			List<DeclareParents> declareParents, List<DeclareAnnotation> declareAnnotationOnTypes, boolean skipInners) {
+			List<DeclareParents> declareParents, List<DeclareAnnotation> declareAnnotationOnTypes, boolean skipInners, int mode) {
 
 		ContextToken tok = CompilationAndWeavingContext.enteringPhase(CompilationAndWeavingContext.WEAVING_INTERTYPE_DECLARATIONS,
 				sourceType.sourceName);
@@ -653,148 +672,148 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 
 		WeaverStateInfo info = onType.getWeaverState();
 
-		// this test isnt quite right - there will be a case where we fail to
-		// flag a problem
-		// with a 'dangerous interface' because the type is reweavable when we
-		// should have
-		// because the type wasn't going to be rewoven... if that happens, we
-		// should perhaps
-		// move this test and dangerous interface processing to the end of this
-		// method and
-		// make it conditional on whether any of the typeMungers passed into
-		// here actually
-		// matched this type.
-		if (info != null && !info.isOldStyle() && !info.isReweavable()) {
-			processTypeMungersFromExistingWeaverState(sourceType, onType);
-			CompilationAndWeavingContext.leavingPhase(tok);
-			return;
-		}
-
-		// Check if the type we are looking at is the topMostImplementor of a
-		// dangerous interface -
-		// report a problem if it is.
-		for (Iterator i = dangerousInterfaces.entrySet().iterator(); i.hasNext();) {
-			Map.Entry entry = (Map.Entry) i.next();
-			ResolvedType interfaceType = (ResolvedType) entry.getKey();
-			if (onType.isTopmostImplementor(interfaceType)) {
-				factory.showMessage(IMessage.ERROR, onType + ": " + entry.getValue(), onType.getSourceLocation(), null);
+		if (mode < 2) {
+			// this test isnt quite right - there will be a case where we fail to
+			// flag a problem
+			// with a 'dangerous interface' because the type is reweavable when we
+			// should have
+			// because the type wasn't going to be rewoven... if that happens, we
+			// should perhaps
+			// move this test and dangerous interface processing to the end of this
+			// method and
+			// make it conditional on whether any of the typeMungers passed into
+			// here actually
+			// matched this type.
+			if (info != null && !info.isOldStyle() && !info.isReweavable()) {
+				processTypeMungersFromExistingWeaverState(sourceType, onType);
+				CompilationAndWeavingContext.leavingPhase(tok);
+				return;
 			}
-		}
 
-		boolean needOldStyleWarning = (info != null && info.isOldStyle());
-
-		onType.clearInterTypeMungers();
-		onType.ensureConsistent();
-
-		// FIXME asc perf Could optimize here, after processing the expected set
-		// of types we may bring
-		// binary types that are not exposed to the weaver, there is no need to
-		// attempt declare parents
-		// or declare annotation really - unless we want to report the
-		// not-exposed to weaver
-		// messages...
-
-		List decpToRepeat = new ArrayList();
-		List<DeclareAnnotation> decaToRepeat = new ArrayList<DeclareAnnotation>();
-		boolean anyNewParents = false;
-		boolean anyNewAnnotations = false;
-
-		// first pass
-		// try and apply all decps - if they match, then great. If they don't
-		// then
-		// check if they are starred-annotation patterns. If they are not
-		// starred
-		// annotation patterns then they might match later...remember that...
-		for (Iterator i = declareParents.iterator(); i.hasNext();) {
-			DeclareParents decp = (DeclareParents) i.next();
-			if (!decp.isMixin()) {
-				boolean didSomething = doDeclareParents(decp, sourceType);
-				if (didSomething) {
-					if (factory.pushinCollector != null) {
-						factory.pushinCollector.tagAsMunged(sourceType, decp.getParents().get(0));
-					}
-					anyNewParents = true;
-				} else {
-					if (!decp.getChild().isStarAnnotation()) {
-						decpToRepeat.add(decp);
-					}
+			// Check if the type we are looking at is the topMostImplementor of a
+			// dangerous interface -
+			// report a problem if it is.
+			for (Iterator i = dangerousInterfaces.entrySet().iterator(); i.hasNext();) {
+				Map.Entry entry = (Map.Entry) i.next();
+				ResolvedType interfaceType = (ResolvedType) entry.getKey();
+				if (onType.isTopmostImplementor(interfaceType)) {
+					factory.showMessage(IMessage.ERROR, onType + ": " + entry.getValue(), onType.getSourceLocation(), null);
 				}
 			}
-		}
 
-		for (Iterator<DeclareAnnotation> i = declareAnnotationOnTypes.iterator(); i.hasNext();) {
-			DeclareAnnotation deca = i.next();
-			boolean didSomething = doDeclareAnnotations(deca, sourceType, true);
-			if (didSomething) {
-				anyNewAnnotations = true;
-			} else {
-				if (!deca.getTypePattern().isStar()) {
-					decaToRepeat.add(deca);
-				}
-			}
-		}
+			boolean needOldStyleWarning = (info != null && info.isOldStyle());
 
-		// now lets loop over and over until we have done all we can
-		while ((anyNewAnnotations || anyNewParents) && (!decpToRepeat.isEmpty() || !decaToRepeat.isEmpty())) {
-			anyNewParents = anyNewAnnotations = false;
-			List forRemoval = new ArrayList();
-			for (Iterator i = decpToRepeat.iterator(); i.hasNext();) {
-				DeclareParents decp = (DeclareParents) i.next();
-				boolean didSomething = doDeclareParents(decp, sourceType);
-				if (didSomething) {
-					if (factory.pushinCollector != null) {
-						factory.pushinCollector.tagAsMunged(sourceType, decp.getParents().get(0));
-					}
-					anyNewParents = true;
-					forRemoval.add(decp);
-				}
-			}
-			decpToRepeat.removeAll(forRemoval);
+			onType.clearInterTypeMungers();
+			onType.ensureConsistent();
 
-			forRemoval.clear();
-			for (Iterator i = declareAnnotationOnTypes.iterator(); i.hasNext();) {
-				DeclareAnnotation deca = (DeclareAnnotation) i.next();
-				boolean didSomething = doDeclareAnnotations(deca, sourceType, false);
-				if (didSomething) {
-					if (factory.pushinCollector != null) {
-						factory.pushinCollector.tagAsMunged(sourceType, deca.getAnnotationString());
-					}
-					anyNewAnnotations = true;
-					forRemoval.add(deca);
-				}
-			}
-			decaToRepeat.removeAll(forRemoval);
-		}
+			// FIXME asc perf Could optimize here, after processing the expected set
+			// of types we may bring
+			// binary types that are not exposed to the weaver, there is no need to
+			// attempt declare parents
+			// or declare annotation really - unless we want to report the
+			// not-exposed to weaver
+			// messages...
 
-		for (Iterator<ConcreteTypeMunger> i = typeMungers.iterator(); i.hasNext();) {
-			EclipseTypeMunger munger = (EclipseTypeMunger) i.next();
-			if (munger.matches(onType)) {
-				if (needOldStyleWarning) {
-					factory.showMessage(IMessage.WARNING, "The class for " + onType
-							+ " should be recompiled with ajc-1.1.1 for best results", onType.getSourceLocation(), null);
-					needOldStyleWarning = false;
-				}
-				onType.addInterTypeMunger(munger, true);
-				if (munger.getMunger()!=null && munger.getMunger().getKind() == ResolvedTypeMunger.InnerClass) {
-					// Must do these right now, because if we do an ITD member afterwards it may attempt to reference the
-					// type being applied (the call above 'addInterTypeMunger' will fail for these ITDs if it needed
-					// it to be in place)
-					if (munger.munge(sourceType, onType)) {
+			List<DeclareParents> decpToRepeat = new ArrayList<DeclareParents>();
+			List<DeclareAnnotation> decaToRepeat = new ArrayList<DeclareAnnotation>();
+			boolean anyNewParents = false;
+			boolean anyNewAnnotations = false;
+
+			// first pass
+			// try and apply all decps - if they match, then great. If they don't
+			// then
+			// check if they are starred-annotation patterns. If they are not
+			// starred
+			// annotation patterns then they might match later...remember that...
+			for (DeclareParents decp : declareParents) {
+				if (!decp.isMixin()) {
+					boolean didSomething = doDeclareParents(decp, sourceType);
+					if (didSomething) {
 						if (factory.pushinCollector != null) {
-							factory.pushinCollector.tagAsMunged(sourceType, munger.getSourceMethod());
+							factory.pushinCollector.tagAsMunged(sourceType, decp.getParents().get(0));
+						}
+						anyNewParents = true;
+					} else {
+						if (!decp.getChild().isStarAnnotation()) {
+							decpToRepeat.add(decp);
 						}
 					}
 				}
 			}
-		}
 
-		onType.checkInterTypeMungers();
-		for (Iterator i = onType.getInterTypeMungers().iterator(); i.hasNext();) {
-			EclipseTypeMunger munger = (EclipseTypeMunger) i.next();
-			if (munger.getMunger()==null || munger.getMunger().getKind() != ResolvedTypeMunger.InnerClass) {
-				if (munger.munge(sourceType, onType)) {
-					if (factory.pushinCollector != null) {
-						factory.pushinCollector.tagAsMunged(sourceType, munger.getSourceMethod());
+			for (DeclareAnnotation deca : declareAnnotationOnTypes) {
+				boolean didSomething = doDeclareAnnotations(deca, sourceType, true);
+				if (didSomething) {
+					anyNewAnnotations = true;
+				} else {
+					if (!deca.getTypePattern().isStar()) {
+						decaToRepeat.add(deca);
+					}
+				}
+			}
+
+			List forRemoval = new ArrayList();
+			// now lets loop over and over until we have done all we can
+			while ((anyNewAnnotations || anyNewParents) && (!decpToRepeat.isEmpty() || !decaToRepeat.isEmpty())) {
+				anyNewParents = anyNewAnnotations = false;
+				forRemoval.clear();
+				for (DeclareParents decp : decpToRepeat) {
+					boolean didSomething = doDeclareParents(decp, sourceType);
+					if (didSomething) {
+						if (factory.pushinCollector != null) {
+							factory.pushinCollector.tagAsMunged(sourceType, decp.getParents().get(0));
+						}
+						anyNewParents = true;
+						forRemoval.add(decp);
+					}
+				}
+				decpToRepeat.removeAll(forRemoval);
+
+				forRemoval.clear();
+				for (DeclareAnnotation deca : decaToRepeat) {
+					boolean didSomething = doDeclareAnnotations(deca, sourceType, false);
+					if (didSomething) {
+						if (factory.pushinCollector != null) {
+							factory.pushinCollector.tagAsMunged(sourceType, deca.getAnnotationString());
+						}
+						anyNewAnnotations = true;
+						forRemoval.add(deca);
+					}
+				}
+				decaToRepeat.removeAll(forRemoval);
+			}
+		}
+		if (mode == 0 || mode == 2) {
+			for (Iterator<ConcreteTypeMunger> i = typeMungers.iterator(); i.hasNext();) {
+				EclipseTypeMunger munger = (EclipseTypeMunger) i.next();
+				if (munger.matches(onType)) {
+					// if (needOldStyleWarning) {
+					// factory.showMessage(IMessage.WARNING, "The class for " + onType
+					// + " should be recompiled with ajc-1.1.1 for best results", onType.getSourceLocation(), null);
+					// needOldStyleWarning = false;
+					// }
+					onType.addInterTypeMunger(munger, true);
+					if (munger.getMunger() != null && munger.getMunger().getKind() == ResolvedTypeMunger.InnerClass) {
+						// Must do these right now, because if we do an ITD member afterwards it may attempt to reference the
+						// type being applied (the call above 'addInterTypeMunger' will fail for these ITDs if it needed
+						// it to be in place)
+						if (munger.munge(sourceType, onType)) {
+							if (factory.pushinCollector != null) {
+								factory.pushinCollector.tagAsMunged(sourceType, munger.getSourceMethod());
+							}
+						}
+					}
+				}
+			}
+
+			onType.checkInterTypeMungers();
+			for (Iterator i = onType.getInterTypeMungers().iterator(); i.hasNext();) {
+				EclipseTypeMunger munger = (EclipseTypeMunger) i.next();
+				if (munger.getMunger() == null || munger.getMunger().getKind() != ResolvedTypeMunger.InnerClass) {
+					if (munger.munge(sourceType, onType)) {
+						if (factory.pushinCollector != null) {
+							factory.pushinCollector.tagAsMunged(sourceType, munger.getSourceMethod());
+						}
 					}
 				}
 			}
@@ -824,7 +843,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		for (int i = 0, length = memberTypes.length; i < length; i++) {
 			if (memberTypes[i] instanceof SourceTypeBinding) {
 				weaveInterTypeDeclarations((SourceTypeBinding) memberTypes[i], typeMungers, declareParents,
-						declareAnnotationOnTypes, false);
+						declareAnnotationOnTypes, false, mode);
 			}
 		}
 		CompilationAndWeavingContext.leavingPhase(tok);
@@ -868,8 +887,8 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 							onType.getSourceLocation(), null);
 				}
 				if (Modifier.isFinal(parent.getModifiers())) {
-					factory.showMessage(IMessage.ERROR, "cannot extend final class " + parent.getClassName(), declareParents
-							.getSourceLocation(), null);
+					factory.showMessage(IMessage.ERROR, "cannot extend final class " + parent.getClassName(),
+							declareParents.getSourceLocation(), null);
 				} else {
 					// do not actually do it if the type isn't exposed - this
 					// will correctly reported as a problem elsewhere
@@ -980,44 +999,43 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		// Might have to retrieve the annotation through BCEL and construct an
 		// eclipse one for it.
 		if (stb instanceof BinaryTypeBinding) {
-			ReferenceType rt = (ReferenceType) factory.fromEclipse(stb);
-			ResolvedMember[] methods = rt.getDeclaredMethods();
-			ResolvedMember decaMethod = null;
-			String nameToLookFor = decA.getAnnotationMethod();
-			for (int i = 0; i < methods.length; i++) {
-				if (methods[i].getName().equals(nameToLookFor)) {
-					decaMethod = methods[i];
-					break;
-				}
-			}
-			if (decaMethod != null) { // could assert this ...
-				AnnotationAJ[] axs = decaMethod.getAnnotations();
-				if (axs != null) { // another error has occurred, dont crash here because of it
-					toAdd = new Annotation[1];
-					toAdd[0] = createAnnotationFromBcelAnnotation(axs[0], decaMethod.getSourceLocation().getOffset(), factory);
-					// BUG BUG BUG - We dont test these abits are correct, in fact
-					// we'll be very lucky if they are.
-					// What does that mean? It means on an incremental compile you
-					// might get away with an
-					// annotation that isn't allowed on a type being put on a type.
-					if (toAdd[0].resolvedType != null) {
-						abits = toAdd[0].resolvedType.getAnnotationTagBits();
-					}
-				}
+			toAdd = retrieveAnnotationFromBinaryTypeBinding(decA, stb);
+			if (toAdd != null && toAdd.length > 0 && toAdd[0].resolvedType != null) {
+				abits = toAdd[0].resolvedType.getAnnotationTagBits();
 			}
 		} else if (stb != null) {
 			// much nicer, its a real SourceTypeBinding so we can stay in
 			// eclipse land
 			// if (decA.getAnnotationMethod() != null) {
-			MethodBinding[] mbs = stb.getMethods(decA.getAnnotationMethod().toCharArray());
-			abits = mbs[0].getAnnotationTagBits(); // ensure resolved
-			TypeDeclaration typeDecl = ((SourceTypeBinding) mbs[0].declaringClass).scope.referenceContext;
-			methodDecl = typeDecl.declarationOf(mbs[0]);
-			toAdd = methodDecl.annotations; // this is what to add
-			toAdd[0] = createAnnotationCopy(toAdd[0]);
-			if (toAdd[0].resolvedType != null) {
-				abits = toAdd[0].resolvedType.getAnnotationTagBits();
-				// }
+			char[] declareSelector = decA.getAnnotationMethod().toCharArray();
+
+			ReferenceBinding rb = stb;
+			String declaringAspectName = decA.getDeclaringType().getRawName();
+			while (rb != null && !new String(CharOperation.concatWith(rb.compoundName, '.')).equals(declaringAspectName)) {
+				rb = rb.superclass();
+			}
+			MethodBinding[] mbs = rb.getMethods(declareSelector);
+
+			ReferenceBinding declaringBinding = mbs[0].declaringClass;
+			if (declaringBinding instanceof ParameterizedTypeBinding) {
+				// Unwrap - this means we don't allow the type of the annotation to be parameterized, may need to revisit that
+				declaringBinding = ((ParameterizedTypeBinding) declaringBinding).type;
+			}
+			if (declaringBinding instanceof BinaryTypeBinding) {
+				toAdd = retrieveAnnotationFromBinaryTypeBinding(decA, declaringBinding);
+				if (toAdd != null && toAdd.length > 0 && toAdd[0].resolvedType != null) {
+					abits = toAdd[0].resolvedType.getAnnotationTagBits();
+				}
+			} else {
+				abits = mbs[0].getAnnotationTagBits(); // ensure resolved
+				TypeDeclaration typeDecl = ((SourceTypeBinding) declaringBinding).scope.referenceContext;
+				methodDecl = typeDecl.declarationOf(mbs[0]);
+				toAdd = methodDecl.annotations; // this is what to add
+				toAdd[0] = createAnnotationCopy(toAdd[0]);
+				if (toAdd[0].resolvedType != null) {
+					abits = toAdd[0].resolvedType.getAnnotationTagBits();
+					// }
+				}
 			}
 		}
 
@@ -1163,6 +1181,36 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 		return true;
 	}
 
+	private Annotation[] retrieveAnnotationFromBinaryTypeBinding(DeclareAnnotation decA, ReferenceBinding declaringBinding) {
+		ReferenceType rt = (ReferenceType) factory.fromEclipse(declaringBinding);
+		ResolvedMember[] methods = rt.getDeclaredMethods();
+		ResolvedMember decaMethod = null;
+		String nameToLookFor = decA.getAnnotationMethod();
+		for (int i = 0; i < methods.length; i++) {
+			if (methods[i].getName().equals(nameToLookFor)) {
+				decaMethod = methods[i];
+				break;
+			}
+		}
+		if (decaMethod != null) { // could assert this ...
+			AnnotationAJ[] axs = decaMethod.getAnnotations();
+			if (axs != null) { // another error has occurred, dont crash here because of it
+				Annotation[] toAdd = new Annotation[1];
+				toAdd[0] = createAnnotationFromBcelAnnotation(axs[0], decaMethod.getSourceLocation().getOffset(), factory);
+				// BUG BUG BUG - We dont test these abits are correct, in fact
+				// we'll be very lucky if they are.
+				// What does that mean? It means on an incremental compile you
+				// might get away with an
+				// annotation that isn't allowed on a type being put on a type.
+				// if (toAdd[0].resolvedType != null) {
+				// abits = toAdd[0].resolvedType.getAnnotationTagBits();
+				// }
+				return toAdd;
+			}
+		}
+		return null;
+	}
+
 	/**
 	 * Transform an annotation from its AJ form to an eclipse form. We *DONT* care about the values of the annotation. that is
 	 * because it is only being stuck on a type during type completion to allow for other constructs (decps, decas) that might be
@@ -1244,9 +1292,13 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 			}
 			filename = filename.substring(takefrom + 1);
 
-			factory.getWorld().getMessageHandler().handleMessage(
-					WeaveMessage.constructWeavingMessage(wmk, new String[] { CharOperation.toString(sourceType.compoundName),
-							filename, parent.getClassName(), getShortname(parent.getSourceLocation().getSourceFile().getPath()) }));
+			factory.getWorld()
+					.getMessageHandler()
+					.handleMessage(
+							WeaveMessage.constructWeavingMessage(wmk,
+									new String[] { CharOperation.toString(sourceType.compoundName), filename,
+											parent.getClassName(),
+											getShortname(parent.getSourceLocation().getSourceFile().getPath()) }));
 		}
 	}
 
@@ -1389,7 +1441,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // System.err.println("Looking for anything that might match "+element+" on "+
 // sourceType.debugName()+"  "+getType(sourceType.
 // compoundName).debugName()+"  "+(sourceType instanceof BinaryTypeBinding));
-//		
+//
 // ReferenceBinding rbb = getType(sourceType.compoundName);
 // // fix me if we ever uncomment this code... should iterate the other way
 // round, over the methods then over the decas
@@ -1401,17 +1453,17 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // ((SourceTypeBinding)rbb).getExactMethod(sourceMb.selector
 // ,sourceMb.parameters);
 // boolean isCtor = sourceMb.selector[0]=='<';
-//			
+//
 // if ((element.isDeclareAtConstuctor() ^ !isCtor)) {
 // System.err.println("Checking "+sourceMb+" ... declaringclass="+sourceMb.
 // declaringClass.debugName()+" rbb="+rbb.debugName()+"  "+
 // sourceMb.declaringClass.equals(rbb));
-//			
+//
 // ResolvedMember rm = null;
 // rm = EclipseFactory.makeResolvedMember(mbbbb);
 // if (element.matches(rm,factory.getWorld())) {
 // System.err.println("MATCH");
-//				
+//
 // // Determine the set of annotations that are currently on the method
 // ReferenceBinding rb = getType(sourceType.compoundName);
 // // TypeBinding tb = factory.makeTypeBinding(decA.getAspect());
@@ -1424,7 +1476,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // AbstractMethodDeclaration methodDecl = typeDecl.declarationOf(sourceMb);
 // Annotation[] currentlyHas = methodDecl.annotations; // this is what to add
 // //abits = toAdd[0].resolvedType.getAnnotationTagBits();
-//				
+//
 // // Determine the annotations to add to that method
 // TypeBinding tb = factory.makeTypeBinding(element.getAspect());
 // MethodBinding[] aspectMbs =
@@ -1438,7 +1490,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // Annotation[] toAdd = methodDecl2.annotations; // this is what to add
 // // abits = toAdd[0].resolvedType.getAnnotationTagBits();
 // System.err.println("Has: "+currentlyHas+"    toAdd: "+toAdd);
-//				
+//
 // // fix me? should check if it already has the annotation
 // //Annotation abefore[] = sourceType.scope.referenceContext.annotations;
 // Annotation[] newset = new
@@ -1473,7 +1525,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // System.err.println("Processing deca "+element+" on "+sourceType.debugName()+
 // "  "+getType(sourceType.compoundName).debugName()+"  "
 // +(sourceType instanceof BinaryTypeBinding));
-//				
+//
 // ReferenceBinding rbb = getType(sourceType.compoundName);
 // // fix me? should iterate the other way round, over the methods then over the
 // decas
@@ -1484,15 +1536,15 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // //FieldBinding fbbbb =
 // ((SourceTypeBinding)rbb).getgetExactMethod(sourceMb.selector
 // ,sourceMb.parameters);
-//					
+//
 // System.err.println("Checking "+sourceFb+" ... declaringclass="+sourceFb.
 // declaringClass.debugName()+" rbb="+rbb.debugName());
-//					
+//
 // ResolvedMember rm = null;
 // rm = EclipseFactory.makeResolvedMember(sourceFb);
 // if (element.matches(rm,factory.getWorld())) {
 // System.err.println("MATCH");
-//						
+//
 // // Determine the set of annotations that are currently on the field
 // ReferenceBinding rb = getType(sourceType.compoundName);
 // // TypeBinding tb = factory.makeTypeBinding(decA.getAspect());
@@ -1504,7 +1556,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // //AbstractMethodDeclaration methodDecl = typeDecl.declarationOf(sourceMb);
 // Annotation[] currentlyHas = fd.annotations; // this is what to add
 // //abits = toAdd[0].resolvedType.getAnnotationTagBits();
-//						
+//
 // // Determine the annotations to add to that method
 // TypeBinding tb = factory.makeTypeBinding(element.getAspect());
 // MethodBinding[] aspectMbs =
@@ -1518,7 +1570,7 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // Annotation[] toAdd = methodDecl2.annotations; // this is what to add
 // // abits = toAdd[0].resolvedType.getAnnotationTagBits();
 // System.err.println("Has: "+currentlyHas+"    toAdd: "+toAdd);
-//						
+//
 // // fix me? check if it already has the annotation
 //
 //
@@ -1535,6 +1587,6 @@ public class AjLookupEnvironment extends LookupEnvironment implements AnonymousC
 // } else
 // System.err.println("NO MATCH");
 // }
-//			
+//
 // }
 // }
