@@ -117,7 +117,8 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 	 * 
 	 * @param cuDeclaration
 	 * @param buildConfig
-	 * @param structureModel hiearchy to add this unit's declarations to
+	 * @param structureModel
+	 *            hiearchy to add this unit's declarations to
 	 */
 	public void buildStructureForCompilationUnit(CompilationUnitDeclaration cuDeclaration, AsmManager structureModel,
 			AjBuildConfig buildConfig) {
@@ -301,8 +302,8 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		if (typeDeclaration.annotations != null) {
 			for (int i = 0; i < typeDeclaration.annotations.length; i++) {
 				Annotation annotation = typeDeclaration.annotations[i];
-				if (Arrays.equals(annotation.type.getTypeBindingPublic(scope).signature(), "Lorg/aspectj/lang/annotation/Aspect;"
-						.toCharArray())) {
+				if (Arrays.equals(annotation.type.getTypeBindingPublic(scope).signature(),
+						"Lorg/aspectj/lang/annotation/Aspect;".toCharArray())) {
 					kind = IProgramElement.Kind.ASPECT;
 					if (!(typeDeclaration instanceof AspectDeclaration)) {
 						isAnnotationStyleAspect = true;
@@ -397,8 +398,8 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		if (memberTypeDeclaration.annotations != null) {
 			for (int i = 0; i < memberTypeDeclaration.annotations.length; i++) {
 				Annotation annotation = memberTypeDeclaration.annotations[i];
-				if (Arrays.equals(annotation.type.getTypeBindingPublic(scope).signature(), "Lorg/aspectj/lang/annotation/Aspect;"
-						.toCharArray())) {
+				if (Arrays.equals(annotation.type.getTypeBindingPublic(scope).signature(),
+						"Lorg/aspectj/lang/annotation/Aspect;".toCharArray())) {
 					kind = IProgramElement.Kind.ASPECT;
 					if (!(memberTypeDeclaration instanceof AspectDeclaration)) {
 						isAnnotationStyleAspect = true;
@@ -457,8 +458,8 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		if (memberTypeDeclaration.annotations != null) {
 			for (int i = 0; i < memberTypeDeclaration.annotations.length; i++) {
 				Annotation annotation = memberTypeDeclaration.annotations[i];
-				if (Arrays.equals(annotation.type.getTypeBindingPublic(scope).signature(), "Lorg/aspectj/lang/annotation/Aspect;"
-						.toCharArray())) {
+				if (Arrays.equals(annotation.type.getTypeBindingPublic(scope).signature(),
+						"Lorg/aspectj/lang/annotation/Aspect;".toCharArray())) {
 					kind = IProgramElement.Kind.ASPECT;
 					if (!(memberTypeDeclaration instanceof AspectDeclaration)) {
 						isAnnotationStyleAspect = true;
@@ -540,14 +541,15 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 			Declare decl = dDeclaration.declareDecl;
 			if (decl instanceof DeclareParents) {
 				TypePatternList tpl = ((DeclareParents) decl).getParents();
-				List parents = new ArrayList();
+				List<String> parents = new ArrayList<String>();
 				for (int i = 0; i < tpl.size(); i++) {
 					parents.add(tpl.get(i).getExactType().getName().replaceAll("\\$", "."));
 				}
 				peNode.setParentTypes(parents);
 			}
 			if (decl instanceof DeclareAnnotation) {
-				ResolvedType annotationType = ((DeclareAnnotation) decl).getAnnotationType();
+				DeclareAnnotation da = (DeclareAnnotation) decl;
+				ResolvedType annotationType = da.getAnnotationType();
 				if (annotationType == null) {
 					String s = ((DeclareAnnotation) decl).getAnnotationString();
 					if (s != null && s.length() > 0) {
@@ -557,6 +559,9 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 				} else {
 					peNode.setAnnotationType(annotationType.getName());
 				}
+				if (da.isRemover()) {
+					peNode.setAnnotationRemover(true);
+				}
 			}
 		}
 		if (methodDeclaration.returnType != null) {
@@ -663,7 +668,8 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 
 	/**
 	 * @param left
-	 * @param pointcuts accumulator for named pointcuts
+	 * @param pointcuts
+	 *            accumulator for named pointcuts
 	 */
 	private void addAllNamed(Pointcut pointcut, List pointcuts) {
 		if (pointcut == null) {
@@ -738,7 +744,7 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 	// } catch (NullPointerException npe) {
 	// memberName = "<undefined>";
 	// }
-	//			
+	//
 	// peNode.setBytecodeName(memberName);
 	// peNode.setBytecodeSignature(memberBytecodeSignature);
 	// }
@@ -796,7 +802,7 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 
 	private ProgramElement getImportReferencesRoot() {
 		IProgramElement element = (IProgramElement) stack.peek();
-		boolean hasPackageDeclaration = ((IProgramElement) element.getChildren().get(0)).getKind().isPackageDeclaration();
+		boolean hasPackageDeclaration = (element.getChildren().get(0)).getKind().isPackageDeclaration();
 		return (ProgramElement) element.getChildren().get(hasPackageDeclaration ? 1 : 0);
 	}
 
@@ -830,10 +836,19 @@ public class AsmHierarchyBuilder extends ASTVisitor {
 		} else {
 			peNode = new ProgramElement(activeStructureModel, new String(fieldDeclaration.name), IProgramElement.Kind.FIELD,
 					makeLocation(fieldDeclaration), fieldDeclaration.modifiers, null, null);
-			peNode.setCorrespondingType(fieldDeclaration.type.toString());
+
+			if (fieldDeclaration.type.resolvedType != null) {
+				char[] cs = fieldDeclaration.type.resolvedType.readableName();
+				// fieldDeclaration.type.resolvedType.genericTypeSignature()
+				peNode.setCorrespondingType(new String(cs));
+			} else {
+				// peNode.setCorrespondingType(null);
+				peNode.setCorrespondingType(fieldDeclaration.type.toString());
+			}
 		}
 		peNode.setSourceSignature(genSourceSignature(fieldDeclaration));
 		peNode.setFormalComment(generateJavadocComment(fieldDeclaration));
+		// peNode.setBytecodeSignature(new String(fieldDeclaration.binding.type.signature()));
 
 		((IProgramElement) stack.peek()).addChild(peNode);
 		stack.push(peNode);
