@@ -12,11 +12,13 @@
  * ******************************************************************/
 package org.aspectj.weaver;
 
+import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.Collection;
-import java.util.Iterator;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 
 import org.aspectj.bridge.ISourceLocation;
 import org.aspectj.weaver.patterns.Declare;
@@ -30,11 +32,13 @@ import org.aspectj.weaver.patterns.PerClause;
  */
 public class ReferenceType extends ResolvedType {
 
+	public static final ReferenceType[] EMPTY_ARRAY = new ReferenceType[0];
+
 	/**
 	 * For generic types, this list holds references to all the derived raw and parameterized versions. We need this so that if the
 	 * generic delegate is swapped during incremental compilation, the delegate of the derivatives is swapped also.
 	 */
-	private final List/* ReferenceType */derivativeTypes = new ArrayList();
+	private final Set<ReferenceType> derivativeTypes = new HashSet<ReferenceType>();
 
 	/**
 	 * For parameterized types (or the raw type) - this field points to the actual reference type from which they are derived.
@@ -49,9 +53,9 @@ public class ReferenceType extends ResolvedType {
 	ResolvedMember[] parameterizedMethods = null;
 	ResolvedMember[] parameterizedFields = null;
 	ResolvedMember[] parameterizedPointcuts = null;
-	ResolvedType[] parameterizedInterfaces = null;
-	Collection parameterizedDeclares = null;
-	Collection parameterizedTypeMungers = null;
+	WeakReference<ResolvedType[]> parameterizedInterfaces = new WeakReference<ResolvedType[]>(null);
+	Collection<Declare> parameterizedDeclares = null;
+	// Collection parameterizedTypeMungers = null;
 
 	// During matching it can be necessary to temporary mark types as annotated. For example
 	// a declare @type may trigger a separate declare parents to match, and so the annotation
@@ -109,14 +113,15 @@ public class ReferenceType extends ResolvedType {
 	// this.delegate = genericReferenceType.getDelegate();
 	// genericReferenceType.addDependentType(this);
 	// }
-	private void addDependentType(ReferenceType dependent) {
+	synchronized void addDependentType(ReferenceType dependent) {
 		this.derivativeTypes.add(dependent);
 	}
 
 	@Override
 	public String getSignatureForAttribute() {
-		if (genericType == null || typeParameters == null)
+		if (genericType == null || typeParameters == null) {
 			return getSignature();
+		}
 		return makeDeclaredSignature(genericType, typeParameters);
 	}
 
@@ -130,22 +135,27 @@ public class ReferenceType extends ResolvedType {
 
 	@Override
 	public boolean isClass() {
-		return delegate.isClass();
+		return getDelegate().isClass();
+	}
+
+	@Override
+	public int getCompilerVersion() {
+		return getDelegate().getCompilerVersion();
 	}
 
 	@Override
 	public boolean isGenericType() {
-		return !isParameterizedType() && !isRawType() && delegate.isGeneric();
+		return !isParameterizedType() && !isRawType() && getDelegate().isGeneric();
 	}
 
 	public String getGenericSignature() {
-		String sig = delegate.getDeclaredGenericSignature();
+		String sig = getDelegate().getDeclaredGenericSignature();
 		return (sig == null) ? "" : sig;
 	}
 
 	@Override
 	public AnnotationAJ[] getAnnotations() {
-		return delegate.getAnnotations();
+		return getDelegate().getAnnotations();
 	}
 
 	@Override
@@ -163,7 +173,7 @@ public class ReferenceType extends ResolvedType {
 	}
 
 	public boolean hasAnnotation(UnresolvedType ofType) {
-		boolean onDelegate = delegate.hasAnnotation(ofType);
+		boolean onDelegate = getDelegate().hasAnnotation(ofType);
 		if (onDelegate) {
 			return true;
 		}
@@ -191,14 +201,14 @@ public class ReferenceType extends ResolvedType {
 
 	@Override
 	public ResolvedType[] getAnnotationTypes() {
-		if (delegate == null) {
+		if (getDelegate() == null) {
 			throw new BCException("Unexpected null delegate for type " + this.getName());
 		}
 		if (annotationTypes == null) {
 			// there are no extras:
-			return delegate.getAnnotationTypes();
+			return getDelegate().getAnnotationTypes();
 		} else {
-			ResolvedType[] delegateAnnotationTypes = delegate.getAnnotationTypes();
+			ResolvedType[] delegateAnnotationTypes = getDelegate().getAnnotationTypes();
 			ResolvedType[] result = new ResolvedType[annotationTypes.length + delegateAnnotationTypes.length];
 			System.arraycopy(delegateAnnotationTypes, 0, result, 0, delegateAnnotationTypes.length);
 			System.arraycopy(annotationTypes, 0, result, delegateAnnotationTypes.length, annotationTypes.length);
@@ -206,9 +216,14 @@ public class ReferenceType extends ResolvedType {
 		}
 	}
 
+	@Override
+	public String getNameAsIdentifier() {
+		return getRawName().replace('.', '_');
+	}
+
 	@Override
 	public AnnotationAJ getAnnotationOfType(UnresolvedType ofType) {
-		AnnotationAJ[] axs = delegate.getAnnotations();
+		AnnotationAJ[] axs = getDelegate().getAnnotations();
 		if (axs == null) {
 			if (annotations != null) {
 				String searchSig = ofType.getSignature();
@@ -230,55 +245,55 @@ public class ReferenceType extends ResolvedType {
 
 	@Override
 	public boolean isAspect() {
-		return delegate.isAspect();
+		return getDelegate().isAspect();
 	}
 
 	@Override
 	public boolean isAnnotationStyleAspect() {
-		return delegate.isAnnotationStyleAspect();
+		return getDelegate().isAnnotationStyleAspect();
 	}
 
 	@Override
 	public boolean isEnum() {
-		return delegate.isEnum();
+		return getDelegate().isEnum();
 	}
 
 	@Override
 	public boolean isAnnotation() {
-		return delegate.isAnnotation();
+		return getDelegate().isAnnotation();
 	}
 
 	@Override
 	public boolean isAnonymous() {
-		return delegate.isAnonymous();
+		return getDelegate().isAnonymous();
 	}
 
 	@Override
 	public boolean isNested() {
-		return delegate.isNested();
+		return getDelegate().isNested();
 	}
 
 	public ResolvedType getOuterClass() {
-		return delegate.getOuterClass();
+		return getDelegate().getOuterClass();
 	}
 
 	public String getRetentionPolicy() {
-		return delegate.getRetentionPolicy();
+		return getDelegate().getRetentionPolicy();
 	}
 
 	@Override
 	public boolean isAnnotationWithRuntimeRetention() {
-		return delegate.isAnnotationWithRuntimeRetention();
+		return getDelegate().isAnnotationWithRuntimeRetention();
 	}
 
 	@Override
 	public boolean canAnnotationTargetType() {
-		return delegate.canAnnotationTargetType();
+		return getDelegate().canAnnotationTargetType();
 	}
 
 	@Override
 	public AnnotationTargetKind[] getAnnotationTargetKinds() {
-		return delegate.getAnnotationTargetKinds();
+		return getDelegate().getAnnotationTargetKinds();
 	}
 
 	// true iff the statement "this = (ThisType) other" would compile
@@ -299,7 +314,7 @@ public class ReferenceType extends ResolvedType {
 		}
 
 		if (this.isRawType() && other.isParameterizedType()) {
-			return this.getGenericType().isCoerceableFrom(((ReferenceType) other.getRawType()));
+			return this.getGenericType().isCoerceableFrom((other.getRawType()));
 		}
 
 		if (!this.isInterface() && !other.isInterface()) {
@@ -314,18 +329,20 @@ public class ReferenceType extends ResolvedType {
 		// always safe
 		for (int ai = 0, alen = a.length; ai < alen; ai++) {
 			for (int bi = 0, blen = b.length; bi < blen; bi++) {
-				if (!b[bi].isCompatibleWith(a[ai]))
+				if (!b[bi].isCompatibleWith(a[ai])) {
 					return false;
+				}
 			}
 		}
 		return true;
 	}
 
 	private final boolean isCoerceableFromParameterizedType(ResolvedType other) {
-		if (!other.isParameterizedType())
+		if (!other.isParameterizedType()) {
 			return false;
-		ResolvedType myRawType = (ResolvedType) getRawType();
-		ResolvedType theirRawType = (ResolvedType) other.getRawType();
+		}
+		ResolvedType myRawType = getRawType();
+		ResolvedType theirRawType = other.getRawType();
 		if (myRawType == theirRawType || myRawType.isCoerceableFrom(theirRawType)) {
 			if (getTypeParameters().length == other.getTypeParameters().length) {
 				// there's a chance it can be done
@@ -337,20 +354,23 @@ public class ReferenceType extends ResolvedType {
 						// coerceable from e.g. List<T>
 						if (myTypeParameters[i].isGenericWildcard()) {
 							BoundedReferenceType wildcard = (BoundedReferenceType) myTypeParameters[i];
-							if (!wildcard.canBeCoercedTo(theirTypeParameters[i]))
+							if (!wildcard.canBeCoercedTo(theirTypeParameters[i])) {
 								return false;
+							}
 						} else if (myTypeParameters[i].isTypeVariableReference()) {
 							TypeVariableReferenceType tvrt = (TypeVariableReferenceType) myTypeParameters[i];
 							TypeVariable tv = tvrt.getTypeVariable();
 							tv.resolve(world);
-							if (!tv.canBeBoundTo(theirTypeParameters[i]))
+							if (!tv.canBeBoundTo(theirTypeParameters[i])) {
 								return false;
+							}
 						} else if (theirTypeParameters[i].isTypeVariableReference()) {
 							TypeVariableReferenceType tvrt = (TypeVariableReferenceType) theirTypeParameters[i];
 							TypeVariable tv = tvrt.getTypeVariable();
 							tv.resolve(world);
-							if (!tv.canBeBoundTo(myTypeParameters[i]))
+							if (!tv.canBeBoundTo(myTypeParameters[i])) {
 								return false;
+							}
 						} else if (theirTypeParameters[i].isGenericWildcard()) {
 							BoundedReferenceType wildcard = (BoundedReferenceType) theirTypeParameters[i];
 							if (!wildcard.canBeCoercedTo(myTypeParameters[i])) {
@@ -387,36 +407,35 @@ public class ReferenceType extends ResolvedType {
 	@Override
 	public boolean isAssignableFrom(ResolvedType other, boolean allowMissing) {
 		if (other.isPrimitiveType()) {
-			if (!world.isInJava5Mode())
+			if (!world.isInJava5Mode()) {
 				return false;
-			if (ResolvedType.validBoxing.contains(this.getSignature() + other.getSignature()))
-				return true;
-		}
-		if (this == other) {
-			return true;
-		}
-		if (this.getSignature().equals(ResolvedType.OBJECT.getSignature())) {
-			return true;
-		}
-
-		if ((this.isRawType() || this.isGenericType()) && other.isParameterizedType()) {
-			if (isAssignableFrom((ResolvedType) other.getRawType())) {
+			}
+			if (ResolvedType.validBoxing.contains(this.getSignature() + other.getSignature())) {
 				return true;
 			}
 		}
-		if (this.isRawType() && other.isGenericType()) {
-			if (isAssignableFrom((ResolvedType) other.getRawType()))
-				return true;
+
+		if (this == other) {
+			return true;
 		}
-		if (this.isGenericType() && other.isRawType()) {
-			if (isAssignableFrom(other.getGenericType()))
-				return true;
+
+		if (this.getSignature().equals("Ljava/lang/Object;")) {
+			return true;
+		}
+
+		if (!isTypeVariableReference() && other.getSignature().equals("Ljava/lang/Object;")) {
+			return false;
+		}
+
+		boolean thisRaw = this.isRawType();
+		if (thisRaw && other.isParameterizedOrGenericType()) {
+			return isAssignableFrom(other.getRawType());
+		}
+
+		boolean thisGeneric = this.isGenericType();
+		if (thisGeneric && other.isParameterizedOrRawType()) {
+			return isAssignableFrom(other.getGenericType());
 		}
-		// if (this.isParameterizedType() && other.isRawType()) {
-		// if (((ReferenceType) this.getRawType()).isAssignableFrom(other.getGenericType())) {
-		// return true;
-		// }
-		// }
 
 		if (this.isParameterizedType()) {
 			// look at wildcards...
@@ -433,8 +452,9 @@ public class ReferenceType extends ResolvedType {
 						}
 					}
 				}
-				if (wildcardsAllTheWay && !other.isParameterizedType())
+				if (wildcardsAllTheWay && !other.isParameterizedType()) {
 					return true;
+				}
 				// we have to match by parameters one at a time
 				ResolvedType[] theirParameters = other.getResolvedTypeParameters();
 				boolean parametersAssignable = true;
@@ -497,24 +517,14 @@ public class ReferenceType extends ResolvedType {
 				} else {
 					parametersAssignable = false;
 				}
-				if (parametersAssignable)
+				if (parametersAssignable) {
 					return true;
+				}
 			}
 		}
 
-		if (isTypeVariableReference() && !other.isTypeVariableReference()) { // eg
-			// .
-			// this
-			// =
-			// T
-			// other
-			// =
-			// Ljava
-			// /
-			// lang
-			// /
-			// Object
-			// ;
+		// eg this=T other=Ljava/lang/Object;
+		if (isTypeVariableReference() && !other.isTypeVariableReference()) {
 			TypeVariable aVar = ((TypeVariableReference) this).getTypeVariable();
 			return aVar.resolve(world).canBeBoundTo(other);
 		}
@@ -522,8 +532,8 @@ public class ReferenceType extends ResolvedType {
 		if (other.isTypeVariableReference()) {
 			TypeVariableReferenceType otherType = (TypeVariableReferenceType) other;
 			if (this instanceof TypeVariableReference) {
-				return ((TypeVariableReference) this).getTypeVariable().resolve(world).canBeBoundTo(
-						otherType.getTypeVariable().getFirstBound().resolve(world));// pr171952
+				return ((TypeVariableReference) this).getTypeVariable().resolve(world)
+						.canBeBoundTo(otherType.getTypeVariable().getFirstBound().resolve(world));// pr171952
 				// return
 				// ((TypeVariableReference)this).getTypeVariable()==otherType
 				// .getTypeVariable();
@@ -533,45 +543,65 @@ public class ReferenceType extends ResolvedType {
 			}
 		}
 
-		if (allowMissing && other.isMissing())
+		if (allowMissing && other.isMissing()) {
 			return false;
+		}
 
-		for (Iterator i = other.getDirectSupertypes(); i.hasNext();) {
-			if (this.isAssignableFrom((ResolvedType) i.next(), allowMissing))
+		ResolvedType[] interfaces = other.getDeclaredInterfaces();
+		for (ResolvedType intface : interfaces) {
+			boolean b;
+			if (thisRaw && intface.isParameterizedOrGenericType()) {
+				b = this.isAssignableFrom(intface.getRawType(), allowMissing);
+			} else {
+				b = this.isAssignableFrom(intface, allowMissing);
+			}
+			if (b) {
 				return true;
+			}
+		}
+		ResolvedType superclass = other.getSuperclass();
+		if (superclass != null) {
+			boolean b;
+			if (thisRaw && superclass.isParameterizedOrGenericType()) {
+				b = this.isAssignableFrom(superclass.getRawType(), allowMissing);
+			} else {
+				b = this.isAssignableFrom(superclass, allowMissing);
+			}
+			if (b) {
+				return true;
+			}
 		}
 		return false;
 	}
 
 	@Override
 	public ISourceContext getSourceContext() {
-		return delegate.getSourceContext();
+		return getDelegate().getSourceContext();
 	}
 
 	@Override
 	public ISourceLocation getSourceLocation() {
-		ISourceContext isc = delegate.getSourceContext();
+		ISourceContext isc = getDelegate().getSourceContext();
 		return isc.makeSourceLocation(new Position(startPos, endPos));
 	}
 
 	@Override
 	public boolean isExposedToWeaver() {
-		return (delegate == null) || delegate.isExposedToWeaver(); // ??? where
-		// does this
-		// belong
+		return (getDelegate() == null) || delegate.isExposedToWeaver();
 	}
 
 	@Override
 	public WeaverStateInfo getWeaverState() {
-		return delegate.getWeaverState();
+		return getDelegate().getWeaverState();
 	}
 
 	@Override
 	public ResolvedMember[] getDeclaredFields() {
-		if (parameterizedFields != null)
+		if (parameterizedFields != null) {
 			return parameterizedFields;
+		}
 		if (isParameterizedType() || isRawType()) {
-			ResolvedMember[] delegateFields = delegate.getDeclaredFields();
+			ResolvedMember[] delegateFields = getDelegate().getDeclaredFields();
 			parameterizedFields = new ResolvedMember[delegateFields.length];
 			for (int i = 0; i < delegateFields.length; i++) {
 				parameterizedFields[i] = delegateFields[i].parameterizedWith(getTypesForMemberParameterization(), this,
@@ -579,7 +609,7 @@ public class ReferenceType extends ResolvedType {
 			}
 			return parameterizedFields;
 		} else {
-			return delegate.getDeclaredFields();
+			return getDelegate().getDeclaredFields();
 		}
 	}
 
@@ -589,52 +619,82 @@ public class ReferenceType extends ResolvedType {
 	 */
 	@Override
 	public ResolvedType[] getDeclaredInterfaces() {
-		if (parameterizedInterfaces != null)
-			return parameterizedInterfaces;
-		ResolvedType[] delegateInterfaces = delegate.getDeclaredInterfaces();
-		if (newInterfaces != null) {
+		ResolvedType[] interfaces = parameterizedInterfaces.get();
+		if (interfaces != null) {
+			return interfaces;
+		}
+		ResolvedType[] delegateInterfaces = getDelegate().getDeclaredInterfaces();
+		if (isRawType()) {			
+			if (newInterfaces!=null) {
+				throw new IllegalStateException("The raw type should never be accumulating new interfaces, they should be on the generic type.  Type is "+this.getName());
+			}
+			ResolvedType[] newInterfacesFromGenericType = genericType.newInterfaces;
+			if (newInterfacesFromGenericType!=null) {
+				ResolvedType[] extraInterfaces = new ResolvedType[delegateInterfaces.length + newInterfacesFromGenericType.length];
+				System.arraycopy(delegateInterfaces, 0, extraInterfaces, 0, delegateInterfaces.length);
+				System.arraycopy(newInterfacesFromGenericType, 0, extraInterfaces, delegateInterfaces.length, newInterfacesFromGenericType.length);
+				delegateInterfaces = extraInterfaces;
+			}
+		} else if (newInterfaces != null) {
+			// OPTIMIZE does this part of the method trigger often?
 			ResolvedType[] extraInterfaces = new ResolvedType[delegateInterfaces.length + newInterfaces.length];
 			System.arraycopy(delegateInterfaces, 0, extraInterfaces, 0, delegateInterfaces.length);
 			System.arraycopy(newInterfaces, 0, extraInterfaces, delegateInterfaces.length, newInterfaces.length);
+			
 			delegateInterfaces = extraInterfaces;
 		}
 		if (isParameterizedType()) {
 			// UnresolvedType[] paramTypes =
 			// getTypesForMemberParameterization();
-			parameterizedInterfaces = new ResolvedType[delegateInterfaces.length];
+			interfaces = new ResolvedType[delegateInterfaces.length];
 			for (int i = 0; i < delegateInterfaces.length; i++) {
 				// We may have to sub/super set the set of parametertypes if the
 				// implemented interface
 				// needs more or less than this type does. (pr124803/pr125080)
 
 				if (delegateInterfaces[i].isParameterizedType()) {
-					parameterizedInterfaces[i] = delegateInterfaces[i].parameterize(getMemberParameterizationMap()).resolve(world);
+					interfaces[i] = delegateInterfaces[i].parameterize(getMemberParameterizationMap()).resolve(world);
 				} else {
-					parameterizedInterfaces[i] = delegateInterfaces[i];
+					interfaces[i] = delegateInterfaces[i];
 				}
 			}
-			return parameterizedInterfaces;
+			parameterizedInterfaces = new WeakReference<ResolvedType[]>(interfaces);
+			return interfaces;
 		} else if (isRawType()) {
 			UnresolvedType[] paramTypes = getTypesForMemberParameterization();
-			parameterizedInterfaces = new ResolvedType[delegateInterfaces.length];
-			for (int i = 0; i < parameterizedInterfaces.length; i++) {
-				parameterizedInterfaces[i] = delegateInterfaces[i];
-				if (parameterizedInterfaces[i].isGenericType()) {
+			interfaces = new ResolvedType[delegateInterfaces.length];
+			for (int i = 0, max = interfaces.length; i < max; i++) {
+				interfaces[i] = delegateInterfaces[i];
+				if (interfaces[i].isGenericType()) {
 					// a generic supertype of a raw type is replaced by its raw
 					// equivalent
-					parameterizedInterfaces[i] = parameterizedInterfaces[i].getRawType().resolve(getWorld());
-				} else if (parameterizedInterfaces[i].isParameterizedType()) {
+					interfaces[i] = interfaces[i].getRawType().resolve(getWorld());
+				} else if (interfaces[i].isParameterizedType()) {
 					// a parameterized supertype collapses any type vars to
 					// their upper bounds
-					UnresolvedType[] toUseForParameterization = determineThoseTypesToUse(parameterizedInterfaces[i], paramTypes);
-					parameterizedInterfaces[i] = parameterizedInterfaces[i].parameterizedWith(toUseForParameterization);
+					UnresolvedType[] toUseForParameterization = determineThoseTypesToUse(interfaces[i], paramTypes);
+					interfaces[i] = interfaces[i].parameterizedWith(toUseForParameterization);
 				}
 			}
-			return parameterizedInterfaces;
+			parameterizedInterfaces = new WeakReference<ResolvedType[]>(interfaces);
+			return interfaces;
+		}
+		if (getDelegate().isCacheable()) {
+			parameterizedInterfaces = new WeakReference<ResolvedType[]>(delegateInterfaces);
 		}
 		return delegateInterfaces;
 	}
 
+	// private String toString(ResolvedType[] delegateInterfaces) {
+	// StringBuffer sb = new StringBuffer();
+	// if (delegateInterfaces != null) {
+	// for (ResolvedType rt : delegateInterfaces) {
+	// sb.append(rt).append(" ");
+	// }
+	// }
+	// return sb.toString();
+	// }
+
 	/**
 	 * Locates the named type variable in the list of those on this generic type and returns the type parameter from the second list
 	 * supplied. Returns null if it can't be found
@@ -656,8 +716,7 @@ public class ReferenceType extends ResolvedType {
 	 * class Foo<T extends String,E extends Number> implements SuperInterface<T> {}
 	 * </code> where <code>
 	 * interface SuperInterface<Z> {}
-	 * </code> In that
-	 * example, a use of the 'Foo' raw type should know that it implements the SuperInterface<String>.
+	 * </code> In that example, a use of the 'Foo' raw type should know that it implements the SuperInterface<String>.
 	 */
 	private UnresolvedType[] determineThoseTypesToUse(ResolvedType parameterizedInterface, UnresolvedType[] paramTypes) {
 		// What are the type parameters for the supertype?
@@ -701,18 +760,20 @@ public class ReferenceType extends ResolvedType {
 		TypeVariable[] thisTypesTVars = getGenericType().getTypeVariables();
 		for (int i = 0; i < thisTypesTVars.length; i++) {
 			TypeVariable tv = thisTypesTVars[i];
-			if (tv.getName().equals(tvname))
+			if (tv.getName().equals(tvname)) {
 				return i;
+			}
 		}
 		return -1;
 	}
 
 	@Override
 	public ResolvedMember[] getDeclaredMethods() {
-		if (parameterizedMethods != null)
+		if (parameterizedMethods != null) {
 			return parameterizedMethods;
+		}
 		if (isParameterizedType() || isRawType()) {
-			ResolvedMember[] delegateMethods = delegate.getDeclaredMethods();
+			ResolvedMember[] delegateMethods = getDelegate().getDeclaredMethods();
 			UnresolvedType[] parameters = getTypesForMemberParameterization();
 			parameterizedMethods = new ResolvedMember[delegateMethods.length];
 			for (int i = 0; i < delegateMethods.length; i++) {
@@ -720,16 +781,17 @@ public class ReferenceType extends ResolvedType {
 			}
 			return parameterizedMethods;
 		} else {
-			return delegate.getDeclaredMethods();
+			return getDelegate().getDeclaredMethods();
 		}
 	}
 
 	@Override
 	public ResolvedMember[] getDeclaredPointcuts() {
-		if (parameterizedPointcuts != null)
+		if (parameterizedPointcuts != null) {
 			return parameterizedPointcuts;
+		}
 		if (isParameterizedType()) {
-			ResolvedMember[] delegatePointcuts = delegate.getDeclaredPointcuts();
+			ResolvedMember[] delegatePointcuts = getDelegate().getDeclaredPointcuts();
 			parameterizedPointcuts = new ResolvedMember[delegatePointcuts.length];
 			for (int i = 0; i < delegatePointcuts.length; i++) {
 				parameterizedPointcuts[i] = delegatePointcuts[i].parameterizedWith(getTypesForMemberParameterization(), this,
@@ -737,7 +799,7 @@ public class ReferenceType extends ResolvedType {
 			}
 			return parameterizedPointcuts;
 		} else {
-			return delegate.getDeclaredPointcuts();
+			return getDelegate().getDeclaredPointcuts();
 		}
 	}
 
@@ -756,15 +818,10 @@ public class ReferenceType extends ResolvedType {
 		return parameters;
 	}
 
-	@Override
-	public UnresolvedType getRawType() {
-		return super.getRawType().resolve(world);
-	}
-
 	@Override
 	public TypeVariable[] getTypeVariables() {
 		if (this.typeVariables == null) {
-			this.typeVariables = delegate.getTypeVariables();
+			this.typeVariables = getDelegate().getTypeVariables();
 			for (int i = 0; i < this.typeVariables.length; i++) {
 				this.typeVariables[i].resolve(world);
 			}
@@ -774,9 +831,9 @@ public class ReferenceType extends ResolvedType {
 
 	@Override
 	public PerClause getPerClause() {
-		PerClause pclause = delegate.getPerClause();
-		if (isParameterizedType()) { // could cache the result here...
-			Map parameterizationMap = getAjMemberParameterizationMap();
+		PerClause pclause = getDelegate().getPerClause();
+		if (pclause != null && isParameterizedType()) { // could cache the result here...
+			Map<String, UnresolvedType> parameterizationMap = getAjMemberParameterizationMap();
 			pclause = (PerClause) pclause.parameterizeWith(parameterizationMap, world);
 		}
 		return pclause;
@@ -784,20 +841,20 @@ public class ReferenceType extends ResolvedType {
 
 	@Override
 	public Collection<Declare> getDeclares() {
-		if (parameterizedDeclares != null)
+		if (parameterizedDeclares != null) {
 			return parameterizedDeclares;
+		}
 		Collection<Declare> declares = null;
 		if (ajMembersNeedParameterization()) {
-			Collection genericDeclares = delegate.getDeclares();
-			parameterizedDeclares = new ArrayList();
-			Map parameterizationMap = getAjMemberParameterizationMap();
-			for (Iterator iter = genericDeclares.iterator(); iter.hasNext();) {
-				Declare declareStatement = (Declare) iter.next();
+			Collection<Declare> genericDeclares = getDelegate().getDeclares();
+			parameterizedDeclares = new ArrayList<Declare>();
+			Map<String, UnresolvedType> parameterizationMap = getAjMemberParameterizationMap();
+			for (Declare declareStatement : genericDeclares) {
 				parameterizedDeclares.add(declareStatement.parameterizeWith(parameterizationMap, world));
 			}
 			declares = parameterizedDeclares;
 		} else {
-			declares = delegate.getDeclares();
+			declares = getDelegate().getDeclares();
 		}
 		for (Declare d : declares) {
 			d.setDeclaringType(this);
@@ -806,13 +863,13 @@ public class ReferenceType extends ResolvedType {
 	}
 
 	@Override
-	public Collection getTypeMungers() {
-		return delegate.getTypeMungers();
+	public Collection<ConcreteTypeMunger> getTypeMungers() {
+		return getDelegate().getTypeMungers();
 	}
 
 	// GENERICITDFIX
 	// // Map parameterizationMap = getAjMemberParameterizationMap();
-	//		
+	//
 	// // if (parameterizedTypeMungers != null) return parameterizedTypeMungers;
 	// Collection ret = null;
 	// if (ajMembersNeedParameterization()) {
@@ -832,33 +889,44 @@ public class ReferenceType extends ResolvedType {
 	// }
 
 	@Override
-	public Collection getPrivilegedAccesses() {
-		return delegate.getPrivilegedAccesses();
+	public Collection<ResolvedMember> getPrivilegedAccesses() {
+		return getDelegate().getPrivilegedAccesses();
 	}
 
 	@Override
 	public int getModifiers() {
-		return delegate.getModifiers();
+		return getDelegate().getModifiers();
 	}
 
+	WeakReference<ResolvedType> superclassReference = new WeakReference<ResolvedType>(null);
+
 	@Override
 	public ResolvedType getSuperclass() {
+		ResolvedType ret = null;// superclassReference.get();
+		// if (ret != null) {
+		// return ret;
+		// }
 		if (newSuperclass != null) {
 			if (this.isParameterizedType() && newSuperclass.isParameterizedType()) {
 				return newSuperclass.parameterize(getMemberParameterizationMap()).resolve(getWorld());
 			}
+			if (getDelegate().isCacheable()) {
+				superclassReference = new WeakReference<ResolvedType>(ret);
+			}
 			return newSuperclass;
 		}
-		ResolvedType ret = null;
 		try {
 			world.setTypeVariableLookupScope(this);
-			ret = delegate.getSuperclass();
+			ret = getDelegate().getSuperclass();
 		} finally {
 			world.setTypeVariableLookupScope(null);
 		}
 		if (this.isParameterizedType() && ret.isParameterizedType()) {
 			ret = ret.parameterize(getMemberParameterizationMap()).resolve(getWorld());
 		}
+		if (getDelegate().isCacheable()) {
+			superclassReference = new WeakReference<ResolvedType>(ret);
+		}
 		return ret;
 	}
 
@@ -870,11 +938,11 @@ public class ReferenceType extends ResolvedType {
 		// Don't copy from BcelObjectType to EclipseSourceType - the context may
 		// be tidied (result null'd) after previous weaving
 		if (this.delegate != null && this.delegate.copySourceContext()
-				&& this.delegate.getSourceContext() != SourceContextImpl.UNKNOWN_SOURCE_CONTEXT)
+				&& this.delegate.getSourceContext() != SourceContextImpl.UNKNOWN_SOURCE_CONTEXT) {
 			((AbstractReferenceTypeDelegate) delegate).setSourceContext(this.delegate.getSourceContext());
+		}
 		this.delegate = delegate;
-		for (Iterator it = this.derivativeTypes.iterator(); it.hasNext();) {
-			ReferenceType dependent = (ReferenceType) it.next();
+		for (ReferenceType dependent : derivativeTypes) {
 			dependent.setDelegate(delegate);
 		}
 
@@ -892,10 +960,12 @@ public class ReferenceType extends ResolvedType {
 
 	private void clearParameterizationCaches() {
 		parameterizedFields = null;
-		parameterizedInterfaces = null;
+		parameterizedInterfaces.clear();
 		parameterizedMethods = null;
 		parameterizedPointcuts = null;
+		superclassReference = new WeakReference<ResolvedType>(null);
 	}
+	
 
 	public int getEndPos() {
 		return endPos;
@@ -915,11 +985,11 @@ public class ReferenceType extends ResolvedType {
 
 	@Override
 	public boolean doesNotExposeShadowMungers() {
-		return delegate.doesNotExposeShadowMungers();
+		return getDelegate().doesNotExposeShadowMungers();
 	}
 
 	public String getDeclaredGenericSignature() {
-		return delegate.getDeclaredGenericSignature();
+		return getDelegate().getDeclaredGenericSignature();
 	}
 
 	public void setGenericType(ReferenceType rt) {
@@ -930,6 +1000,13 @@ public class ReferenceType extends ResolvedType {
 			typeKind = TypeKind.RAW;
 			signatureErasure = signature;
 		}
+		if (typeKind==TypeKind.RAW){
+			genericType.addDependentType(this);
+		}
+		if (this.isRawType() && rt.isRawType()) {
+			new RuntimeException("PR341926 diagnostics: Incorrect setup for a generic type, raw type should not point to raw: "
+					+ this.getName()).printStackTrace();
+		}
 	}
 
 	public void demoteToSimpleType() {
@@ -940,8 +1017,9 @@ public class ReferenceType extends ResolvedType {
 
 	@Override
 	public ResolvedType getGenericType() {
-		if (isGenericType())
+		if (isGenericType()) {
 			return this;
+		}
 		return genericType;
 	}
 
@@ -971,7 +1049,12 @@ public class ReferenceType extends ResolvedType {
 		ret.append(rawSig.substring(0, rawSig.length() - 1));
 		ret.append("<");
 		for (int i = 0; i < someParameters.length; i++) {
-			ret.append(((ReferenceType) someParameters[i]).getSignatureForAttribute());
+			try {
+				ret.append(((ReferenceType) someParameters[i]).getSignatureForAttribute());
+			} catch (ClassCastException cce) {
+				throw new IllegalStateException("DebugFor325731: expected a ReferenceType but was "+someParameters[i]+
+						" of type "+someParameters[i].getClass().getName(),cce);
+			}
 		}
 		ret.append(">;");
 		return ret.toString();
@@ -983,18 +1066,25 @@ public class ReferenceType extends ResolvedType {
 		annotationTypes = null;
 		newSuperclass = null;
 		newInterfaces = null;
+		typeVariables = null;
+		parameterizedInterfaces.clear();
+		superclassReference = new WeakReference<ResolvedType>(null);
+		if (getDelegate() != null) {
+			delegate.ensureConsistent();
+		}
 	}
 
 	@Override
 	public void addParent(ResolvedType newParent) {
 		if (newParent.isClass()) {
 			newSuperclass = newParent;
+			superclassReference = new WeakReference<ResolvedType>(null);
 		} else {
 			if (newInterfaces == null) {
 				newInterfaces = new ResolvedType[1];
 				newInterfaces[0] = newParent;
 			} else {
-				ResolvedType[] existing = delegate.getDeclaredInterfaces();
+				ResolvedType[] existing = getDelegate().getDeclaredInterfaces();
 				if (existing != null) {
 					for (int i = 0; i < existing.length; i++) {
 						if (existing[i].equals(newParent)) {
@@ -1006,8 +1096,13 @@ public class ReferenceType extends ResolvedType {
 				System.arraycopy(newInterfaces, 0, newNewInterfaces, 1, newInterfaces.length);
 				newNewInterfaces[0] = newParent;
 				newInterfaces = newNewInterfaces;
-				parameterizedInterfaces = null;// invalidate cached info
 			}
+			if (this.isGenericType()) {
+				for (ReferenceType derivativeType:derivativeTypes) {
+					derivativeType.parameterizedInterfaces.clear();
+				}
+			}
+			parameterizedInterfaces.clear();
 		}
 	}
 }
\ No newline at end of file
