@@ -27,8 +27,7 @@ import org.aspectj.bridge.ISourceLocation;
  * @author PARC
  * @author Andy Clement
  */
-public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, AnnotatedElement, TypeVariableDeclaringElement,
-		ResolvedMember {
+public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, ResolvedMember {
 
 	private String[] parameterNames = null;
 	protected UnresolvedType[] checkedExceptions = UnresolvedType.NONE;
@@ -524,6 +523,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 				for (int i = 0; i < tvcount; i++) {
 					m.typeVariables[i] = TypeVariable.read(s);
 					m.typeVariables[i].setDeclaringElement(m);
+					m.typeVariables[i].setRank(i);
 				}
 			}
 			if (s.getMajorVersion() >= AjAttribute.WeaverVersionInfo.WEAVER_VERSION_MAJOR_AJ150M4) {
@@ -742,10 +742,8 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 				typeMap.put(typeVariables[i].getName(), ut);
 			}
 		}
-		// For ITDs on generic types that use type variables from the target
-		// type, the aliases
-		// record the alternative names used throughout the ITD expression that
-		// must map to
+		// For ITDs on generic types that use type variables from the target type, the aliases
+		// record the alternative names used throughout the ITD expression that must map to
 		// the same value as the type variables real name.
 		if (aliases != null) {
 			int posn = 0;
@@ -756,13 +754,13 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 			}
 		}
 
-		UnresolvedType parameterizedReturnType = parameterize(getGenericReturnType(), typeMap, isParameterized, newDeclaringType
-				.getWorld());
+		UnresolvedType parameterizedReturnType = parameterize(getGenericReturnType(), typeMap, isParameterized,
+				newDeclaringType.getWorld());
 		UnresolvedType[] parameterizedParameterTypes = new UnresolvedType[getGenericParameterTypes().length];
 		UnresolvedType[] genericParameterTypes = getGenericParameterTypes();
 		for (int i = 0; i < parameterizedParameterTypes.length; i++) {
-			parameterizedParameterTypes[i] = parameterize(genericParameterTypes[i], typeMap, isParameterized, newDeclaringType
-					.getWorld());
+			parameterizedParameterTypes[i] = parameterize(genericParameterTypes[i], typeMap, isParameterized,
+					newDeclaringType.getWorld());
 		}
 		ResolvedMemberImpl ret = new ResolvedMemberImpl(getKind(), newDeclaringType, getModifiers(), parameterizedReturnType,
 				getName(), parameterizedParameterTypes, getExceptions(), this);
@@ -930,7 +928,7 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	// }
 	// return null;
 	// }
-	//	
+	//
 	// private ResolvedMember myErasure = null;
 	// private boolean calculatedMyErasure = false;
 	public boolean hasBackingGenericMember() {
@@ -1039,17 +1037,13 @@ public class ResolvedMemberImpl extends MemberImpl implements IHasPosition, Anno
 	 * getParam
 	 */
 	public String getParameterSignatureErased() {
-		if (myParameterSignatureErasure != null) {
-			return myParameterSignatureErasure;
+		if (myParameterSignatureErasure == null) {
+			StringBuilder sig = new StringBuilder();
+			for (UnresolvedType parameter : getParameterTypes()) {
+				sig.append(parameter.getErasureSignature());
+			}
+			myParameterSignatureErasure = sig.toString();
 		}
-		StringBuffer sig = new StringBuffer();
-		UnresolvedType[] myParameterTypes = getParameterTypes();
-		for (int i = 0; i < myParameterTypes.length; i++) {
-			UnresolvedType thisParameter = myParameterTypes[i];
-			// type vars will be erased to first bound
-			sig.append(thisParameter.getErasureSignature());
-		}
-		myParameterSignatureErasure = sig.toString();
 		return myParameterSignatureErasure;
 	}
 
