@@ -21,13 +21,13 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipInputStream;
 
 import org.aspectj.bridge.IMessage;
+import org.aspectj.weaver.AjAttribute.WeaverVersionInfo;
 
 /**
  * WeaverStateInfo represents how a type was processed. It is used by the weaver to determine how a type was previously treated and
@@ -39,16 +39,18 @@ import org.aspectj.bridge.IMessage;
  * previously woven <String> The fully qualified name of each type Int: Length of class file data (i.e. the unwovenclassfile)
  * Byte[]: The class file data, compressed if REWEAVABLE_COMPRESSION_BIT set.
  */
-
 public class WeaverStateInfo {
-	private List/* Entry */typeMungers;
+	private List<Entry> typeMungers;
 	private boolean oldStyle;
 
 	private boolean reweavable;
 	private boolean reweavableCompressedMode; // If true, unwovenClassFile is uncompressed on read
 	private boolean reweavableDiffMode; // if true, unwovenClassFile is written and read as a diff
 
-	private Set /* String */aspectsAffectingType; // These must exist in the world for reweaving to be valid
+	// These must exist in the world for reweaving to be valid.
+	// It is a set of signatures 'La/b/c/D;'
+	private Set<String> aspectsAffectingType;
+
 	private byte[] unwovenClassFile; // Original 'untouched' class file
 	private static boolean reweavableDefault = true; // ajh02: changed from false;
 	private static boolean reweavableCompressedModeDefault = false;
@@ -62,22 +64,40 @@ public class WeaverStateInfo {
 	private static byte[] key = { -51, 34, 105, 56, -34, 65, 45, 78, -26, 125, 114, 97, 98, 1, -1, -42 };
 	private boolean unwovenClassFileIsADiff = false;
 
+	int compressionEnabled = 0; // 0=dont know, 1=no, 2=yes
+
+	private void checkCompressionEnabled() {
+		if (compressionEnabled == 0) {
+			// work it out!
+			compressionEnabled = 1;
+			try {
+				String value = System.getProperty("aspectj.compression.weaverstateinfo", "false");
+				if (value.equalsIgnoreCase("true")) {
+					System.out.println("ASPECTJ: aspectj.compression.weaverstateinfo=true: compressing weaverstateinfo");
+					compressionEnabled = 2;
+				}
+			} catch (Throwable t) {
+				// nop
+			}
+		}
+	}
+
 	private WeaverStateInfo() {
 		// this(new ArrayList(), false,reweavableDefault,reweavableCompressedModeDefault,reweavableDiffModeDefault);
 	}
 
 	public WeaverStateInfo(boolean reweavable) {
-		this(new ArrayList(), false, reweavable, reweavableCompressedModeDefault, reweavableDiffModeDefault);
+		this(new ArrayList<Entry>(), false, reweavable, reweavableCompressedModeDefault, reweavableDiffModeDefault);
 	}
 
-	private WeaverStateInfo(List typeMungers, boolean oldStyle, boolean reweavableMode, boolean reweavableCompressedMode,
+	private WeaverStateInfo(List<Entry> typeMungers, boolean oldStyle, boolean reweavableMode, boolean reweavableCompressedMode,
 			boolean reweavableDiffMode) {
 		this.typeMungers = typeMungers;
 		this.oldStyle = oldStyle;
 		this.reweavable = reweavableMode;
 		this.reweavableCompressedMode = reweavableCompressedMode;
 		this.reweavableDiffMode = reweavableMode ? reweavableDiffMode : false;
-		this.aspectsAffectingType = new HashSet();
+		this.aspectsAffectingType = new HashSet<String>();
 		this.unwovenClassFile = null;
 	}
 
@@ -119,15 +139,32 @@ public class WeaverStateInfo {
 		case WOVEN:
 			return new WeaverStateInfo(Collections.EMPTY_LIST, true, isReweavable, isReweavableCompressed, isReweavableDiff);
 		case EXTENDED:
+			boolean isCompressed = false;
+			if (s.isAtLeast169()) {
+				isCompressed = s.readBoolean();
+			}
+
 			int n = s.readShort();
 			List l = new ArrayList();
 			for (int i = 0; i < n; i++) {
-				UnresolvedType aspectType = UnresolvedType.read(s);
+				// conditional on version
+				UnresolvedType aspectType = null;
+				if (isCompressed) {
+					int cpIndex = s.readShort();
+					String signature = s.readUtf8(cpIndex);
+					if (signature.charAt(0) == '@') { // '@missing@'
+						aspectType = ResolvedType.MISSING;
+					} else {
+						aspectType = UnresolvedType.forSignature(signature);
+					}
+				} else {
+					aspectType = UnresolvedType.read(s);
+				}
 				ResolvedTypeMunger typeMunger = ResolvedTypeMunger.read(s, context);
 				l.add(new Entry(aspectType, typeMunger));
 			}
 			WeaverStateInfo wsi = new WeaverStateInfo(l, false, isReweavable, isReweavableCompressed, isReweavableDiff);
-			readAnyReweavableData(wsi, s);
+			readAnyReweavableData(wsi, s, isCompressed);
 			return wsi;
 		}
 		throw new RuntimeException("bad WeaverState.Kind: " + b + ".  File was :"
@@ -154,14 +191,16 @@ public class WeaverStateInfo {
 	 * be inserted. When the key is replaced with the diff, the 'kind' byte moves to the front of the attribute - thats why in the
 	 * read logic you'll see it expecting the kind as the first byte.
 	 */
-	public void write(DataOutputStream s) throws IOException {
+	public void write(CompressingDataOutputStream s) throws IOException {
+		checkCompressionEnabled();
 		if (oldStyle || reweavableCompressedMode) {
 			throw new RuntimeException("shouldn't be writing this");
 		}
 
 		byte weaverStateInfoKind = EXTENDED;
-		if (reweavable)
+		if (reweavable) {
 			weaverStateInfoKind |= REWEAVABLE_BIT;
+		}
 
 		if (reweavableDiffMode) {
 			s.write(key); // put key in so we can replace it with the diff later
@@ -170,14 +209,25 @@ public class WeaverStateInfo {
 
 		s.writeByte(weaverStateInfoKind);
 
-		int n = typeMungers.size();
-		s.writeShort(n);
-		for (int i = 0; i < n; i++) {
-			Entry e = (Entry) typeMungers.get(i);
-			e.aspectType.write(s);
-			e.typeMunger.write(s);
+		// Tag whether the remainder of the data is subject to cp compression
+		try {
+			s.compressionEnabled = compressionEnabled == 2;
+			s.writeBoolean(s.canCompress());
+
+			int n = typeMungers.size();
+			s.writeShort(n);
+			for (Entry e : typeMungers) {
+				if (s.canCompress()) {
+					s.writeCompressedSignature(e.aspectType.getSignature());
+				} else {
+					e.aspectType.write(s);
+				}
+				e.typeMunger.write(s);
+			}
+			writeAnyReweavableData(this, s, s.canCompress());
+		} finally {
+			s.compressionEnabled = true;
 		}
-		writeAnyReweavableData(this, s);
 	}
 
 	public void addConcreteMunger(ConcreteTypeMunger munger) {
@@ -185,14 +235,13 @@ public class WeaverStateInfo {
 	}
 
 	public String toString() {
-		return "WeaverStateInfo(" + typeMungers + ", " + oldStyle + ")";
+		return "WeaverStateInfo(aspectsAffectingType=" + aspectsAffectingType + "," + typeMungers + ", " + oldStyle + ")";
 	}
 
-	public List getTypeMungers(ResolvedType onType) {
+	public List<ConcreteTypeMunger> getTypeMungers(ResolvedType onType) {
 		World world = onType.getWorld();
-		List ret = new ArrayList();
-		for (Iterator i = typeMungers.iterator(); i.hasNext();) {
-			Entry entry = (Entry) i.next();
+		List<ConcreteTypeMunger> ret = new ArrayList<ConcreteTypeMunger>();
+		for (Entry entry : typeMungers) {
 			ResolvedType aspectType = world.resolve(entry.aspectType, true);
 			if (aspectType.isMissing()) {
 				world.showMessage(IMessage.ERROR, WeaverMessages.format(WeaverMessages.ASPECT_NEEDED, entry.aspectType, onType),
@@ -229,24 +278,39 @@ public class WeaverStateInfo {
 		reweavable = rw;
 	}
 
-	public void addAspectsAffectingType(Collection /* String */aspects) {
+	public void addAspectsAffectingType(Collection<String> aspects) {
 		aspectsAffectingType.addAll(aspects);
 	}
 
-	public void addAspectAffectingType(String aspectType) {
-		aspectsAffectingType.add(aspectType);
+	public void addAspectAffectingType(String aspectSignature) {
+		aspectsAffectingType.add(aspectSignature);
 	}
 
-	public Set /* String */getAspectsAffectingType() {
+	public Set<String> getAspectsAffectingType() {
 		return this.aspectsAffectingType;
 	}
 
-	private static void readAnyReweavableData(WeaverStateInfo wsi, DataInputStream s) throws IOException {
+	private static void readAnyReweavableData(WeaverStateInfo wsi, VersionedDataInputStream s, boolean compressed)
+			throws IOException {
 		if (wsi.isReweavable()) {
 			// Load list of aspects that need to exist in the world for reweaving to be 'legal'
 			int numberAspectsAffectingType = s.readShort();
 			for (int i = 0; i < numberAspectsAffectingType; i++) {
-				wsi.addAspectAffectingType(s.readUTF());
+				String str = null;
+				if (compressed) {
+					str = s.readSignature();
+				} else {
+					str = s.readUTF();
+					// Prior to 1.6.9 we were writing out names (com.foo.Bar) rather than signatures (Lcom/foo/Bar;)
+					// From 1.6.9 onwards we write out signatures (pr319431)
+					if (s.getMajorVersion() < WeaverVersionInfo.WEAVER_VERSION_AJ169) {
+						// It is a name, make it a signature
+						StringBuilder sb = new StringBuilder();
+						sb.append("L").append(str.replace('.', '/')).append(";");
+						str = sb.toString();
+					}
+				}
+				wsi.addAspectAffectingType(str);
 			}
 
 			int unwovenClassFileSize = s.readInt();
@@ -264,15 +328,17 @@ public class WeaverStateInfo {
 					bytesToGo -= amount;
 				}
 				zis.closeEntry();
-				if (bytesToGo != 0)
+				if (bytesToGo != 0) {
 					throw new IOException("ERROR whilst reading compressed reweavable data, expected " + unwovenClassFileSize
 							+ " bytes, only found " + current);
+				}
 			} else {
 				classData = new byte[unwovenClassFileSize];
 				int bytesread = s.read(classData);
-				if (bytesread != unwovenClassFileSize)
+				if (bytesread != unwovenClassFileSize) {
 					throw new IOException("ERROR whilst reading reweavable data, expected " + unwovenClassFileSize
 							+ " bytes, only found " + bytesread);
+				}
 			}
 
 			// if it was diffMode we'll have to remember to apply the diff if someone
@@ -334,18 +400,20 @@ public class WeaverStateInfo {
 
 	private static final int findEndOfKey(byte[] wovenClassFile) {
 		// looks through the classfile backwards (as the attributes are all near the end)
-		for (int i = wovenClassFile.length - 1; i > 0; i--)
+		for (int i = wovenClassFile.length - 1; i > 0; i--) {
 			if (endOfKeyHere(wovenClassFile, i)) {
 				return i + 1;
 			}
+		}
 		throw new RuntimeException("key not found in wovenClassFile"); // should never happen
 	}
 
 	private static final boolean endOfKeyHere(byte lookIn[], int i) {
-		for (int j = 0; j < key.length; j++)
+		for (int j = 0; j < key.length; j++) {
 			if (key[key.length - 1 - j] != lookIn[i - j]) {
 				return false;
 			}
+		}
 		return true;
 	}
 
@@ -452,18 +520,19 @@ public class WeaverStateInfo {
 		return bos.toByteArray();
 	}
 
-	private static void writeAnyReweavableData(WeaverStateInfo wsi, DataOutputStream s) throws IOException {
+	private static void writeAnyReweavableData(WeaverStateInfo wsi, CompressingDataOutputStream s, boolean compress)
+			throws IOException {
 		if (wsi.isReweavable()) {
 			// Write out list of aspects that must exist next time we try and weave this class
 			s.writeShort(wsi.aspectsAffectingType.size());
-			if (wsi.aspectsAffectingType.size() > 0) {
-				for (Iterator iter = wsi.aspectsAffectingType.iterator(); iter.hasNext();) {
-					String type = (String) iter.next();
+			for (String type : wsi.aspectsAffectingType) {
+				if (compress) {
+					s.writeCompressedSignature(type);
+				} else {
 					s.writeUTF(type);
 				}
 			}
 			byte[] data = wsi.unwovenClassFile;
-
 			// if we're not in diffMode, write the unwovenClassFile now,
 			// otherwise we'll insert it as a diff later
 			if (!wsi.reweavableDiffMode) {
@@ -473,4 +542,17 @@ public class WeaverStateInfo {
 		}
 	}
 
+	/**
+	 * @return true if the supplied aspect is already in the list of those affecting this type
+	 */
+	public boolean isAspectAlreadyApplied(ResolvedType someAspect) {
+		String someAspectSignature = someAspect.getSignature();
+		for (String aspectSignature : aspectsAffectingType) {
+			if (aspectSignature.equals(someAspectSignature)) {
+				return true;
+			}
+		}
+		return false;
+	}
+
 }
