@@ -9,11 +9,12 @@
  *  
  * Contributors: 
  *     PARC     initial implementation
- *     Adrian Colyer, Andy Clement, overhaul for generics 
+ *     Adrian Colyer, Andy Clement, overhaul for generics, Abraham Nevado 
  * ******************************************************************/
 
 package org.aspectj.weaver;
 
+import java.lang.ref.Reference;
 import java.lang.ref.ReferenceQueue;
 import java.lang.ref.SoftReference;
 import java.lang.ref.WeakReference;
@@ -29,14 +30,16 @@ import java.util.Set;
 import java.util.WeakHashMap;
 
 import org.aspectj.bridge.IMessage;
+import org.aspectj.bridge.IMessage.Kind;
 import org.aspectj.bridge.IMessageHandler;
 import org.aspectj.bridge.ISourceLocation;
 import org.aspectj.bridge.Message;
 import org.aspectj.bridge.MessageUtil;
-import org.aspectj.bridge.IMessage.Kind;
 import org.aspectj.bridge.context.PinpointingMessageHandler;
 import org.aspectj.util.IStructureModel;
+import org.aspectj.weaver.ResolvedType.Primitive;
 import org.aspectj.weaver.UnresolvedType.TypeKind;
+import org.aspectj.weaver.patterns.Declare;
 import org.aspectj.weaver.patterns.DeclareAnnotation;
 import org.aspectj.weaver.patterns.DeclareParents;
 import org.aspectj.weaver.patterns.DeclarePrecedence;
@@ -135,8 +138,15 @@ public abstract class World implements Dump.INode {
 	protected boolean bcelRepositoryCaching = xsetBCEL_REPOSITORY_CACHING_DEFAULT.equalsIgnoreCase("true");
 	private boolean fastMethodPacking = false;
 	private int itdVersion = 2; // defaults to 2nd generation itds
+
+	// Minimal Model controls whether model entities that are not involved in relationships are deleted post-build
+	private boolean minimalModel = true;
+	private boolean useFinal = true;
+	private boolean targettingRuntime1_6_10 = false;
+
 	private boolean completeBinaryTypes = false;
 	private boolean overWeaving = false;
+	private static boolean systemPropertyOverWeaving = false;
 	public boolean forDEBUG_structuralChangesCode = false;
 	public boolean forDEBUG_bridgingCode = false;
 	public boolean optimizedMatching = true;
@@ -153,30 +163,47 @@ public abstract class World implements Dump.INode {
 	/**
 	 * A list of RuntimeExceptions containing full stack information for every type we couldn't find.
 	 */
-	private List dumpState_cantFindTypeExceptions = null;
+	private List<RuntimeException> dumpState_cantFindTypeExceptions = null;
+
+	static {
+		try {
+			String value = System.getProperty("aspectj.overweaving", "false");
+			if (value.equalsIgnoreCase("true")) {
+				System.out.println("ASPECTJ: aspectj.overweaving=true: overweaving switched ON");
+				systemPropertyOverWeaving = true;
+			}
+		} catch (Throwable t) {
+			System.err.println("ASPECTJ: Unable to read system properties");
+			t.printStackTrace();
+		}
+	}
+
+	public final Primitive BYTE = new Primitive("B", 1, 0);
+	public final Primitive CHAR = new Primitive("C", 1, 1);
+	public final Primitive DOUBLE = new Primitive("D", 2, 2);
+	public final Primitive FLOAT = new Primitive("F", 1, 3);
+	public final Primitive INT = new Primitive("I", 1, 4);
+	public final Primitive LONG = new Primitive("J", 2, 5);
+	public final Primitive SHORT = new Primitive("S", 1, 6);
+	public final Primitive BOOLEAN = new Primitive("Z", 1, 7);
+	public final Primitive VOID = new Primitive("V", 0, 8);
 
 	/**
-	 * Play God. On the first day, God created the primitive types and put them in the type map.
+	 * Insert the primitives
 	 */
 	protected World() {
 		super();
-		if (trace.isTraceEnabled()) {
-			trace.enter("<init>", this);
-		}
-		Dump.registerNode(this.getClass(), this);
-		typeMap.put("B", ResolvedType.BYTE);
-		typeMap.put("S", ResolvedType.SHORT);
-		typeMap.put("I", ResolvedType.INT);
-		typeMap.put("J", ResolvedType.LONG);
-		typeMap.put("F", ResolvedType.FLOAT);
-		typeMap.put("D", ResolvedType.DOUBLE);
-		typeMap.put("C", ResolvedType.CHAR);
-		typeMap.put("Z", ResolvedType.BOOLEAN);
-		typeMap.put("V", ResolvedType.VOID);
+		// Dump.registerNode(this.getClass(), this);
+		typeMap.put("B", BYTE);
+		typeMap.put("S", SHORT);
+		typeMap.put("I", INT);
+		typeMap.put("J", LONG);
+		typeMap.put("F", FLOAT);
+		typeMap.put("D", DOUBLE);
+		typeMap.put("C", CHAR);
+		typeMap.put("Z", BOOLEAN);
+		typeMap.put("V", VOID);
 		precedenceCalculator = new AspectPrecedenceCalculator(this);
-		if (trace.isTraceEnabled()) {
-			trace.exit("<init>");
-		}
 	}
 
 	/**
@@ -199,10 +226,8 @@ public abstract class World implements Dump.INode {
 	}
 
 	// ==========================================================================
-	// ===
 	// T Y P E R E S O L U T I O N
 	// ==========================================================================
-	// ===
 
 	/**
 	 * Resolve a type that we require to be present in the world
@@ -238,7 +263,7 @@ public abstract class World implements Dump.INode {
 	 */
 	public ResolvedType[] resolve(UnresolvedType[] types) {
 		if (types == null) {
-			return new ResolvedType[0];
+			return ResolvedType.NONE;
 		}
 
 		ResolvedType[] ret = new ResolvedType[types.length];
@@ -257,12 +282,16 @@ public abstract class World implements Dump.INode {
 		if (ty instanceof ResolvedType) {
 			ResolvedType rty = (ResolvedType) ty;
 			rty = resolve(rty);
-			return rty;
+			// A TypeVariableReferenceType may look like it is resolved (it extends ResolvedType) but the internal
+			// type variable may not yet have been resolved
+			if (!rty.isTypeVariableReference() || ((TypeVariableReferenceType) rty).isTypeVariableResolved()) {
+				return rty;
+			}
 		}
 
 		// dispatch back to the type variable reference to resolve its
-		// constituent parts
-		// don't do this for other unresolved types otherwise you'll end up in a
+		// constituent parts don't do this for other unresolved types otherwise
+		// you'll end up in a
 		// loop
 		if (ty.isTypeVariableReference()) {
 			return ty.resolve(this);
@@ -289,24 +318,25 @@ public abstract class World implements Dump.INode {
 		}
 
 		// no existing resolved type, create one
-		if (ty.isArray()) {
-			ResolvedType componentType = resolve(ty.getComponentType(), allowMissing);
-			// String brackets =
-			// signature.substring(0,signature.lastIndexOf("[")+1);
-			ret = new ArrayReferenceType(signature, "[" + componentType.getErasureSignature(), this, componentType);
-		} else {
-			ret = resolveToReferenceType(ty, allowMissing);
-			if (!allowMissing && ret.isMissing()) {
-				ret = handleRequiredMissingTypeDuringResolution(ty);
-			}
-			if (completeBinaryTypes) {
-				completeBinaryType(ret);
+		synchronized (buildingTypeLock) {
+			if (ty.isArray()) {
+				ResolvedType componentType = resolve(ty.getComponentType(), allowMissing);
+				ret = new ArrayReferenceType(signature, "[" + componentType.getErasureSignature(), this, componentType);
+			} else {
+				ret = resolveToReferenceType(ty, allowMissing);
+				if (!allowMissing && ret.isMissing()) {
+					ret = handleRequiredMissingTypeDuringResolution(ty);
+				}
+				if (completeBinaryTypes) {
+					completeBinaryType(ret);
+				}
 			}
 		}
 
 		// Pulling in the type may have already put the right entry in the map
 		ResolvedType result = typeMap.get(signature);
 		if (result == null && !ret.isMissing()) {
+			ret = ensureRawTypeIfNecessary(ret);
 			typeMap.put(signature, ret);
 			return ret;
 		}
@@ -317,6 +347,8 @@ public abstract class World implements Dump.INode {
 		}
 	}
 
+	private Object buildingTypeLock = new Object();
+
 	// Only need one representation of '?' in a world - can be shared
 	private BoundedReferenceType wildcard;
 
@@ -351,7 +383,7 @@ public abstract class World implements Dump.INode {
 		// MessageUtil.error(messageHandler,
 		// WeaverMessages.format(WeaverMessages.CANT_FIND_TYPE,ty.getName()));
 		if (dumpState_cantFindTypeExceptions == null) {
-			dumpState_cantFindTypeExceptions = new ArrayList();
+			dumpState_cantFindTypeExceptions = new ArrayList<RuntimeException>();
 		}
 		if (dumpState_cantFindTypeExceptions.size() < 100) { // limit growth
 			dumpState_cantFindTypeExceptions.add(new RuntimeException("Can't find type " + ty.getName()));
@@ -385,11 +417,7 @@ public abstract class World implements Dump.INode {
 	 * @return a type suitable for putting into the world
 	 */
 	private ResolvedType ensureRawTypeIfNecessary(ResolvedType type) {
-		if (!isInJava5Mode()) {
-			// Don't care, not running in 1.5 mode
-			return type;
-		}
-		if (type.isRawType()) {
+		if (!isInJava5Mode() || type.isRawType()) {
 			return type;
 		}
 		// Key requirement here is if it is generic, create a RAW entry to be put in the map that points to it
@@ -415,6 +443,10 @@ public abstract class World implements Dump.INode {
 		return ret;
 	}
 
+	public ReferenceType resolveToReferenceType(String name) {
+		return (ReferenceType) resolve(name);
+	}
+
 	public ResolvedType resolve(String name, boolean allowMissing) {
 		return resolve(UnresolvedType.forName(name), allowMissing);
 	}
@@ -436,7 +468,8 @@ public abstract class World implements Dump.INode {
 
 		} else if (ty.isGenericType()) {
 			// ======= generic types ======================
-			ReferenceType genericType = (ReferenceType) resolveGenericTypeFor(ty, false);
+			ResolvedType rt = resolveGenericTypeFor(ty, false);
+			ReferenceType genericType = (ReferenceType) rt;
 			return genericType;
 
 		} else if (ty.isGenericWildcard()) {
@@ -514,6 +547,7 @@ public abstract class World implements Dump.INode {
 
 		if (genericType != null) {
 			genericType.world = this;
+			((ReferenceType) genericType).addDependentType((ReferenceType) rawType);
 			return genericType;
 		} else {
 			// Fault in the generic that underpins the raw type ;)
@@ -529,8 +563,8 @@ public abstract class World implements Dump.INode {
 	private ReferenceType makeGenericTypeFrom(ReferenceTypeDelegate delegate, ReferenceType rawType) {
 		String genericSig = delegate.getDeclaredGenericSignature();
 		if (genericSig != null) {
-			return new ReferenceType(UnresolvedType.forGenericTypeSignature(rawType.getSignature(), delegate
-					.getDeclaredGenericSignature()), this);
+			return new ReferenceType(UnresolvedType.forGenericTypeSignature(rawType.getSignature(),
+					delegate.getDeclaredGenericSignature()), this);
 		} else {
 			return new ReferenceType(UnresolvedType.forGenericTypeVariables(rawType.getSignature(), delegate.getTypeVariables()),
 					this);
@@ -646,8 +680,8 @@ public abstract class World implements Dump.INode {
 	 */
 	public final Advice createAdviceMunger(AdviceKind kind, Pointcut p, Member signature, int extraParameterFlags,
 			IHasSourceLocation loc, ResolvedType declaringAspect) {
-		AjAttribute.AdviceAttribute attribute = new AjAttribute.AdviceAttribute(kind, p, extraParameterFlags, loc.getStart(), loc
-				.getEnd(), loc.getSourceContext());
+		AjAttribute.AdviceAttribute attribute = new AjAttribute.AdviceAttribute(kind, p, extraParameterFlags, loc.getStart(),
+				loc.getEnd(), loc.getSourceContext());
 		return getWeavingSupport().createAdviceMunger(attribute, p, signature, declaringAspect);
 	}
 
@@ -797,6 +831,20 @@ public abstract class World implements Dump.INode {
 		Xpinpoint = b;
 	}
 
+	public boolean useFinal() {
+		return useFinal;
+	}
+
+	public boolean isMinimalModel() {
+		ensureAdvancedConfigurationProcessed();
+		return minimalModel;
+	}
+
+	public boolean isTargettingRuntime1_6_10() {
+		ensureAdvancedConfigurationProcessed();
+		return targettingRuntime1_6_10;
+	}
+
 	public void setBehaveInJava5Way(boolean b) {
 		behaveInJava5Way = b;
 	}
@@ -875,6 +923,8 @@ public abstract class World implements Dump.INode {
 		return extraConfiguration;
 	}
 
+	public final static String xsetAVOID_FINAL = "avoidFinal"; // default true
+
 	public final static String xsetWEAVE_JAVA_PACKAGES = "weaveJavaPackages"; // default
 	// false
 	// -
@@ -912,6 +962,8 @@ public abstract class World implements Dump.INode {
 	public final static String xsetITD_VERSION_ORIGINAL = "1";
 	public final static String xsetITD_VERSION_2NDGEN = "2";
 	public final static String xsetITD_VERSION_DEFAULT = xsetITD_VERSION_2NDGEN;
+	public final static String xsetMINIMAL_MODEL = "minimalModel";
+	public final static String xsetTARGETING_RUNTIME_1610 = "targetRuntime1_6_10";
 
 	public boolean isInJava5Mode() {
 		return behaveInJava5Way;
@@ -968,17 +1020,18 @@ public abstract class World implements Dump.INode {
 	 */
 	public static class TypeMap {
 
-		private static boolean debug = false;
-
-		private boolean demotionSystemActive;
-		private boolean debugDemotion = false;
-
 		// Strategy for entries in the expendable map
 		public final static int DONT_USE_REFS = 0; // Hang around forever
 		public final static int USE_WEAK_REFS = 1; // Collected asap
-		public final static int USE_SOFT_REFS = 2; // Collected when short on
-		// memory
-		List<String> addedSinceLastDemote;
+		public final static int USE_SOFT_REFS = 2; // Collected when short on memory
+
+		public List<String> addedSinceLastDemote;
+		public List<String> writtenClasses;
+
+		private static boolean debug = false;
+		public static boolean useExpendableMap = true; // configurable for reliable testing
+		private boolean demotionSystemActive;
+		private boolean debugDemotion = false;
 
 		public int policy = USE_WEAK_REFS;
 
@@ -986,7 +1039,8 @@ public abstract class World implements Dump.INode {
 		final Map<String, ResolvedType> tMap = new HashMap<String, ResolvedType>();
 
 		// Map of types that may be ejected from the cache if we need space
-		final Map expendableMap = Collections.synchronizedMap(new WeakHashMap());
+		final Map<String, Reference<ResolvedType>> expendableMap = Collections
+				.synchronizedMap(new WeakHashMap<String, Reference<ResolvedType>>());
 
 		private final World w;
 
@@ -994,28 +1048,34 @@ public abstract class World implements Dump.INode {
 		private boolean memoryProfiling = false;
 		private int maxExpendableMapSize = -1;
 		private int collectedTypes = 0;
-		private final ReferenceQueue rq = new ReferenceQueue();
+		private final ReferenceQueue<ResolvedType> rq = new ReferenceQueue<ResolvedType>();
 
-		private static Trace trace = TraceFactory.getTraceFactory().getTrace(World.TypeMap.class);
+		// private static Trace trace = TraceFactory.getTraceFactory().getTrace(World.TypeMap.class);
 
 		TypeMap(World w) {
-			demotionSystemActive = w.isDemotionActive();
+			// Demotion activated when switched on and loadtime weaving or in AJDT
+			demotionSystemActive = w.isDemotionActive() && (w.isLoadtimeWeaving() || w.couldIncrementalCompileFollow());
 			addedSinceLastDemote = new ArrayList<String>();
+			writtenClasses = new ArrayList<String>();
 			this.w = w;
 			memoryProfiling = false;// !w.getMessageHandler().isIgnoring(Message.
 			// INFO);
 		}
 
 		// For testing
-		public Map getExpendableMap() {
+		public Map<String, Reference<ResolvedType>> getExpendableMap() {
 			return expendableMap;
 		}
 
 		// For testing
-		public Map getMainMap() {
+		public Map<String, ResolvedType> getMainMap() {
 			return tMap;
 		}
 
+		public int demote() {
+			return demote(false);
+		}
+
 		/**
 		 * Go through any types added during the previous file weave. If any are suitable for demotion, then put them in the
 		 * expendable map where GC can claim them at some point later. Demotion means: the type is not an aspect, the type is not
@@ -1024,33 +1084,103 @@ public abstract class World implements Dump.INode {
 		 * 
 		 * @return number of types demoted
 		 */
-		public int demote() {
+		public int demote(boolean atEndOfCompile) {
 			if (!demotionSystemActive) {
 				return 0;
 			}
+			if (debugDemotion) {
+				System.out.println("Demotion running " + addedSinceLastDemote);
+			}
+			boolean isLtw = w.isLoadtimeWeaving();
 			int demotionCounter = 0;
-			for (String key : addedSinceLastDemote) {
-				ResolvedType type = tMap.get(key);
-				if (type != null && !type.isAspect() && !type.equals(UnresolvedType.OBJECT) && !type.isPrimitiveType()) {
-					List<ConcreteTypeMunger> typeMungers = type.getInterTypeMungers();
-					if (typeMungers == null || typeMungers.size() == 0) {
-						tMap.remove(key);
-						if (!expendableMap.containsKey(key)) {
-							if (policy == USE_SOFT_REFS) {
-								expendableMap.put(key, new SoftReference(type));
-							} else {
-								expendableMap.put(key, new WeakReference(type));
-							}
+			if (isLtw) {
+				// Loadtime weaving demotion strategy
+				for (String key : addedSinceLastDemote) {
+					ResolvedType type = tMap.get(key);
+					if (type != null && !type.isAspect() && !type.equals(UnresolvedType.OBJECT) && !type.isPrimitiveType()) {
+						List<ConcreteTypeMunger> typeMungers = type.getInterTypeMungers();
+						if (typeMungers == null || typeMungers.size() == 0) {
+							tMap.remove(key);
+							insertInExpendableMap(key, type);
+							demotionCounter++;
 						}
-						demotionCounter++;
+					}
+				}
+				addedSinceLastDemote.clear();
+			} else {
+				// Compile time demotion strategy
+				List<String> forRemoval = new ArrayList<String>();
+				for (String key : addedSinceLastDemote) {
+					ResolvedType type = tMap.get(key);
+					if (type == null) {
+						// TODO not 100% sure why it is not there, where did it go?
+						forRemoval.add(key);
+						continue;
+					}
+					if (!writtenClasses.contains(type.getName())) { // COSTLY
+						continue;
+					}
+					if (type != null && !type.isAspect() && !type.equals(UnresolvedType.OBJECT) && !type.isPrimitiveType()) {
+						List<ConcreteTypeMunger> typeMungers = type.getInterTypeMungers();
+						if (typeMungers == null || typeMungers.size() == 0) {
+							/*
+							 * if (type.isNested()) { try { ReferenceType rt = (ReferenceType) w.resolve(type.getOutermostType());
+							 * if (!rt.isMissing()) { ReferenceTypeDelegate delegate = ((ReferenceType) type).getDelegate(); boolean
+							 * isWeavable = delegate == null ? false : delegate.isExposedToWeaver(); boolean hasBeenWoven = delegate
+							 * == null ? false : delegate.hasBeenWoven(); if (isWeavable && !hasBeenWoven) { // skip demotion of
+							 * this inner type for now continue; } } } catch (ClassCastException cce) { cce.printStackTrace();
+							 * System.out.println("outer of " + key + " is not a reftype? " + type.getOutermostType()); // throw new
+							 * IllegalStateException(cce); } }
+							 */
+							ReferenceTypeDelegate delegate = ((ReferenceType) type).getDelegate();
+							boolean isWeavable = delegate == null ? false : delegate.isExposedToWeaver();
+							boolean hasBeenWoven = delegate == null ? false : delegate.hasBeenWoven();
+							if (!isWeavable || hasBeenWoven) {
+								if (debugDemotion) {
+									System.out.println("Demoting " + key);
+								}
+								forRemoval.add(key);
+								tMap.remove(key);
+								insertInExpendableMap(key, type);
+								demotionCounter++;
+							}
+						} else {
+							// no need to try this again, it will never be demoted
+							writtenClasses.remove(type.getName());
+							forRemoval.add(key);
+						}
+					} else {
+						writtenClasses.remove(type.getName());
+						// no need to try this again, it will never be demoted
+						forRemoval.add(key);
+					}
+				}
+				addedSinceLastDemote.removeAll(forRemoval);
+			}
+			if (debugDemotion) {
+				System.out.println("Demoted " + demotionCounter + " types.  Types remaining in fixed set #" + tMap.keySet().size()
+						+ ".  addedSinceLastDemote size is " + addedSinceLastDemote.size());
+				System.out.println("writtenClasses.size() = " + writtenClasses.size() + ": " + writtenClasses);
+			}
+			if (atEndOfCompile) {
+				if (debugDemotion) {
+					System.out.println("Clearing writtenClasses");
+				}
+				writtenClasses.clear();
+			}
+			return demotionCounter;
+		}
+
+		private void insertInExpendableMap(String key, ResolvedType type) {
+			if (useExpendableMap) {
+				if (!expendableMap.containsKey(key)) {
+					if (policy == USE_SOFT_REFS) {
+						expendableMap.put(key, new SoftReference<ResolvedType>(type));
+					} else {
+						expendableMap.put(key, new WeakReference<ResolvedType>(type));
 					}
 				}
 			}
-			if (debugDemotion) {
-				System.out.println("Demoted " + demotionCounter + " types.  Types remaining in fixed set #" + tMap.keySet().size());
-			}
-			addedSinceLastDemote.clear();
-			return demotionCounter;
 		}
 
 		/**
@@ -1105,22 +1235,31 @@ public abstract class World implements Dump.INode {
 				return type;
 			}
 
+			// TODO should this be in as a permanent assertion?
+			/*
+			 * if ((type instanceof ReferenceType) && type.getWorld().isInJava5Mode() && (((ReferenceType) type).getDelegate() !=
+			 * null) && type.isGenericType()) { throw new BCException("Attempt to add generic type to typemap " + type.toString() +
+			 * " (should be raw)"); }
+			 */
+
 			if (w.isExpendable(type)) {
-				// Dont use reference queue for tracking if not profiling...
-				if (policy == USE_WEAK_REFS) {
-					if (memoryProfiling) {
-						expendableMap.put(key, new WeakReference(type, rq));
-					} else {
-						expendableMap.put(key, new WeakReference(type));
+				if (useExpendableMap) {
+					// Dont use reference queue for tracking if not profiling...
+					if (policy == USE_WEAK_REFS) {
+						if (memoryProfiling) {
+							expendableMap.put(key, new WeakReference<ResolvedType>(type, rq));
+						} else {
+							expendableMap.put(key, new WeakReference<ResolvedType>(type));
+						}
+					} else if (policy == USE_SOFT_REFS) {
+						if (memoryProfiling) {
+							expendableMap.put(key, new SoftReference<ResolvedType>(type, rq));
+						} else {
+							expendableMap.put(key, new SoftReference<ResolvedType>(type));
+						}
+						// } else {
+						// expendableMap.put(key, type);
 					}
-				} else if (policy == USE_SOFT_REFS) {
-					if (memoryProfiling) {
-						expendableMap.put(key, new SoftReference(type, rq));
-					} else {
-						expendableMap.put(key, new SoftReference(type));
-					}
-				} else {
-					expendableMap.put(key, type);
 				}
 				if (memoryProfiling && expendableMap.size() > maxExpendableMapSize) {
 					maxExpendableMapSize = expendableMap.size();
@@ -1129,6 +1268,7 @@ public abstract class World implements Dump.INode {
 			} else {
 
 				if (demotionSystemActive) {
+					// System.out.println("Added since last demote " + key);
 					addedSinceLastDemote.add(key);
 				}
 
@@ -1165,17 +1305,17 @@ public abstract class World implements Dump.INode {
 			ResolvedType ret = tMap.get(key);
 			if (ret == null) {
 				if (policy == USE_WEAK_REFS) {
-					WeakReference ref = (WeakReference) expendableMap.get(key);
-					if (ref != null) {
-						ret = (ResolvedType) ref.get();
-					}
-				} else if (policy == USE_SOFT_REFS) {
-					SoftReference<ResolvedType> ref = (SoftReference) expendableMap.get(key);
+					WeakReference<ResolvedType> ref = (WeakReference<ResolvedType>) expendableMap.get(key);
 					if (ref != null) {
 						ret = ref.get();
 					}
-				} else {
-					return (ResolvedType) expendableMap.get(key);
+				} else if (policy == USE_SOFT_REFS) {
+					SoftReference<ResolvedType> ref = (SoftReference<ResolvedType>) expendableMap.get(key);
+					if (ref != null) {
+						ret = ref.get();
+					}
+					// } else {
+					// return (ResolvedType) expendableMap.get(key);
 				}
 			}
 			return ret;
@@ -1186,22 +1326,41 @@ public abstract class World implements Dump.INode {
 			ResolvedType ret = tMap.remove(key);
 			if (ret == null) {
 				if (policy == USE_WEAK_REFS) {
-					WeakReference wref = (WeakReference) expendableMap.remove(key);
+					WeakReference<ResolvedType> wref = (WeakReference<ResolvedType>) expendableMap.remove(key);
 					if (wref != null) {
-						ret = (ResolvedType) wref.get();
+						ret = wref.get();
 					}
 				} else if (policy == USE_SOFT_REFS) {
-					SoftReference wref = (SoftReference) expendableMap.remove(key);
+					SoftReference<ResolvedType> wref = (SoftReference<ResolvedType>) expendableMap.remove(key);
 					if (wref != null) {
-						ret = (ResolvedType) wref.get();
+						ret = wref.get();
 					}
-				} else {
-					ret = (ResolvedType) expendableMap.remove(key);
+					// } else {
+					// ret = (ResolvedType) expendableMap.remove(key);
 				}
 			}
 			return ret;
 		}
 
+		public void classWriteEvent(String classname) {
+			// that is a name com.Foo and not a signature Lcom/Foo; boooooooooo!
+			if (demotionSystemActive) {
+				writtenClasses.add(classname);
+			}
+			if (debugDemotion) {
+				System.out.println("Class write event for " + classname);
+			}
+		}
+
+		public void demote(ResolvedType type) {
+			String key = type.getSignature();
+			if (debugDemotion) {
+				addedSinceLastDemote.remove(key);
+			}
+			tMap.remove(key);
+			insertInExpendableMap(key, type);
+		}
+
 		// public ResolvedType[] getAllTypes() {
 		// List/* ResolvedType */results = new ArrayList();
 		//
@@ -1231,11 +1390,11 @@ public abstract class World implements Dump.INode {
 	private static class AspectPrecedenceCalculator {
 
 		private final World world;
-		private final Map cachedResults;
+		private final Map<PrecedenceCacheKey, Integer> cachedResults;
 
 		public AspectPrecedenceCalculator(World forSomeWorld) {
 			world = forSomeWorld;
-			cachedResults = new HashMap();
+			cachedResults = new HashMap<PrecedenceCacheKey, Integer>();
 		}
 
 		/**
@@ -1245,13 +1404,13 @@ public abstract class World implements Dump.INode {
 		public int compareByPrecedence(ResolvedType firstAspect, ResolvedType secondAspect) {
 			PrecedenceCacheKey key = new PrecedenceCacheKey(firstAspect, secondAspect);
 			if (cachedResults.containsKey(key)) {
-				return ((Integer) cachedResults.get(key)).intValue();
+				return (cachedResults.get(key)).intValue();
 			} else {
 				int order = 0;
 				DeclarePrecedence orderer = null; // Records the declare
 				// precedence statement that
 				// gives the first ordering
-				for (Iterator i = world.getCrosscuttingMembersSet().getDeclareDominates().iterator(); i.hasNext();) {
+				for (Iterator<Declare> i = world.getCrosscuttingMembersSet().getDeclareDominates().iterator(); i.hasNext();) {
 					DeclarePrecedence d = (DeclarePrecedence) i.next();
 					int thisOrder = d.compare(firstAspect, secondAspect);
 					if (thisOrder != 0) {
@@ -1276,7 +1435,7 @@ public abstract class World implements Dump.INode {
 		}
 
 		public Integer getPrecedenceIfAny(ResolvedType aspect1, ResolvedType aspect2) {
-			return (Integer) cachedResults.get(new PrecedenceCacheKey(aspect1, aspect2));
+			return cachedResults.get(new PrecedenceCacheKey(aspect1, aspect2));
 		}
 
 		public int compareByPrecedenceAndHierarchy(ResolvedType firstAspect, ResolvedType secondAspect) {
@@ -1330,7 +1489,7 @@ public abstract class World implements Dump.INode {
 	// resolving types (*cough* java.lang.Enum) ---
 
 	public boolean isDemotionActive() {
-		return false;
+		return true;
 	}
 
 	// --- this first map is for java15 delegates which may try and recursively
@@ -1338,17 +1497,17 @@ public abstract class World implements Dump.INode {
 	// --- I would rather stash this against a reference type - but we don't
 	// guarantee referencetypes are unique for
 	// so we can't :(
-	private final Map workInProgress1 = new HashMap();
+	private final Map<Class<?>, TypeVariable[]> workInProgress1 = new HashMap<Class<?>, TypeVariable[]>();
 
-	public TypeVariable[] getTypeVariablesCurrentlyBeingProcessed(Class baseClass) {
-		return (TypeVariable[]) workInProgress1.get(baseClass);
+	public TypeVariable[] getTypeVariablesCurrentlyBeingProcessed(Class<?> baseClass) {
+		return workInProgress1.get(baseClass);
 	}
 
-	public void recordTypeVariablesCurrentlyBeingProcessed(Class baseClass, TypeVariable[] typeVariables) {
+	public void recordTypeVariablesCurrentlyBeingProcessed(Class<?> baseClass, TypeVariable[] typeVariables) {
 		workInProgress1.put(baseClass, typeVariables);
 	}
 
-	public void forgetTypeVariablesCurrentlyBeingProcessed(Class baseClass) {
+	public void forgetTypeVariablesCurrentlyBeingProcessed(Class<?> baseClass) {
 		workInProgress1.remove(baseClass);
 	}
 
@@ -1388,6 +1547,21 @@ public abstract class World implements Dump.INode {
 					itdVersion = 1;
 				}
 
+				s = p.getProperty(xsetAVOID_FINAL, "false");
+				if (s.equalsIgnoreCase("true")) {
+					useFinal = false; // if avoidFinal=true, then set useFinal to false
+				}
+
+				s = p.getProperty(xsetMINIMAL_MODEL, "true");
+				if (s.equalsIgnoreCase("false")) {
+					minimalModel = false;
+				}
+
+				s = p.getProperty(xsetTARGETING_RUNTIME_1610, "false");
+				if (s.equalsIgnoreCase("true")) {
+					targettingRuntime1_6_10 = true;
+				}
+
 				s = p.getProperty(xsetFAST_PACK_METHODS, "true");
 				fastMethodPacking = s.equalsIgnoreCase("true");
 
@@ -1404,9 +1578,16 @@ public abstract class World implements Dump.INode {
 							MessageUtil.info("[completeBinaryTypes=true] Completion of binary types activated"));
 				}
 
-				s = p.getProperty(xsetTYPE_DEMOTION, "true"); // default is ON
-				if (s.equalsIgnoreCase("false")) {
-					typeMap.demotionSystemActive = false;
+				s = p.getProperty(xsetTYPE_DEMOTION); // default is: ON
+				if (s != null) {
+					boolean b = typeMap.demotionSystemActive;
+					if (b && s.equalsIgnoreCase("false")) {
+						System.out.println("typeDemotion=false: type demotion switched OFF");
+						typeMap.demotionSystemActive = false;
+					} else if (!b && s.equalsIgnoreCase("true")) {
+						System.out.println("typeDemotion=true: type demotion switched ON");
+						typeMap.demotionSystemActive = true;
+					}
 				}
 
 				s = p.getProperty(xsetOVERWEAVING, "false");
@@ -1461,11 +1642,20 @@ public abstract class World implements Dump.INode {
 
 			}
 			try {
-				String value = System.getProperty("aspectj.overweaving", "false");
-				if (value.equalsIgnoreCase("true")) {
-					System.out.println("ASPECTJ: aspectj.overweaving=true: overweaving switched ON");
+				if (systemPropertyOverWeaving) {
 					overWeaving = true;
 				}
+				String value = null;
+				value = System.getProperty("aspectj.typeDemotion", "false");
+				if (value.equalsIgnoreCase("true")) {
+					System.out.println("ASPECTJ: aspectj.typeDemotion=true: type demotion switched ON");
+					typeMap.demotionSystemActive = true;
+				}
+				value = System.getProperty("aspectj.minimalModel", "false");
+				if (value.equalsIgnoreCase("true")) {
+					System.out.println("ASPECTJ: aspectj.minimalModel=true: minimal model switched ON");
+					minimalModel = true;
+				}
 			} catch (Throwable t) {
 				System.err.println("ASPECTJ: Unable to read system properties");
 				t.printStackTrace();
@@ -1562,12 +1752,12 @@ public abstract class World implements Dump.INode {
 		return true;
 	}
 
-    /**
-     * Determine if the named aspect requires a particular type around in order to
-     * be useful.  The type is named in the aop.xml file against the aspect.
-     *
-     * @return true if there is a type missing that this aspect really needed around
-     */
+	/**
+	 * Determine if the named aspect requires a particular type around in order to be useful. The type is named in the aop.xml file
+	 * against the aspect.
+	 * 
+	 * @return true if there is a type missing that this aspect really needed around
+	 */
 	public boolean hasUnsatisfiedDependency(ResolvedType aspectType) {
 		return false;
 	}
@@ -1576,11 +1766,11 @@ public abstract class World implements Dump.INode {
 		return null;
 	}
 
-	public Map getFixed() {
+	public Map<String, ResolvedType> getFixed() {
 		return typeMap.tMap;
 	}
 
-	public Map getExpendable() {
+	public Map<String, Reference<ResolvedType>> getExpendable() {
 		return typeMap.expendableMap;
 	}
 
@@ -1607,7 +1797,8 @@ public abstract class World implements Dump.INode {
 	 * Reference types we don't intend to weave may be ejected from the cache if we need the space.
 	 */
 	protected boolean isExpendable(ResolvedType type) {
-		return (!type.equals(UnresolvedType.OBJECT) && (!type.isExposedToWeaver()) && (!type.isPrimitiveType()));
+		return !type.equals(UnresolvedType.OBJECT) && !type.isExposedToWeaver() && !type.isPrimitiveType()
+				&& !type.isPrimitiveArray();
 	}
 
 	// map from aspect > excluded types
@@ -1787,7 +1978,7 @@ public abstract class World implements Dump.INode {
 	}
 
 	public static void reset() {
-		ResolvedType.resetPrimitives();
+		// ResolvedType.resetPrimitives();
 	}
 
 	/**
@@ -1800,4 +1991,11 @@ public abstract class World implements Dump.INode {
 		return itdVersion;
 	}
 
+	// if not loadtime weaving then we are compile time weaving or post-compile time weaving
+	public abstract boolean isLoadtimeWeaving();
+
+	public void classWriteEvent(char[][] compoundName) {
+		// override if interested in write events
+	}
+
 }
\ No newline at end of file
