@@ -1597,7 +1597,7 @@ public class AtAjAttributes {
 					|| AjcMemberMaker.TYPEX_PROCEEDINGJOINPOINT.equals(argumentType)
 					|| AjcMemberMaker.TYPEX_STATICJOINPOINT.equals(argumentType)
 					|| AjcMemberMaker.TYPEX_ENCLOSINGSTATICJOINPOINT.equals(argumentType) || AjcMemberMaker.AROUND_CLOSURE_TYPE
-					.equals(argumentType))) {
+						.equals(argumentType))) {
 				// continue;// skip
 				bindings.add(new FormalBinding.ImplicitFormalBinding(argumentType, argumentName, i));
 			} else {
@@ -1757,10 +1757,16 @@ public class AtAjAttributes {
 			LocalVariable[] lvt = lt.getLocalVariableTable();
 			for (int j = 0; j < lvt.length; j++) {
 				LocalVariable localVariable = lvt[j];
-				if (localVariable.getStartPC() == 0) {
-					if (localVariable.getIndex() >= startAtStackIndex) {
-						arguments.add(new MethodArgument(localVariable.getName(), localVariable.getIndex()));
+				if (localVariable != null) { // pr348488
+					if (localVariable.getStartPC() == 0) {
+						if (localVariable.getIndex() >= startAtStackIndex) {
+							arguments.add(new MethodArgument(localVariable.getName(), localVariable.getIndex()));
+						}
 					}
+				} else {
+					String typename = (methodStruct.enclosingType != null ? methodStruct.enclosingType.getName() : "");
+					System.err.println("AspectJ: 348488 debug: unusual local variable table for method " + typename + "."
+							+ method.getName());
 				}
 			}
 			if (arguments.size() == 0) {
@@ -1771,12 +1777,14 @@ public class AtAjAttributes {
 				// LocalVariable(start_pc = 6, length = 40, index = 2:int __cobertura__line__number__)
 				// LocalVariable(start_pc = 6, length = 40, index = 3:int __cobertura__branch__number__)
 				LocalVariable localVariable = lvt[0];
-				if (localVariable.getStartPC() != 0) {
-					// looks suspicious so let's use this information
-					for (int j = 0; j < lvt.length && arguments.size() < method.getArgumentTypes().length; j++) {
-						localVariable = lvt[j];
-						if (localVariable.getIndex() >= startAtStackIndex) {
-							arguments.add(new MethodArgument(localVariable.getName(), localVariable.getIndex()));
+				if (localVariable != null) { // pr348488
+					if (localVariable.getStartPC() != 0) {
+						// looks suspicious so let's use this information
+						for (int j = 0; j < lvt.length && arguments.size() < method.getArgumentTypes().length; j++) {
+							localVariable = lvt[j];
+							if (localVariable.getIndex() >= startAtStackIndex) {
+								arguments.add(new MethodArgument(localVariable.getName(), localVariable.getIndex()));
+							}
 						}
 					}
 				}
@@ -1865,7 +1873,7 @@ public class AtAjAttributes {
 
 		public LazyResolvedPointcutDefinition(UnresolvedType declaringType, int modifiers, String name,
 				UnresolvedType[] parameterTypes, UnresolvedType returnType, Pointcut pointcut, IScope binding) {
-			super(declaringType, modifiers, name, parameterTypes, returnType, null);
+			super(declaringType, modifiers, name, parameterTypes, returnType, Pointcut.makeMatchesNothing(Pointcut.RESOLVED));
 			m_pointcutUnresolved = pointcut;
 			m_binding = binding;
 		}
