@@ -31,9 +31,10 @@ import org.aspectj.apache.bcel.classfile.annotation.NameValuePair;
 import org.aspectj.bridge.ISourceLocation;
 import org.aspectj.bridge.SourceLocation;
 import org.aspectj.util.GenericSignature;
-import org.aspectj.util.GenericSignatureParser;
 import org.aspectj.util.GenericSignature.TypeVariableSignature;
+import org.aspectj.util.GenericSignatureParser;
 import org.aspectj.weaver.AjAttribute;
+import org.aspectj.weaver.AjAttribute.WeaverVersionInfo;
 import org.aspectj.weaver.AnnotationAJ;
 import org.aspectj.weaver.BCException;
 import org.aspectj.weaver.ISourceContext;
@@ -91,12 +92,25 @@ class BcelMethod extends ResolvedMemberImpl {
 		unpackAjAttributes(bcelObjectType.getWorld());
 	}
 
+	/**
+	 * This constructor expects to be passed the attributes, rather than deserializing them.
+	 */
+	BcelMethod(BcelObjectType declaringType, Method method, List<AjAttribute> attributes) {
+		super(method.getName().equals("<init>") ? CONSTRUCTOR : (method.getName().equals("<clinit>") ? STATIC_INITIALIZATION
+				: METHOD), declaringType.getResolvedTypeX(), declaringType.isInterface() ? method.getModifiers()
+				| Modifier.INTERFACE : method.getModifiers(), method.getName(), method.getSignature());
+		this.method = method;
+		sourceContext = declaringType.getResolvedTypeX().getSourceContext();
+		bcelObjectType = declaringType;
+		unpackJavaAttributes();
+		processAttributes(bcelObjectType.getWorld(), attributes);
+	}
+
 	// ----
 
 	private void unpackJavaAttributes() {
 		ExceptionTable exnTable = method.getExceptionTable();
 		checkedExceptions = (exnTable == null) ? UnresolvedType.NONE : UnresolvedType.forNames(exnTable.getExceptionNames());
-
 	}
 
 	@Override
@@ -126,7 +140,8 @@ class BcelMethod extends ResolvedMemberImpl {
 		int len = getArity();
 		if (varTable == null) {
 			// do we have an annotation with the argNames value specified...
-			if (hasAnnotations()) {
+			AnnotationAJ[] annos = getAnnotations();
+			if (annos != null && annos.length != 0) {
 				AnnotationAJ[] axs = getAnnotations();
 				for (int i = 0; i < axs.length; i++) {
 					AnnotationAJ annotationX = axs[i];
@@ -181,12 +196,12 @@ class BcelMethod extends ResolvedMemberImpl {
 	private void unpackAjAttributes(World world) {
 		associatedShadowMunger = null;
 		ResolvedType resolvedDeclaringType = getDeclaringType().resolve(world);
+		WeaverVersionInfo wvinfo = bcelObjectType.getWeaverVersionAttribute();
 		List<AjAttribute> as = Utility.readAjAttributes(resolvedDeclaringType.getClassName(), method.getAttributes(),
-				resolvedDeclaringType.getSourceContext(), world, bcelObjectType.getWeaverVersionAttribute(),
-				new BcelConstantPoolReader(method.getConstantPool()));
+				resolvedDeclaringType.getSourceContext(), world, wvinfo, new BcelConstantPoolReader(method.getConstantPool()));
 		processAttributes(world, as);
-		as = AtAjAttributes.readAj5MethodAttributes(method, this, resolvedDeclaringType, preResolvedPointcut, resolvedDeclaringType
-				.getSourceContext(), world.getMessageHandler());
+		as = AtAjAttributes.readAj5MethodAttributes(method, this, resolvedDeclaringType, preResolvedPointcut,
+				resolvedDeclaringType.getSourceContext(), world.getMessageHandler());
 		processAttributes(world, as);
 	}
 
@@ -379,6 +394,39 @@ class BcelMethod extends ResolvedMemberImpl {
 		bitflags |= HAS_ANNOTATIONS;
 	}
 
+	public void removeAnnotation(ResolvedType annotationType) {
+		ensureAnnotationsRetrieved();
+		if ((bitflags & HAS_ANNOTATIONS) == 0) {
+			// nothing to do, why did we get called?
+		} else {
+			int len = annotations.length;
+			if (len == 1) {
+				bitflags &= ~HAS_ANNOTATIONS;
+				annotations = null;
+				annotationTypes = null;
+				return;
+			}
+			AnnotationAJ[] ret = new AnnotationAJ[len - 1];
+			int p = 0;
+			for (AnnotationAJ annotation : annotations) {
+				if (!annotation.getType().equals(annotationType)) {
+					ret[p++] = annotation;
+				}
+			}
+			annotations = ret;
+
+			ResolvedType[] newAnnotationTypes = new ResolvedType[len - 1];
+			p = 0;
+			for (AnnotationAJ annotation : annotations) {
+				if (!annotation.getType().equals(annotationType)) {
+					newAnnotationTypes[p++] = annotationType;
+				}
+			}
+			annotationTypes = newAnnotationTypes;
+		}
+		bitflags |= HAS_ANNOTATIONS;
+	}
+
 	public static final AnnotationAJ[] NO_PARAMETER_ANNOTATIONS = new AnnotationAJ[] {};
 
 	public void addParameterAnnotation(int param, AnnotationAJ anno) {
