@@ -28,6 +28,7 @@ import org.aspectj.apache.bcel.classfile.Attribute;
 import org.aspectj.apache.bcel.classfile.AttributeUtils;
 import org.aspectj.apache.bcel.classfile.ConstantClass;
 import org.aspectj.apache.bcel.classfile.ConstantPool;
+import org.aspectj.apache.bcel.classfile.EnclosingMethod;
 import org.aspectj.apache.bcel.classfile.Field;
 import org.aspectj.apache.bcel.classfile.InnerClass;
 import org.aspectj.apache.bcel.classfile.InnerClasses;
@@ -37,6 +38,7 @@ import org.aspectj.apache.bcel.classfile.Signature;
 import org.aspectj.apache.bcel.classfile.annotation.AnnotationGen;
 import org.aspectj.apache.bcel.classfile.annotation.EnumElementValue;
 import org.aspectj.apache.bcel.classfile.annotation.NameValuePair;
+import org.aspectj.asm.AsmManager;
 import org.aspectj.bridge.IMessageHandler;
 import org.aspectj.bridge.MessageUtil;
 import org.aspectj.util.GenericSignature;
@@ -377,8 +379,8 @@ public class BcelObjectType extends AbstractReferenceTypeDelegate {
 		List<AjAttribute> l = null;
 		try {
 			l = Utility.readAjAttributes(className, javaClass.getAttributes(), getResolvedTypeX().getSourceContext(),
-					getResolvedTypeX().getWorld(), AjAttribute.WeaverVersionInfo.UNKNOWN, new BcelConstantPoolReader(javaClass
-							.getConstantPool()));
+					getResolvedTypeX().getWorld(), AjAttribute.WeaverVersionInfo.UNKNOWN,
+					new BcelConstantPoolReader(javaClass.getConstantPool()));
 		} catch (RuntimeException re) {
 			throw new RuntimeException("Problem processing attributes in " + javaClass.getFileName(), re);
 		}
@@ -386,8 +388,10 @@ public class BcelObjectType extends AbstractReferenceTypeDelegate {
 		typeMungers = new ArrayList<ConcreteTypeMunger>();
 		declares = new ArrayList<Declare>();
 		processAttributes(l, pointcuts, false);
-		l = AtAjAttributes.readAj5ClassAttributes(((BcelWorld) getResolvedTypeX().getWorld()).getModelAsAsmManager(), javaClass,
-				getResolvedTypeX(), getResolvedTypeX().getSourceContext(), msgHandler, isCodeStyleAspect);
+		ReferenceType type = getResolvedTypeX();
+		AsmManager asmManager = ((BcelWorld) type.getWorld()).getModelAsAsmManager();
+		l = AtAjAttributes.readAj5ClassAttributes(asmManager, javaClass, type, type.getSourceContext(), msgHandler,
+				isCodeStyleAspect);
 		AjAttribute.Aspect deferredAspectAttribute = processAttributes(l, pointcuts, true);
 
 		if (pointcuts.size() == 0) {
@@ -434,15 +438,14 @@ public class BcelObjectType extends AbstractReferenceTypeDelegate {
 			} else if (a instanceof AjAttribute.SourceContextAttribute) {
 				if (getResolvedTypeX().getSourceContext() instanceof SourceContextImpl) {
 					AjAttribute.SourceContextAttribute sca = (AjAttribute.SourceContextAttribute) a;
-					((SourceContextImpl) getResolvedTypeX().getSourceContext()).configureFromAttribute(sca.getSourceFileName(), sca
-							.getLineBreaks());
+					((SourceContextImpl) getResolvedTypeX().getSourceContext()).configureFromAttribute(sca.getSourceFileName(),
+							sca.getLineBreaks());
 
 					setSourcefilename(sca.getSourceFileName());
 				}
 			} else if (a instanceof AjAttribute.WeaverVersionInfo) {
-				wvInfo = (AjAttribute.WeaverVersionInfo) a; // Set the weaver
-				// version used to
-				// build this type
+				// Set the weaver version used to build this type
+				wvInfo = (AjAttribute.WeaverVersionInfo) a;
 			} else {
 				throw new BCException("bad attribute " + a);
 			}
@@ -840,7 +843,7 @@ public class BcelObjectType extends AbstractReferenceTypeDelegate {
 
 	public ResolvedType getOuterClass() {
 		if (!isNested()) {
-			throw new IllegalStateException("Can't get the outer class of a non-nested type");
+			throw new IllegalStateException("Can't get the outer class of non-nested type: " + className);
 		}
 
 		// try finding outer class name from InnerClasses attribute assigned to this class
@@ -876,6 +879,19 @@ public class BcelObjectType extends AbstractReferenceTypeDelegate {
 			}
 		}
 
+		for (Attribute attr : javaClass.getAttributes()) { // bug339300
+			ConstantPool cpool = javaClass.getConstantPool();
+			if (attr instanceof EnclosingMethod) {
+				EnclosingMethod enclosingMethodAttribute = (EnclosingMethod) attr;
+				if (enclosingMethodAttribute.getEnclosingClassIndex() != 0) {
+					ConstantClass outerClassInfo = enclosingMethodAttribute.getEnclosingClass();
+					String outerClassName = cpool.getConstantUtf8(outerClassInfo.getNameIndex()).getValue().replace('/', '.');
+					UnresolvedType outer = UnresolvedType.forName(outerClassName);
+					return outer.resolve(getResolvedTypeX().getWorld());
+				}
+			}
+		}
+
 		// try finding outer class name by assuming standard class name mangling convention of javac for this class
 		int lastDollar = className.lastIndexOf('$');
 		String superClassName = className.substring(0, lastDollar);
@@ -985,4 +1001,8 @@ public class BcelObjectType extends AbstractReferenceTypeDelegate {
 		superTypeReference.clear();
 		superInterfaceReferences.clear();
 	}
+
+	public boolean isWeavable() {
+		return true;
+	}
 }
