@@ -1,5 +1,5 @@
 /* *******************************************************************
- * Copyright (c) 2002 Contributors
+ * Copyright (c) 2002-2010 Contributors
  * All rights reserved. 
  * This program and the accompanying materials are made available 
  * under the terms of the Eclipse Public License v1.0 
@@ -9,6 +9,7 @@
  * Contributors: 
  *     PARC                 initial implementation 
  *     Andy Clement  6Jul05 generics - signature attribute
+ *     Abraham Nevado
  * ******************************************************************/
 
 package org.aspectj.weaver.bcel;
@@ -28,6 +29,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.SortedMap;
 import java.util.TreeMap;
+import java.util.Vector;
 
 import org.aspectj.apache.bcel.Constants;
 import org.aspectj.apache.bcel.classfile.Attribute;
@@ -51,6 +53,8 @@ import org.aspectj.bridge.IMessage;
 import org.aspectj.bridge.ISourceLocation;
 import org.aspectj.bridge.SourceLocation;
 import org.aspectj.weaver.AjAttribute;
+import org.aspectj.weaver.AjAttribute.WeaverState;
+import org.aspectj.weaver.AjAttribute.WeaverVersionInfo;
 import org.aspectj.weaver.BCException;
 import org.aspectj.weaver.Member;
 import org.aspectj.weaver.NameMangler;
@@ -60,11 +64,10 @@ import org.aspectj.weaver.Shadow;
 import org.aspectj.weaver.SignatureUtils;
 import org.aspectj.weaver.TypeVariable;
 import org.aspectj.weaver.UnresolvedType;
+import org.aspectj.weaver.UnresolvedType.TypeKind;
 import org.aspectj.weaver.WeaverMessages;
 import org.aspectj.weaver.WeaverStateInfo;
 import org.aspectj.weaver.World;
-import org.aspectj.weaver.AjAttribute.WeaverVersionInfo;
-import org.aspectj.weaver.UnresolvedType.TypeKind;
 import org.aspectj.weaver.bcel.asm.AsmDetector;
 import org.aspectj.weaver.bcel.asm.StackMapAdder;
 
@@ -91,7 +94,7 @@ public final class LazyClassGen {
 	private final World world;
 	private final String packageName = null;
 
-	private final List fields = new ArrayList();
+	private final List<BcelField> fields = new ArrayList<BcelField>();
 	private final List<LazyMethodGen> methodGens = new ArrayList<LazyMethodGen>();
 	private final List<LazyClassGen> classGens = new ArrayList<LazyClassGen>();
 	private final List<AnnotationGen> annotations = new ArrayList<AnnotationGen>();
@@ -265,7 +268,7 @@ public final class LazyClassGen {
 			// for (int i = 0; i < fields.length; i++) {
 			// ResolvedMember field = fields[i];
 			// if (field.getName().equals("serialVersionUID")
-			// && field.isStatic() && field.getType().equals(ResolvedType.LONG))
+			// && field.isStatic() && field.getType().equals(UnresolvedType.LONG))
 			// {
 			// hasSerialVersionUIDField = true;
 			// }
@@ -293,8 +296,8 @@ public final class LazyClassGen {
 				serialVersionUIDRequiresInitialization = true;
 				// warn about what we've done?
 				if (world.getLint().calculatingSerialVersionUID.isEnabled()) {
-					world.getLint().calculatingSerialVersionUID.signal(new String[] { getClassName(),
-							Long.toString(calculatedSerialVersionUID) + "L" }, null, null);
+					world.getLint().calculatingSerialVersionUID.signal(
+							new String[] { getClassName(), Long.toString(calculatedSerialVersionUID) + "L" }, null, null);
 				}
 			}
 		}
@@ -311,7 +314,7 @@ public final class LazyClassGen {
 
 		ResolvedMember[] fields = myType.getDeclaredFields();
 		for (int i = 0; i < fields.length; i++) {
-			this.fields.add(fields[i]);
+			this.fields.add((BcelField) fields[i]);
 		}
 	}
 
@@ -321,7 +324,7 @@ public final class LazyClassGen {
 		for (int i = 0; i < fields.length; i++) {
 			ResolvedMember field = fields[i];
 			if (field.getName().equals("serialVersionUID") && Modifier.isStatic(field.getModifiers())
-					&& field.getType().equals(ResolvedType.LONG)) {
+					&& field.getType().equals(UnresolvedType.LONG)) {
 				return true;
 			}
 		}
@@ -457,8 +460,11 @@ public final class LazyClassGen {
 			myGen.addAttribute(Utility.bcelAttribute(new AjAttribute.WeaverVersionInfo(), getConstantPool()));
 		}
 
-		if (myType != null && myType.getWeaverState() != null) {
-			myGen.addAttribute(Utility.bcelAttribute(new AjAttribute.WeaverState(myType.getWeaverState()), getConstantPool()));
+		// 352389: don't add another one (there will already be one there and this new one won't deserialize correctly)
+		if (!world.isOverWeaving() || !myGen.hasAttribute(WeaverState.AttributeName)) {
+			if (myType != null && myType.getWeaverState() != null) {
+				myGen.addAttribute(Utility.bcelAttribute(new AjAttribute.WeaverState(myType.getWeaverState()), getConstantPool()));
+			}
 		}
 
 		// FIXME ATAJ needed only for slow Aspects.aspectOf() - keep or remove
@@ -481,6 +487,7 @@ public final class LazyClassGen {
 
 		int len = methodGens.size();
 		myGen.setMethods(Method.NoMethods);
+
 		for (LazyMethodGen gen : methodGens) {
 			// we skip empty clinits
 			if (isEmptyClinit(gen)) {
@@ -492,7 +499,7 @@ public final class LazyClassGen {
 		len = fields.size();
 		myGen.setFields(Field.NoFields);
 		for (int i = 0; i < len; i++) {
-			BcelField gen = (BcelField) fields.get(i);
+			BcelField gen = fields.get(i);
 			myGen.addField(gen.getField(cp));
 		}
 
@@ -651,8 +658,8 @@ public final class LazyClassGen {
 		// create an empty myGen so that we can give back a return value that
 		// doesn't upset the
 		// rest of the process.
-		myGen = new ClassGen(myGen.getClassName(), myGen.getSuperclassName(), myGen.getFileName(), myGen.getModifiers(), myGen
-				.getInterfaceNames());
+		myGen = new ClassGen(myGen.getClassName(), myGen.getSuperclassName(), myGen.getFileName(), myGen.getModifiers(),
+				myGen.getInterfaceNames());
 		// raise an error against this compilation unit.
 		getWorld().showMessage(IMessage.ERROR, WeaverMessages.format(WeaverMessages.CLASS_TOO_BIG, this.getClassName()),
 				new SourceLocation(new File(myGen.getFileName()), 0), null);
@@ -671,12 +678,12 @@ public final class LazyClassGen {
 		writeBack(world);
 		byte[] wovenClassFileData = myGen.getJavaClass().getBytes();
 		// if is java 6 class file
-		if (myGen.getMajor() >= Constants.MAJOR_1_6 && world.shouldGenerateStackMaps() && AsmDetector.isAsmAround) {
+		if (((myGen.getMajor() == Constants.MAJOR_1_6 && world.shouldGenerateStackMaps()) || myGen.getMajor() > Constants.MAJOR_1_6)
+				&& AsmDetector.isAsmAround) {
 			wovenClassFileData = StackMapAdder.addStackMaps(world, wovenClassFileData);
 		}
 
-		WeaverStateInfo wsi = myType.getWeaverState();// getOrCreateWeaverStateInfo
-		// ();
+		WeaverStateInfo wsi = myType.getWeaverState();// getOrCreateWeaverStateInfo();
 		if (wsi != null && wsi.isReweavable()) { // && !reweavableDataInserted
 			// reweavableDataInserted = true;
 			return wsi.replaceKeyWithDiff(wovenClassFileData);
@@ -741,9 +748,9 @@ public final class LazyClassGen {
 		return ret;
 	}
 
-	public List getChildClasses(BcelWorld world) {
+	public List<UnwovenClassFile.ChildClass> getChildClasses(BcelWorld world) {
 		if (classGens.isEmpty()) {
-			return Collections.EMPTY_LIST;
+			return Collections.emptyList();
 		}
 		List<UnwovenClassFile.ChildClass> ret = new ArrayList<UnwovenClassFile.ChildClass>();
 		for (LazyClassGen clazz : classGens) {
@@ -784,9 +791,9 @@ public final class LazyClassGen {
 	}
 
 	public void print(PrintStream out) {
-		List classGens = getClassGens();
-		for (Iterator iter = classGens.iterator(); iter.hasNext();) {
-			LazyClassGen element = (LazyClassGen) iter.next();
+		List<LazyClassGen> classGens = getClassGens();
+		for (Iterator<LazyClassGen> iter = classGens.iterator(); iter.hasNext();) {
+			LazyClassGen element = iter.next();
 			element.printOne(out);
 			if (iter.hasNext()) {
 				out.println();
@@ -821,9 +828,9 @@ public final class LazyClassGen {
 			out.print("  ");
 			out.println(fields[i]);
 		}
-		List methodGens = getMethodGens();
-		for (Iterator iter = methodGens.iterator(); iter.hasNext();) {
-			LazyMethodGen gen = (LazyMethodGen) iter.next();
+		List<LazyMethodGen> methodGens = getMethodGens();
+		for (Iterator<LazyMethodGen> iter = methodGens.iterator(); iter.hasNext();) {
+			LazyMethodGen gen = iter.next();
 			// we skip empty clinits
 			if (isEmptyClinit(gen)) {
 				continue;
@@ -934,6 +941,24 @@ public final class LazyClassGen {
 		return ajcPreClinit;
 	}
 
+	/**
+	 * factory method for building multiple extended clinit methods. Constructs a new clinit method that invokes the previous one
+	 * and then returns it. The index is used as a name suffix.
+	 * 
+	 * @param previousPreClinit
+	 * @param i
+	 */
+	public LazyMethodGen createExtendedAjcPreClinit(LazyMethodGen previousPreClinit, int i) {
+		LazyMethodGen ajcPreClinit = new LazyMethodGen(Modifier.PRIVATE | Modifier.STATIC, Type.VOID,
+				NameMangler.AJC_PRE_CLINIT_NAME + i, Type.NO_ARGS, NO_STRINGS, this);
+		ajcPreClinit.getBody().insert(InstructionConstants.RETURN);
+		methodGens.add(ajcPreClinit);
+		previousPreClinit.getBody().insert(Utility.createInvoke(fact, ajcPreClinit));
+		return ajcPreClinit;
+	}
+
+	//
+
 	// reflective thisJoinPoint support
 	private Map<BcelShadow, Field> tjpFields = new HashMap<BcelShadow, Field>();
 	Map<CacheKey, Field> annotationCachingFieldCache = new HashMap<CacheKey, Field>();
@@ -1023,10 +1048,11 @@ public final class LazyClassGen {
 	 * @param shadow the shadow at which the @annotation result is being cached
 	 * @return a field
 	 */
-	public Field getAnnotationCachingField(BcelShadow shadow, ResolvedType toType) {
+	public Field getAnnotationCachingField(BcelShadow shadow, ResolvedType toType, boolean isWithin) {
 		// Multiple annotation types at a shadow. A different field would be required for each
-		CacheKey cacheKey = new CacheKey(shadow, toType);
+		CacheKey cacheKey = new CacheKey(shadow, toType, isWithin);
 		Field field = annotationCachingFieldCache.get(cacheKey);
+		// System.out.println(field + " for shadow " + shadow);
 		if (field == null) {
 			// private static Annotation ajc$anno$<nnn>
 			StringBuilder sb = new StringBuilder();
@@ -1041,17 +1067,20 @@ public final class LazyClassGen {
 	}
 
 	static class CacheKey {
-		private BcelShadow shadow;
+		private Object key;
 		private ResolvedType annotationType;
 
-		CacheKey(BcelShadow shadow, ResolvedType annotationType) {
-			this.shadow = shadow;
+		// If the annotation is being accessed via @annotation on a shadow then we can use the shadows toString() (so two shadows
+		// the same share a variable), but if it is @withincode() or @within() we can't share them (as the shadows may look the same
+		// but be occurring 'within' different things). In the within cases we continue to use the shadow itself as the key.
+		CacheKey(BcelShadow shadow, ResolvedType annotationType, boolean isWithin) {
+			this.key = isWithin ? shadow : shadow.toString();
 			this.annotationType = annotationType;
 		}
 
 		@Override
 		public int hashCode() {
-			return shadow.hashCode() * 37 + annotationType.hashCode();
+			return key.hashCode() * 37 + annotationType.hashCode();
 		}
 
 		@Override
@@ -1060,7 +1089,7 @@ public final class LazyClassGen {
 				return false;
 			}
 			CacheKey oCacheKey = (CacheKey) other;
-			return shadow.equals(oCacheKey.shadow) && annotationType.equals(oCacheKey.annotationType);
+			return key.equals(oCacheKey.key) && annotationType.equals(oCacheKey.annotationType);
 		}
 	}
 
@@ -1088,27 +1117,45 @@ public final class LazyClassGen {
 		if (tjpFields.size() == 0 && !serialVersionUIDRequiresInitialization) {
 			return;
 		}
-		InstructionList il = null;
+		InstructionList[] il = null;
 
 		if (tjpFields.size() > 0) {
 			il = initializeAllTjps();
 		}
 
 		if (serialVersionUIDRequiresInitialization) {
+			InstructionList[] ilSVUID = new InstructionList[1];
+			ilSVUID[0] = new InstructionList();
+			ilSVUID[0].append(InstructionFactory.PUSH(getConstantPool(), calculatedSerialVersionUID));
+			ilSVUID[0].append(getFactory().createFieldAccess(getClassName(), "serialVersionUID", BasicType.LONG,
+					Constants.PUTSTATIC));
 			if (il == null) {
-				il = new InstructionList();
+				il = ilSVUID;
+			} else {
+				InstructionList[] newIl = new InstructionList[il.length + ilSVUID.length];
+				System.arraycopy(il, 0, newIl, 0, il.length);
+				System.arraycopy(ilSVUID, 0, newIl, il.length, ilSVUID.length);
+				il = newIl;
 			}
-			il.append(InstructionFactory.PUSH(getConstantPool(), calculatedSerialVersionUID));
-			il.append(getFactory().createFieldAccess(getClassName(), "serialVersionUID", BasicType.LONG, Constants.PUTSTATIC));
 		}
+
+		LazyMethodGen prevMethod;
+		LazyMethodGen nextMethod = null;
 		if (this.isInterface()) { // Cannot sneak stuff into another static method in an interface
-			getStaticInitializer().getBody().insert(il);
+			prevMethod = getStaticInitializer();
 		} else {
-			getAjcPreClinit().getBody().insert(il);
+			prevMethod = getAjcPreClinit();
+		}
+		for (int counter = 1; counter <= il.length; counter++) {
+			if (il.length > counter) {
+				nextMethod = createExtendedAjcPreClinit(prevMethod, counter);
+			}
+			prevMethod.getBody().insert(il[counter - 1]);
+			prevMethod = nextMethod;
 		}
 	}
 
-	private InstructionList initializeAllTjps() {
+	private InstructionList initInstructionList() {
 		InstructionList list = new InstructionList();
 		InstructionFactory fact = getFactory();
 
@@ -1120,16 +1167,23 @@ public final class LazyClassGen {
 
 		// load the current Class object
 		// XXX check that this works correctly for inners/anonymous
-		list.append(InstructionFactory.PUSH(getConstantPool(), getClassName()));
+		list.append(fact.PUSHCLASS(cp, myGen.getClassName()));
 		// XXX do we need to worry about the fact the theorectically this could
 		// throw
 		// a ClassNotFoundException
-		list.append(fact.createInvoke("java.lang.Class", "forName", classType, new Type[] { Type.STRING }, Constants.INVOKESTATIC));
 
 		list.append(fact.createInvoke(factoryType.getClassName(), "<init>", Type.VOID, new Type[] { Type.STRING, classType },
 				Constants.INVOKESPECIAL));
 
 		list.append(InstructionFactory.createStore(factoryType, 0));
+		return list;
+	}
+
+	private InstructionList[] initializeAllTjps() {
+		Vector<InstructionList> lists = new Vector<InstructionList>();
+
+		InstructionList list = initInstructionList();
+		lists.add(list);
 
 		List<Map.Entry<BcelShadow, Field>> entries = new ArrayList<Map.Entry<BcelShadow, Field>>(tjpFields.entrySet());
 		Collections.sort(entries, new Comparator<Map.Entry<BcelShadow, Field>>() {
@@ -1138,18 +1192,28 @@ public final class LazyClassGen {
 			}
 		});
 
+		long estimatedSize = 0;
 		for (Iterator<Map.Entry<BcelShadow, Field>> i = entries.iterator(); i.hasNext();) {
 			Map.Entry<BcelShadow, Field> entry = i.next();
+			if (estimatedSize > Constants.MAX_CODE_SIZE) {
+				estimatedSize = 0;
+				list = initInstructionList();
+				lists.add(list);
+			}
+			estimatedSize += entry.getValue().getSignature().getBytes().length;
 			initializeTjp(fact, list, entry.getValue(), entry.getKey());
 		}
-
-		return list;
+		InstructionList listArrayModel[] = new InstructionList[1];
+		return lists.toArray(listArrayModel);
 	}
 
 	private void initializeTjp(InstructionFactory fact, InstructionList list, Field field, BcelShadow shadow) {
+		boolean fastSJP = false;
+		// avoid fast SJP if it is for an enclosing joinpoint
+		boolean isFastSJPAvailable = shadow.getWorld().isTargettingRuntime1_6_10()
+				&& !enclosingStaticTjpType.equals(field.getType());
+
 		Member sig = shadow.getSignature();
-		// ResolvedMember mem =
-		// shadow.getSignature().resolve(shadow.getWorld());
 
 		// load the factory
 		list.append(InstructionFactory.createLoad(factoryType, 0));
@@ -1158,135 +1222,110 @@ public final class LazyClassGen {
 		list.append(InstructionFactory.PUSH(getConstantPool(), shadow.getKind().getName()));
 
 		// create the signature
-		list.append(InstructionFactory.createLoad(factoryType, 0));
+		if (world.isTargettingAspectJRuntime12() || !isFastSJPAvailable || !sig.getKind().equals(Member.METHOD)) {
+			list.append(InstructionFactory.createLoad(factoryType, 0));
+		}
 
 		String signatureMakerName = SignatureUtils.getSignatureMakerName(sig);
 		ObjectType signatureType = new ObjectType(SignatureUtils.getSignatureType(sig));
-
+		UnresolvedType[] exceptionTypes = null;
 		if (world.isTargettingAspectJRuntime12()) { // TAG:SUPPORTING12: We
 			// didn't have optimized
 			// factory methods in 1.2
-			list.append(InstructionFactory.PUSH(getConstantPool(), SignatureUtils.getSignatureString(sig, shadow.getWorld())));
+			list.append(InstructionFactory.PUSH(cp, SignatureUtils.getSignatureString(sig, shadow.getWorld())));
 			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY1,
 					Constants.INVOKEVIRTUAL));
 		} else if (sig.getKind().equals(Member.METHOD)) {
 			BcelWorld w = shadow.getWorld();
 
 			// For methods, push the parts of the signature on.
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getModifiers(w))));
-			list.append(InstructionFactory.PUSH(getConstantPool(), sig.getName()));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getDeclaringType())));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterTypes())));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterNames(w))));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getExceptions(w))));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getReturnType())));
-			// And generate a call to the variant of makeMethodSig() that takes
-			// 7 strings
-			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY7,
-					Constants.INVOKEVIRTUAL));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getModifiers(w))));
+			list.append(InstructionFactory.PUSH(cp, sig.getName()));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getDeclaringType())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterTypes())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterNames(w))));
+			exceptionTypes = sig.getExceptions(w);
+			if (isFastSJPAvailable && exceptionTypes.length == 0) {
+				fastSJP = true;
+			} else {
+				list.append(InstructionFactory.PUSH(cp, makeString(exceptionTypes)));
+			}
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getReturnType())));
+			// And generate a call to the variant of makeMethodSig() that takes the strings
+			if (isFastSJPAvailable) {
+				fastSJP = true;
+			} else {
+				list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY7,
+						Constants.INVOKEVIRTUAL));
+			}
+
 		} else if (sig.getKind().equals(Member.MONITORENTER)) {
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getDeclaringType())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getDeclaringType())));
 			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY1,
 					Constants.INVOKEVIRTUAL));
 		} else if (sig.getKind().equals(Member.MONITOREXIT)) {
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getDeclaringType())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getDeclaringType())));
 			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY1,
 					Constants.INVOKEVIRTUAL));
 		} else if (sig.getKind().equals(Member.HANDLER)) {
 			BcelWorld w = shadow.getWorld();
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getDeclaringType())));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterTypes())));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterNames(w))));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getDeclaringType())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterTypes())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterNames(w))));
 			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY3,
 					Constants.INVOKEVIRTUAL));
 		} else if (sig.getKind().equals(Member.CONSTRUCTOR)) {
 			BcelWorld w = shadow.getWorld();
 			if (w.isJoinpointArrayConstructionEnabled() && sig.getDeclaringType().isArray()) {
 				// its the magical new jp
-				list.append(InstructionFactory.PUSH(getConstantPool(), makeString(Modifier.PUBLIC)));
-				list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getDeclaringType())));
-				list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterTypes())));
-				list.append(InstructionFactory.PUSH(getConstantPool(), ""));// makeString
-				// (
-				// ""
-				// )
-				// )
-				// )
-				// ;
-				// /
-				// /
-				// sig
-				// .
-				// getParameterNames
-				// (
-				// w
-				// )
-				// )
-				// )
-				// )
-				// ;
-				list.append(InstructionFactory.PUSH(getConstantPool(), ""));// makeString
-				// (
-				// ""
-				// )
-				// )
-				// )
-				// ;
-				// /
-				// /
-				// sig
-				// .
-				// getExceptions
-				// (
-				// w
-				// )
-				// )
-				// )
-				// )
-				// ;
+				list.append(InstructionFactory.PUSH(cp, makeString(Modifier.PUBLIC)));
+				list.append(InstructionFactory.PUSH(cp, makeString(sig.getDeclaringType())));
+				list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterTypes())));
+				list.append(InstructionFactory.PUSH(cp, "")); // sig.getParameterNames?
+				list.append(InstructionFactory.PUSH(cp, ""));// sig.getExceptions?
 				list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY5,
 						Constants.INVOKEVIRTUAL));
 			} else {
-				list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getModifiers(w))));
-				list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getDeclaringType())));
-				list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterTypes())));
-				list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterNames(w))));
-				list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getExceptions(w))));
+				list.append(InstructionFactory.PUSH(cp, makeString(sig.getModifiers(w))));
+				list.append(InstructionFactory.PUSH(cp, makeString(sig.getDeclaringType())));
+				list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterTypes())));
+				list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterNames(w))));
+				list.append(InstructionFactory.PUSH(cp, makeString(sig.getExceptions(w))));
 				list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY5,
 						Constants.INVOKEVIRTUAL));
 			}
 		} else if (sig.getKind().equals(Member.FIELD)) {
 			BcelWorld w = shadow.getWorld();
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getModifiers(w))));
-			list.append(InstructionFactory.PUSH(getConstantPool(), sig.getName()));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getModifiers(w))));
+			list.append(InstructionFactory.PUSH(cp, sig.getName()));
 			// see pr227401
 			UnresolvedType dType = sig.getDeclaringType();
 			if (dType.getTypekind() == TypeKind.PARAMETERIZED || dType.getTypekind() == TypeKind.GENERIC) {
 				dType = sig.getDeclaringType().resolve(world).getGenericType();
 			}
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(dType)));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getReturnType())));
+			list.append(InstructionFactory.PUSH(cp, makeString(dType)));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getReturnType())));
 			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY4,
 					Constants.INVOKEVIRTUAL));
 		} else if (sig.getKind().equals(Member.ADVICE)) {
 			BcelWorld w = shadow.getWorld();
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getModifiers(w))));
-			list.append(InstructionFactory.PUSH(getConstantPool(), sig.getName()));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getDeclaringType())));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterTypes())));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getParameterNames(w))));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getExceptions(w))));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString((sig.getReturnType()))));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getModifiers(w))));
+			list.append(InstructionFactory.PUSH(cp, sig.getName()));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getDeclaringType())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterTypes())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getParameterNames(w))));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getExceptions(w))));
+			list.append(InstructionFactory.PUSH(cp, makeString((sig.getReturnType()))));
 			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, new Type[] { Type.STRING,
 					Type.STRING, Type.STRING, Type.STRING, Type.STRING, Type.STRING, Type.STRING }, Constants.INVOKEVIRTUAL));
 		} else if (sig.getKind().equals(Member.STATIC_INITIALIZATION)) {
 			BcelWorld w = shadow.getWorld();
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getModifiers(w))));
-			list.append(InstructionFactory.PUSH(getConstantPool(), makeString(sig.getDeclaringType())));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getModifiers(w))));
+			list.append(InstructionFactory.PUSH(cp, makeString(sig.getDeclaringType())));
 			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY2,
 					Constants.INVOKEVIRTUAL));
 		} else {
-			list.append(InstructionFactory.PUSH(getConstantPool(), SignatureUtils.getSignatureString(sig, shadow.getWorld())));
+			list.append(InstructionFactory.PUSH(cp, SignatureUtils.getSignatureString(sig, shadow.getWorld())));
 			list.append(fact.createInvoke(factoryType.getClassName(), signatureMakerName, signatureType, Type.STRINGARRAY1,
 					Constants.INVOKEVIRTUAL));
 		}
@@ -1296,9 +1335,8 @@ public final class LazyClassGen {
 
 		final String factoryMethod;
 
-		if (world.isTargettingAspectJRuntime12()) { // TAG:SUPPORTING12: We
-			// didn't have makeESJP() in
-			// 1.2
+		// TAG:SUPPORTING12: We didn't have makeESJP() in 1.2
+		if (world.isTargettingAspectJRuntime12()) {
 			list.append(fact.createInvoke(factoryType.getClassName(), "makeSJP", staticTjpType, new Type[] { Type.STRING, sigType,
 					Type.INT }, Constants.INVOKEVIRTUAL));
 
@@ -1313,13 +1351,30 @@ public final class LazyClassGen {
 			} else {
 				throw new Error("should not happen");
 			}
-			list.append(fact.createInvoke(factoryType.getClassName(), factoryMethod, field.getType(), new Type[] { Type.STRING,
-					sigType, Type.INT }, Constants.INVOKEVIRTUAL));
+
+			if (fastSJP) {
+				if (exceptionTypes != null && exceptionTypes.length != 0) {
+					list.append(fact.createInvoke(factoryType.getClassName(), factoryMethod, field.getType(), ARRAY_8STRING_INT,
+							Constants.INVOKEVIRTUAL));
+				} else {
+					list.append(fact.createInvoke(factoryType.getClassName(), factoryMethod, field.getType(), ARRAY_7STRING_INT,
+							Constants.INVOKEVIRTUAL));
+				}
+			} else {
+				list.append(fact.createInvoke(factoryType.getClassName(), factoryMethod, field.getType(), new Type[] { Type.STRING,
+						sigType, Type.INT }, Constants.INVOKEVIRTUAL));
+			}
+
 			// put it in the field
 			list.append(fact.createFieldAccess(getClassName(), field.getName(), field.getType(), Constants.PUTSTATIC));
 		}
 	}
 
+	private static final Type[] ARRAY_7STRING_INT = new Type[] { Type.STRING, Type.STRING, Type.STRING, Type.STRING, Type.STRING,
+			Type.STRING, Type.STRING, Type.INT };
+	private static final Type[] ARRAY_8STRING_INT = new Type[] { Type.STRING, Type.STRING, Type.STRING, Type.STRING, Type.STRING,
+			Type.STRING, Type.STRING, Type.STRING, Type.INT };
+
 	protected String makeString(int i) {
 		return Integer.toString(i, 16); // ??? expensive
 	}
@@ -1343,10 +1398,12 @@ public final class LazyClassGen {
 		if (types == null) {
 			return "";
 		}
-		StringBuffer buf = new StringBuffer();
+		StringBuilder buf = new StringBuilder();
 		for (int i = 0, len = types.length; i < len; i++) {
+			if (i > 0) {
+				buf.append(':');
+			}
 			buf.append(makeString(types[i]));
-			buf.append(':');
 		}
 		return buf.toString();
 	}
@@ -1355,10 +1412,12 @@ public final class LazyClassGen {
 		if (names == null) {
 			return "";
 		}
-		StringBuffer buf = new StringBuffer();
+		StringBuilder buf = new StringBuilder();
 		for (int i = 0, len = names.length; i < len; i++) {
+			if (i > 0) {
+				buf.append(':');
+			}
 			buf.append(names[i]);
-			buf.append(':');
 		}
 		return buf.toString();
 	}
@@ -1420,9 +1479,9 @@ public final class LazyClassGen {
 		}
 	}
 
-	private boolean hasSyntheticAttribute(List attributes) {
+	private boolean hasSyntheticAttribute(List<Attribute> attributes) {
 		for (int i = 0; i < attributes.size(); i++) {
-			if (((Attribute) attributes.get(i)).getName().equals("Synthetic")) {
+			if ((attributes.get(i)).getName().equals("Synthetic")) {
 				return true;
 			}
 		}
@@ -1457,8 +1516,7 @@ public final class LazyClassGen {
 	}
 
 	public LazyMethodGen getLazyMethodGen(String name, String signature, boolean allowMissing) {
-		for (Iterator i = methodGens.iterator(); i.hasNext();) {
-			LazyMethodGen gen = (LazyMethodGen) i.next();
+		for (LazyMethodGen gen : methodGens) {
 			if (gen.getName().equals(name) && gen.getSignature().equals(signature)) {
 				return gen;
 			}
@@ -1540,9 +1598,8 @@ public final class LazyClassGen {
 	 */
 	public String allocateField(String prefix) {
 		int highestAllocated = -1;
-		List/* BcelField */fs = getFieldGens();
-		for (int i = 0; i < fs.size(); i++) {
-			BcelField field = (BcelField) fs.get(i);
+		List<BcelField> fs = getFieldGens();
+		for (BcelField field : fs) {
 			if (field.getName().startsWith(prefix)) {
 				try {
 					int num = Integer.parseInt(field.getName().substring(prefix.length()));
