@@ -15,19 +15,25 @@ package org.aspectj.weaver;
 import junit.framework.Test;
 import junit.framework.TestSuite;
 
-import org.aspectj.weaver.reflect.ReflectionWorldTest;
+import org.aspectj.weaver.reflect.ReflectionWorldReferenceTypeTest;
+import org.aspectj.weaver.reflect.ReflectionWorldSpecificTest;
 import org.aspectj.weaver.tools.PointcutExpressionTest;
 
 public class AllWeaver5Tests {
 
 	public static Test suite() {
 		TestSuite suite = new TestSuite(AllWeaver5Tests.class.getName());
-		//$JUnit-BEGIN$
-        suite.addTest(AllTracing5Tests.suite());
-        suite.addTest(BcweaverModuleTests15.suite());
+		// $JUnit-BEGIN$
+		suite.addTest(AllTracing5Tests.suite());
+		suite.addTest(BcweaverModuleTests15.suite());
+		suite.addTestSuite(ReflectionWorldReferenceTypeTest.class);
 		suite.addTestSuite(PointcutExpressionTest.class);
-		suite.addTestSuite(ReflectionWorldTest.class);
-		//$JUnit-END$
+		suite.addTestSuite(JoinPointSignatureIteratorTests.class);
+		// gives incompatible class version error...
+		// suite.addTestSuite(ReflectionWorldAdvancedPointcutExpressionTests.class
+		// );
+		suite.addTestSuite(ReflectionWorldSpecificTest.class);
+		// $JUnit-END$
 		return suite;
 	}
 
